@@ -484,6 +484,14 @@ rdns_process_retransmit (int fd, void *arg)
 			req->async_event);
 	req->async_event = NULL;
 
+	if (req->state == RDNS_REQUEST_FAKE) {
+		/* Reply is ready */
+		req->func (req->reply, req->arg);
+		REF_RELEASE (req);
+
+		return;
+	}
+
 	r = rdns_send_request (req, fd, false);
 
 	if (r == 0) {
@@ -513,6 +521,9 @@ rdns_process_retransmit (int fd, void *arg)
 	}
 }
 
+#define align_ptr(p, a)                                                   \
+    (guint8 *) (((uintptr_t) (p) + ((uintptr_t) a - 1)) & ~((uintptr_t) a - 1))
+
 struct rdns_request*
 rdns_make_request_full (
 		struct rdns_resolver *resolver,
@@ -532,13 +543,24 @@ rdns_make_request_full (
 	size_t olen;
 	const char *cur_name, *last_name = NULL;
 	struct rdns_compression_entry *comp = NULL;
+	struct rdns_fake_reply *fake_rep = NULL;
+	char fake_buf[MAX_FAKE_NAME + sizeof (struct rdns_fake_reply_idx) + 16];
+	struct rdns_fake_reply_idx *idx;
 
 	if (resolver == NULL || !resolver->initialized) {
+		if (resolver == NULL) {
+			return NULL;
+		}
+
+		rdns_err ("resolver is uninitialized");
+
 		return NULL;
 	}
 
 	req = malloc (sizeof (struct rdns_request));
 	if (req == NULL) {
+		rdns_err ("failed to allocate memory for request: %s",
+				strerror (errno));
 		return NULL;
 	}
 
@@ -555,6 +577,9 @@ rdns_make_request_full (
 
 	if (req->requested_names == NULL) {
 		free (req);
+		rdns_err ("failed to allocate memory for request data: %s",
+				strerror (errno));
+
 		return NULL;
 	}
 
@@ -569,65 +594,97 @@ rdns_make_request_full (
 	for (i = 0; i < queries * 2; i += 2) {
 		cur = i / 2;
 		cur_name = va_arg (args, const char *);
+		type = va_arg (args, int);
+
 		if (cur_name != NULL) {
-			last_name = cur_name;
 			clen = strlen (cur_name);
+
 			if (clen == 0) {
-				rdns_info ("got empty name to resolve");
+				rdns_warn ("got empty name to resolve");
 				rdns_request_free (req);
 				return NULL;
 			}
+
+			if (last_name == NULL && queries == 1 && clen < MAX_FAKE_NAME) {
+				/* We allocate structure in the static space */
+				idx = (struct rdns_fake_reply_idx *)align_ptr (fake_buf, 16);
+				idx->type = type;
+				idx->len = clen;
+				memcpy (idx->request, cur_name, clen);
+				HASH_FIND (hh, resolver->fake_elts, idx, sizeof (*idx) + clen,
+						fake_rep);
+
+				if (fake_rep) {
+					/* We actually treat it as a short-circuit */
+					req->reply = rdns_make_reply (req, fake_rep->rcode);
+					req->reply->entries = fake_rep->result;
+					req->state = RDNS_REQUEST_FAKE;
+				}
+			}
+
+			last_name = cur_name;
 			tlen += clen;
 		}
 		else if (last_name == NULL) {
-			rdns_info ("got NULL as the first name to resolve");
+			rdns_err ("got NULL as the first name to resolve");
 			rdns_request_free (req);
 			return NULL;
 		}
 
-		if (!rdns_format_dns_name (resolver, last_name, clen,
-				&req->requested_names[cur].name, &olen)) {
-			rdns_request_free (req);
-			return NULL;
-		}
-
-		type = va_arg (args, int);
-		req->requested_names[cur].type = type;
-		req->requested_names[cur].len = olen;
-	}
-	va_end (args);
-
-	rdns_allocate_packet (req, tlen);
-	rdns_make_dns_header (req, queries);
-
-	for (i = 0; i < queries; i ++) {
-		cur_name = req->requested_names[i].name;
-		clen = req->requested_names[i].len;
-		type = req->requested_names[i].type;
-		if (queries > 1) {
-			if (!rdns_add_rr (req, cur_name, clen, type, &comp)) {
-				REF_RELEASE (req);
-				rnds_compression_free (comp);
+		if (req->state != RDNS_REQUEST_FAKE) {
+			if (!rdns_format_dns_name (resolver, last_name, clen,
+					&req->requested_names[cur].name, &olen)) {
+				rdns_err ("cannot format %s", last_name);
+				rdns_request_free (req);
 				return NULL;
 			}
+
+			req->requested_names[cur].len = olen;
 		}
 		else {
-			if (!rdns_add_rr (req, cur_name, clen, type, NULL)) {
-				REF_RELEASE (req);
-				rnds_compression_free (comp);
-				return NULL;
-			}
+			req->requested_names[cur].len = clen;
 		}
+
+		req->requested_names[cur].type = type;
 	}
 
-	rnds_compression_free (comp);
+	va_end (args);
 
-	/* Add EDNS RR */
-	rdns_add_edns0 (req);
+	if (req->state != RDNS_REQUEST_FAKE) {
+		rdns_allocate_packet (req, tlen);
+		rdns_make_dns_header (req, queries);
+
+		for (i = 0; i < queries; i++) {
+			cur_name = req->requested_names[i].name;
+			clen = req->requested_names[i].len;
+			type = req->requested_names[i].type;
+			if (queries > 1) {
+				if (!rdns_add_rr (req, cur_name, clen, type, &comp)) {
+					rdns_err ("cannot add rr", cur_name);
+					REF_RELEASE (req);
+					rnds_compression_free (comp);
+					return NULL;
+				}
+			} else {
+				if (!rdns_add_rr (req, cur_name, clen, type, NULL)) {
+					rdns_err ("cannot add rr", cur_name);
+					REF_RELEASE (req);
+					rnds_compression_free (comp);
+					return NULL;
+				}
+			}
+		}
+
+		rnds_compression_free (comp);
+
+		/* Add EDNS RR */
+		rdns_add_edns0 (req);
+
+		req->retransmits = repeats;
+		req->timeout = timeout;
+		req->state = RDNS_REQUEST_NEW;
+	}
 
-	req->retransmits = repeats;
-	req->timeout = timeout;
-	req->state = RDNS_REQUEST_NEW;
 	req->async = resolver->async;
 
 	if (resolver->ups) {
@@ -656,14 +713,22 @@ rdns_make_request_full (
 
 	/* Select random IO channel */
 	req->io = serv->io_channels[ottery_rand_uint32 () % serv->io_cnt];
-	req->io->uses ++;
 
-	/* Now send request to server */
-	r = rdns_send_request (req, req->io->sock, true);
+	if (req->state == RDNS_REQUEST_FAKE) {
+		req->async_event = resolver->async->add_write (resolver->async->data,
+				req->io->sock, req);
+	}
+	else {
+		req->io->uses++;
 
-	if (r == -1) {
-		REF_RELEASE (req);
-		return NULL;
+		/* Now send request to server */
+		r = rdns_send_request (req, req->io->sock, true);
+
+		if (r == -1) {
+			rdns_info ("cannot send DNS request");
+			REF_RELEASE (req);
+			return NULL;
+		}
 	}
 
 	REF_RETAIN (req->io);
@@ -680,10 +745,12 @@ rdns_resolver_init (struct rdns_resolver *resolver)
 	struct rdns_io_channel *ioc;
 
 	if (!resolver->async_binded) {
+		rdns_err ("no async backend specified");
 		return false;
 	}
 
 	if (resolver->servers == NULL) {
+		rdns_err ("no DNS servers defined");
 		return false;
 	}
 
@@ -693,13 +760,16 @@ rdns_resolver_init (struct rdns_resolver *resolver)
 		for (i = 0; i < serv->io_cnt; i ++) {
 			ioc = calloc (1, sizeof (struct rdns_io_channel));
 			if (ioc == NULL) {
-				rdns_err ("cannot allocate memory for the resolver");
+				rdns_err ("cannot allocate memory for the resolver IO channels");
 				return false;
 			}
+
 			ioc->sock = rdns_make_client_socket (serv->name, serv->port, SOCK_DGRAM);
-			ioc->active = true;
+
 			if (ioc->sock == -1) {
-				rdns_err ("cannot open socket to %s:%d %s", serv->name, serv->port, strerror (errno));
+				ioc->active = false;
+				rdns_err ("cannot open socket to %s:%d %s",
+						serv->name, serv->port, strerror (errno));
 				free (ioc);
 				return false;
 			}
@@ -889,3 +959,49 @@ rdns_resolver_set_dnssec (struct rdns_resolver *resolver, bool enabled)
 		resolver->enable_dnssec = enabled;
 	}
 }
+
+
+void rdns_resolver_set_fake_reply (struct rdns_resolver *resolver,
+								   const char *name,
+								   enum rdns_request_type type,
+								   enum dns_rcode rcode,
+								   struct rdns_reply_entry *reply)
+{
+	struct rdns_fake_reply *fake_rep;
+	struct rdns_fake_reply_idx *srch;
+	unsigned len = strlen (name);
+
+	assert (len < MAX_FAKE_NAME);
+	srch = malloc (sizeof (*srch) + len);
+	srch->len = len;
+	srch->type = type;
+	memcpy (srch->request, name, len);
+
+	HASH_FIND (hh, resolver->fake_elts, srch, len + sizeof (*srch), fake_rep);
+
+	if (fake_rep) {
+		/* Append reply to the existing list */
+		fake_rep->rcode = rcode;
+
+		if (reply) {
+			DL_APPEND (fake_rep->result, reply);
+		}
+	}
+	else {
+		fake_rep = calloc (1, sizeof (*fake_rep) + len);
+
+		if (fake_rep == NULL) {
+			abort ();
+		}
+
+		fake_rep->rcode = rcode;
+
+		memcpy (&fake_rep->key, srch, sizeof (*srch) + len);
+
+		if (reply) {
+			DL_APPEND (fake_rep->result, reply);
+		}
+
+		HASH_ADD (hh, resolver->fake_elts, key, sizeof (*srch) + len, fake_rep);
+	}
+}
\ No newline at end of file
