@@ -532,6 +532,10 @@ ucl_chunk_free (struct ucl_chunk *chunk)
 			}
 		}
 
+		if (chunk->fname) {
+			free (chunk->fname);
+		}
+
 		UCL_FREE (sizeof (*chunk), chunk);
 	}
 }
@@ -1318,7 +1322,10 @@ ucl_include_file_single (const unsigned char *data, size_t len,
 			return false;
 		}
 		st->obj = nest_obj;
-		st->level = parser->stack->level;
+		st->e.params.level = parser->stack->e.params.level;
+		st->e.params.flags = parser->stack->e.params.flags;
+		st->e.params.line = parser->stack->e.params.line;
+		st->chunk = parser->chunks;
 		LL_PREPEND (parser->stack, st);
 		parser->cur_obj = nest_obj;
 	}
