@@ -26,6 +26,8 @@ local E = {}
 local rcvd_cb_id = rspamd_config:register_symbol{
   name = 'CHECK_RECEIVED',
   type = 'callback',
+  score = 0.0,
+  group = 'headers',
   callback = function(task)
     local cnts = {
       [1] = 'ONE',
@@ -59,7 +61,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = 'No received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_ONE',
@@ -67,7 +69,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = 'One received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_TWO',
@@ -75,7 +77,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = 'Two received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_THREE',
@@ -83,7 +85,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '3-5 received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_FIVE',
@@ -91,7 +93,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '5-7 received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_SEVEN',
@@ -99,7 +101,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '7-11 received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_TWELVE',
@@ -107,12 +109,14 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '12+ received',
-  group = 'header',
+  group = 'headers',
 }
 
 local prio_cb_id = rspamd_config:register_symbol {
   name = 'HAS_X_PRIO',
   type = 'callback',
+  score = 0.0,
+  group = 'headers',
   callback = function (task)
      local cnts = {
       [1] = 'ONE',
@@ -141,7 +145,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 0',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_ONE',
@@ -149,7 +153,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 1',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_TWO',
@@ -157,7 +161,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 2',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_THREE',
@@ -165,7 +169,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 3-4',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_FIVE',
@@ -173,18 +177,24 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 5+',
-  group = 'header',
+  group = 'headers',
 }
 
 local function get_raw_header(task, name)
   return ((task:get_header_full(name) or {})[1] or {})['value']
 end
 
-local check_replyto_id = rspamd_config:register_callback_symbol('CHECK_REPLYTO', 1.0,
-  function (task)
+local check_replyto_id = rspamd_config:register_symbol({
+  type = 'callback',
+  name = 'CHECK_REPLYTO',
+  score = 0.0,
+  group = 'headers',
+  callback = function(task)
     local replyto = get_raw_header(task, 'Reply-To')
-    if not replyto then return false end
-    local rt = util.parse_mail_address(replyto)
+    if not replyto then
+      return false
+    end
+    local rt = util.parse_mail_address(replyto, task:get_mempool())
     if not (rt and rt[1] and (string.len(rt[1].addr) > 0)) then
       task:insert_result('REPLYTO_UNPARSEABLE', 1.0)
       return false
@@ -205,7 +215,9 @@ local check_replyto_id = rspamd_config:register_callback_symbol('CHECK_REPLYTO',
     -- See if Reply-To matches From in some way
     local from = task:get_from(2)
     local from_h = get_raw_header(task, 'From')
-    if not (from and from[1]) then return false end
+    if not (from and from[1]) then
+      return false
+    end
     if (from_h and from_h == replyto) then
       -- From and Reply-To are identical
       task:insert_result('REPLYTO_EQ_FROM', 1.0)
@@ -218,7 +230,20 @@ local check_replyto_id = rspamd_config:register_callback_symbol('CHECK_REPLYTO',
           if (util.strequal_caseless(from[1].domain, rt[1].domain)) then
             task:insert_result('REPLYTO_DOM_EQ_FROM_DOM', 1.0)
           else
-            task:insert_result('REPLYTO_DOM_NEQ_FROM_DOM', 1.0)
+            -- See if Reply-To matches the To address
+            local to = task:get_recipients(2)
+            if (to and to[1] and to[1].addr:lower() == rt[1].addr:lower()) then
+              -- Ignore this for mailing-lists and automatic submissions
+              if (not (task:get_header('List-Unsubscribe') or
+                  task:get_header('X-To-Get-Off-This-List') or
+                  task:get_header('X-List') or
+                  task:get_header('Auto-Submitted')))
+              then
+                task:insert_result('REPLYTO_EQ_TO_ADDR', 1.0)
+              end
+            else
+              task:insert_result('REPLYTO_DOM_NEQ_FROM_DOM', 1.0)
+            end
           end
         end
         -- See if the Display Names match
@@ -229,7 +254,7 @@ local check_replyto_id = rspamd_config:register_callback_symbol('CHECK_REPLYTO',
       end
     end
   end
-)
+})
 
 rspamd_config:register_symbol{
   name = 'REPLYTO_UNPARSEABLE',
@@ -237,7 +262,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header could not be parsed',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_REPLYTO',
@@ -245,7 +270,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Has Reply-To header',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_EQ_FROM',
@@ -253,7 +278,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header is identical to From header',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_ADDR_EQ_FROM',
@@ -261,7 +286,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header is identical to SMTP From',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_DOM_EQ_FROM_DOM',
@@ -269,7 +294,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To domain matches the From domain',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_DOM_NEQ_FROM_DOM',
@@ -277,7 +302,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To domain does not match the From domain',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_DN_EQ_FROM_DN',
@@ -285,7 +310,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To display name matches From',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_EMAIL_HAS_TITLE',
@@ -293,13 +318,24 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header has title',
-  group = 'header',
+  group = 'headers',
 }
-rspamd_config:register_dependency(check_replyto_id, 'FROM_NAME_HAS_TITLE')
+rspamd_config:register_symbol{
+  name = 'REPLYTO_EQ_TO_ADDR',
+  score = 5.0,
+  parent = check_replyto_id,
+  type = 'virtual',
+  description = 'Reply-To is the same as the To address',
+  group = 'headers',
+}
+
+rspamd_config:register_dependency('CHECK_REPLYTO', 'CHECK_FROM')
 
 local check_mime_id = rspamd_config:register_symbol{
   name = 'CHECK_MIME',
   type = 'callback',
+  group = 'headers',
+  score = 0.0,
   callback = function(task)
     local parts = task:get_parts()
     if not parts then return false end
@@ -359,7 +395,7 @@ rspamd_config:register_symbol{
   parent = check_mime_id,
   type = 'virtual',
   description = 'MIME-Version header is missing',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'MIME_MA_MISSING_TEXT',
@@ -367,7 +403,7 @@ rspamd_config:register_symbol{
   parent = check_mime_id,
   type = 'virtual',
   description = 'MIME multipart/alternative missing text/plain part',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'MIME_MA_MISSING_HTML',
@@ -375,7 +411,7 @@ rspamd_config:register_symbol{
   parent = check_mime_id,
   type = 'virtual',
   description = 'MIME multipart/alternative missing text/html part',
-  group = 'header',
+  group = 'headers',
 }
 
 -- Used to be called IS_LIST
@@ -398,6 +434,7 @@ rspamd_config.PREVIOUSLY_DELIVERED = {
     end
   end,
   description = 'Message either to a list or was forwarded',
+  group = 'headers',
   score = 0.0
 }
 rspamd_config.BROKEN_HEADERS = {
@@ -405,7 +442,7 @@ rspamd_config.BROKEN_HEADERS = {
     return task:has_flag('broken_headers')
   end,
   score = 10.0,
-  group = 'header',
+  group = 'headers',
   description = 'Headers structure is likely broken'
 }
 
@@ -415,7 +452,7 @@ rspamd_config.BROKEN_CONTENT_TYPE = {
       task:get_parts())
   end,
   score = 1.5,
-  group = 'header',
+  group = 'headers',
   description = 'Message has part with broken content type'
 }
 
@@ -430,7 +467,7 @@ rspamd_config.HEADER_RCONFIRM_MISMATCH = {
 
     local header_cread = nil
     if cread then
-      local headers_cread = util.parse_mail_address(cread)
+      local headers_cread = util.parse_mail_address(cread, task:get_mempool())
       if headers_cread then header_cread = headers_cread[1] end
     end
 
@@ -444,7 +481,7 @@ rspamd_config.HEADER_RCONFIRM_MISMATCH = {
   end,
 
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   description = 'Read confirmation address is different to from address'
 }
 
@@ -459,7 +496,7 @@ rspamd_config.HEADER_FORGED_MDN = {
     end
 
     -- Parse mail addr
-    local headers_mdn = util.parse_mail_address(mdn)
+    local headers_mdn = util.parse_mail_address(mdn, task:get_mempool())
 
     if headers_mdn and not header_rp  then return true end
     if header_rp  and not headers_mdn then return false end
@@ -477,50 +514,55 @@ rspamd_config.HEADER_FORGED_MDN = {
   end,
 
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   description = 'Read confirmation address is different to return path'
 }
 
 local headers_unique = {
-  'Content-Type',
-  'Content-Transfer-Encoding',
+  ['Content-Type'] = 1.0,
+  ['Content-Transfer-Encoding'] = 1.0,
   -- https://tools.ietf.org/html/rfc5322#section-3.6
-  'Date',
-  'From',
-  'Sender',
-  'Reply-To',
-  'To',
-  'Cc',
-  'Bcc',
-  'Message-ID',
-  'In-Reply-To',
-  'References',
-  'Subject'
+  ['Date'] = 0.1,
+  ['From'] = 1.0,
+  ['Sender'] = 1.0,
+  ['Reply-To'] = 1.0,
+  ['To'] = 0.2,
+  ['Cc'] = 0.1,
+  ['Bcc'] = 0.1,
+  ['Message-ID'] = 0.7,
+  ['In-Reply-To'] = 0.7,
+  ['References'] = 0.3,
+  ['Subject'] = 0.7
 }
 
 rspamd_config.MULTIPLE_UNIQUE_HEADERS = {
   callback = function(task)
     local res = 0
+    local max_mult = 0.0
     local res_tbl = {}
 
-    for _,hdr in ipairs(headers_unique) do
-      local h = task:get_header_full(hdr)
+    for hdr,mult in pairs(headers_unique) do
+      local hc = task:get_header_count(hdr)
 
-      if h and #h > 1 then
+      if hc > 1 then
         res = res + 1
         table.insert(res_tbl, hdr)
+        if max_mult < mult then
+          max_mult = mult
+        end
       end
     end
 
     if res > 0 then
-      return true,res,table.concat(res_tbl, ',')
+      return true,max_mult,table.concat(res_tbl, ',')
     end
 
     return false
   end,
 
-  score = 5.0,
-  group = 'header',
+  score = 7.0,
+  group = 'headers',
+  one_shot = true,
   description = 'Repeated unique headers'
 }
 
@@ -533,16 +575,38 @@ rspamd_config.MISSING_FROM = {
     return false
   end,
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   description = 'Missing From: header'
 }
+
+rspamd_config.MULTIPLE_FROM = {
+  callback = function(task)
+    local from = task:get_from('mime')
+    if from and from[1] then
+      if #from > 1 then
+        return true,1.0,table.concat(
+            fun.totable(
+                fun.map(function(a) return a.addr end,
+                    fun.filter(function(a) return a.addr and a.addr ~= '' end,
+                        from))),
+            ',')
+      end
+    end
+    return false
+  end,
+  score = 9.0,
+  group = 'headers',
+  description = 'Multiple addresses in From'
+}
+
 rspamd_config.MV_CASE = {
   callback = function (task)
     local mv = task:get_header('Mime-Version', true)
     if (mv) then return true end
   end,
   description = 'Mime-Version .vs. MIME-Version',
-  score = 0.5
+  score = 0.5,
+  group = 'headers',
 }
 
 rspamd_config.FAKE_REPLY = {
@@ -556,12 +620,15 @@ rspamd_config.FAKE_REPLY = {
     return false
   end,
   description = 'Fake reply',
-  score = 1.0
+  score = 1.0,
+  group = 'headers'
 }
 
 local check_from_id = rspamd_config:register_symbol{
   name = 'CHECK_FROM',
   type = 'callback',
+  score = 0.0,
+  group = 'headers',
   callback = function(task)
     local envfrom = task:get_from(1)
     local from = task:get_from(2)
@@ -593,11 +660,11 @@ local check_from_id = rspamd_config:register_symbol{
     then
       task:insert_result('FROM_EQ_ENVFROM', 1.0)
     elseif (envfrom and envfrom[1] and envfrom[1].addr) then
-      task:insert_result('FROM_NEQ_ENVFROM', 1.0, from and from[1].addr or '', envfrom[1].addr)
+      task:insert_result('FROM_NEQ_ENVFROM', 1.0, ((from or E)[1] or E).addr or '', envfrom[1].addr)
     end
 
     local to = task:get_recipients(2)
-    if not (to and to[1] and #to == 1 and from) then return false end
+    if not (to and to[1] and #to == 1 and from and from[1]) then return false end
     -- Check if FROM == TO
     if (util.strequal_caseless(to[1].addr, from[1].addr)) then
       task:insert_result('TO_EQ_FROM', 1.0)
@@ -612,7 +679,7 @@ local check_from_id = rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NO_DN',
   score = 0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header does not have a display name',
@@ -620,7 +687,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_DN_EQ_ADDR',
   score = 1.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header display name is the same as the address',
@@ -628,7 +695,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_HAS_DN',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header has a display name',
@@ -636,7 +703,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NAME_EXCESS_SPACE',
   score = 1.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header display name contains excess whitespace',
@@ -644,7 +711,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NAME_HAS_TITLE',
   score = 1.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header display name has a title (Mr/Mrs/Dr)',
@@ -652,7 +719,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_EQ_ENVFROM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From address is the same as the envelope',
@@ -660,7 +727,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NEQ_ENVFROM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From address is different to the envelope',
@@ -668,7 +735,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_EQ_FROM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'To address matches the From address',
@@ -676,7 +743,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DOM_EQ_FROM_DOM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'To domain is the same as the From domain',
@@ -685,6 +752,8 @@ rspamd_config:register_symbol{
 local check_to_cc_id = rspamd_config:register_symbol{
   name = 'CHECK_TO_CC',
   type = 'callback',
+  score = 0.0,
+  group = 'headers',
   callback = function(task)
     local rcpts = task:get_recipients(1)
     local to = task:get_recipients(2)
@@ -761,7 +830,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'No recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_ONE',
@@ -769,7 +838,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'One recipient',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_TWO',
@@ -777,7 +846,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Two recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_THREE',
@@ -785,7 +854,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '3-5 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_FIVE',
@@ -793,7 +862,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '5-7 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_SEVEN',
@@ -801,7 +870,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '7-11 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_TWELVE',
@@ -809,7 +878,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '12-50 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_GT_50',
@@ -817,13 +886,13 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '50+ recipients',
-  group = 'header',
+  group = 'headers',
 }
 
 rspamd_config:register_symbol{
   name = 'TO_DN_RECIPIENTS',
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'To header display name is "Recipients"',
@@ -831,7 +900,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_NONE',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'None of the recipients have display names',
@@ -839,7 +908,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_ALL',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'All the recipients have display names',
@@ -847,7 +916,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_SOME',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Some of the recipients have display names',
@@ -855,7 +924,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_EQ_ADDR_ALL',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'All of the recipients have display names that are the same as their address',
@@ -863,7 +932,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_EQ_ADDR_SOME',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Some of the recipients have display names that are the same as their address',
@@ -871,7 +940,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_MATCH_ENVRCPT_ALL',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'All of the recipients match the envelope',
@@ -879,7 +948,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_MATCH_ENVRCPT_SOME',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Some of the recipients match the envelope',
@@ -907,7 +976,7 @@ rspamd_config.CTYPE_MISSING_DISPOSITION = {
   end,
   description = 'Binary content-type not specified as an attachment',
   score = 4.0,
-  group = 'header'
+  group = 'headers'
 }
 
 rspamd_config.CTYPE_MIXED_BOGUS = {
@@ -1009,3 +1078,13 @@ rspamd_config.INVALID_RCPT_8BIT = {
   score = 6.0,
   group = 'headers'
 }
+
+rspamd_config.XM_CASE = {
+  callback = function (task)
+    local xm = task:get_header('X-mailer', true)
+    if (xm) then return true end
+  end,
+  description = 'X-mailer .vs. X-Mailer',
+  score = 0.5,
+  group = 'headers',
+}
