@@ -17,6 +17,7 @@
  * Rspamd fuzzy storage server
  */
 
+#include <src/libserver/fuzzy_wire.h>
 #include "config.h"
 #include "util.h"
 #include "rspamd.h"
@@ -126,7 +127,6 @@ struct rspamd_fuzzy_storage_ctx {
 	struct rspamd_config *cfg;
 	/* END OF COMMON PART */
 	struct fuzzy_global_stat stat;
-	char *hashfile;
 	gdouble expire;
 	gdouble sync_timeout;
 	struct rspamd_radix_map_helper *update_ips;
@@ -154,6 +154,7 @@ struct rspamd_fuzzy_storage_ctx {
 	GHashTable *keys;
 	gboolean encrypted_only;
 	gboolean collection_mode;
+	gboolean read_only;
 	struct rspamd_cryptobox_keypair *collection_keypair;
 	struct rspamd_cryptobox_pubkey *collection_sign_key;
 	gchar *collection_id_file;
@@ -240,6 +241,10 @@ rspamd_fuzzy_check_client (struct fuzzy_session *session, gboolean is_write)
 	}
 
 	if (is_write) {
+		if (session->ctx->read_only) {
+			return FALSE;
+		}
+
 		if (session->ctx->update_ips != NULL) {
 			if (rspamd_match_radix_map_addr (session->ctx->update_ips,
 					session->addr) == NULL) {
@@ -314,6 +319,7 @@ struct rspamd_fuzzy_updates_cbdata {
 	struct rspamd_http_message *msg;
 	struct fuzzy_slave_connection *conn;
 	struct rspamd_fuzzy_mirror *m;
+	GArray *updates_pending;
 };
 
 static void
@@ -335,12 +341,13 @@ fuzzy_mirror_updates_version_cb (guint64 rev64, void *ud)
 	ctx = cbdata->ctx;
 	msg = cbdata->msg;
 	m = cbdata->m;
-	g_free (cbdata);
+
 	rev32 = GUINT32_TO_LE (rev32);
 	len = sizeof (guint32) * 2; /* revision + last chunk */
 
-	for (i = 0; i < ctx->updates_pending->len; i ++) {
-		io_cmd = &g_array_index (ctx->updates_pending, struct fuzzy_peer_cmd, i);
+	for (i = 0; i < cbdata->updates_pending->len; i ++) {
+		io_cmd = &g_array_index (cbdata->updates_pending,
+				struct fuzzy_peer_cmd, i);
 
 		if (io_cmd->is_shingle) {
 			len += sizeof (guint32) + sizeof (guint32) +
@@ -356,8 +363,8 @@ fuzzy_mirror_updates_version_cb (guint64 rev64, void *ud)
 	reply = rspamd_fstring_append (reply, (const char *)&rev32,
 			sizeof (rev32));
 
-	for (i = 0; i < ctx->updates_pending->len; i ++) {
-		io_cmd = &g_array_index (ctx->updates_pending, struct fuzzy_peer_cmd, i);
+	for (i = 0; i < cbdata->updates_pending->len; i ++) {
+		io_cmd = &g_array_index (cbdata->updates_pending, struct fuzzy_peer_cmd, i);
 
 		if (io_cmd->is_shingle) {
 			len = sizeof (guint32) +
@@ -384,13 +391,17 @@ fuzzy_mirror_updates_version_cb (guint64 rev64, void *ud)
 			conn->sock,
 			&tv, ctx->ev_base);
 	msg_info ("send update request to %s", m->name);
+
+	g_array_free (cbdata->updates_pending, TRUE);
+	g_free (cbdata);
 }
 
 static void
 fuzzy_mirror_updates_to_http (struct rspamd_fuzzy_mirror *m,
-		struct fuzzy_slave_connection *conn,
-		struct rspamd_fuzzy_storage_ctx *ctx,
-		struct rspamd_http_message *msg)
+							  struct fuzzy_slave_connection *conn,
+							  struct rspamd_fuzzy_storage_ctx *ctx,
+							  struct rspamd_http_message *msg,
+							  GArray *updates)
 {
 
 	struct rspamd_fuzzy_updates_cbdata *cbdata;
@@ -400,6 +411,10 @@ fuzzy_mirror_updates_to_http (struct rspamd_fuzzy_mirror *m,
 	cbdata->msg = msg;
 	cbdata->conn = conn;
 	cbdata->m = m;
+	/* Copy queue */
+	cbdata->updates_pending = g_array_sized_new (FALSE, FALSE,
+			sizeof (struct fuzzy_peer_cmd), updates->len);
+	g_array_append_vals (cbdata->updates_pending, updates->data, updates->len);
 	rspamd_fuzzy_backend_version (ctx->backend, local_db_name,
 			fuzzy_mirror_updates_version_cb, cbdata);
 }
@@ -431,7 +446,7 @@ fuzzy_mirror_finish_handler (struct rspamd_http_connection *conn,
 
 static void
 rspamd_fuzzy_send_update_mirror (struct rspamd_fuzzy_storage_ctx *ctx,
-		struct rspamd_fuzzy_mirror *m)
+		struct rspamd_fuzzy_mirror *m, GArray *updates)
 {
 	struct fuzzy_slave_connection *conn;
 	struct rspamd_http_message *msg;
@@ -452,7 +467,7 @@ rspamd_fuzzy_send_update_mirror (struct rspamd_fuzzy_storage_ctx *ctx,
 
 	if (conn->sock == -1) {
 		msg_err ("cannot connect upstream for %s", m->name);
-		rspamd_upstream_fail (conn->up);
+		rspamd_upstream_fail (conn->up, TRUE);
 		return;
 	}
 
@@ -470,10 +485,11 @@ rspamd_fuzzy_send_update_mirror (struct rspamd_fuzzy_storage_ctx *ctx,
 	rspamd_http_connection_set_key (conn->http_conn,
 			ctx->sync_keypair);
 	msg->peer_key = rspamd_pubkey_ref (m->key);
-	fuzzy_mirror_updates_to_http (m, conn, ctx, msg);
+	fuzzy_mirror_updates_to_http (m, conn, ctx, msg, updates);
 }
 
 struct rspamd_updates_cbdata {
+	GArray *updates_pending;
 	struct rspamd_fuzzy_storage_ctx *ctx;
 	gchar *source;
 };
@@ -505,7 +521,12 @@ rspamd_fuzzy_stat_callback (gint fd, gshort what, gpointer ud)
 }
 
 static void
-rspamd_fuzzy_updates_cb (gboolean success, void *ud)
+rspamd_fuzzy_updates_cb (gboolean success,
+						 guint nadded,
+						 guint ndeleted,
+						 guint nextended,
+						 guint nignored,
+						 void *ud)
 {
 	struct rspamd_updates_cbdata *cbdata = ud;
 	struct rspamd_fuzzy_mirror *m;
@@ -523,14 +544,17 @@ rspamd_fuzzy_updates_cb (gboolean success, void *ud)
 			for (i = 0; i < ctx->mirrors->len; i ++) {
 				m = g_ptr_array_index (ctx->mirrors, i);
 
-				rspamd_fuzzy_send_update_mirror (ctx, m);
+				rspamd_fuzzy_send_update_mirror (ctx, m,
+						cbdata->updates_pending);
 			}
 		}
 
-		msg_info ("successfully updated fuzzy storage: %d updates processed",
-				ctx->updates_pending->len);
-		/* Clear updates */
-		ctx->updates_pending->len = 0;
+		msg_info ("successfully updated fuzzy storage: %d updates in queue; "
+				  "%d pending currently; "
+				  "%d added, %d deleted, %d extended, %d duplicates",
+				cbdata->updates_pending->len,
+				ctx->updates_pending->len,
+				nadded, ndeleted, nextended, nignored);
 		rspamd_fuzzy_backend_version (ctx->backend, source,
 				fuzzy_update_version_callback, g_strdup (source));
 		ctx->updates_failed = 0;
@@ -538,17 +562,22 @@ rspamd_fuzzy_updates_cb (gboolean success, void *ud)
 	else {
 		if (++ctx->updates_failed > ctx->updates_maxfail) {
 			msg_err ("cannot commit update transaction to fuzzy backend, discard "
-					"%ud updates after %d retries",
-					ctx->updates_pending->len,
+					 "%ud updates after %d retries",
+					cbdata->updates_pending->len,
 					ctx->updates_maxfail);
 			ctx->updates_failed = 0;
-			ctx->updates_pending->len = 0;
 		}
 		else {
 			msg_err ("cannot commit update transaction to fuzzy backend, "
-					"%ud updates are still pending, %d updates left",
+					 "%ud updates are still left; %ud currently pending;"
+					 " %d updates left",
+					cbdata->updates_pending->len,
 					ctx->updates_pending->len,
 					ctx->updates_maxfail - ctx->updates_failed);
+			/* Move the remaining updates to ctx queue */
+			g_array_append_vals (ctx->updates_pending,
+					cbdata->updates_pending->data,
+					cbdata->updates_pending->len);
 		}
 	}
 
@@ -562,6 +591,7 @@ rspamd_fuzzy_updates_cb (gboolean success, void *ud)
 		event_base_loopexit (ctx->ev_base, &tv);
 	}
 
+	g_array_free (cbdata->updates_pending, TRUE);
 	g_free (cbdata->source);
 	g_free (cbdata);
 }
@@ -576,8 +606,13 @@ rspamd_fuzzy_process_updates_queue (struct rspamd_fuzzy_storage_ctx *ctx,
 	if ((forced ||ctx->updates_pending->len > 0)) {
 		cbdata = g_malloc (sizeof (*cbdata));
 		cbdata->ctx = ctx;
+		cbdata->updates_pending = ctx->updates_pending;
+		ctx->updates_pending = g_array_sized_new (FALSE, FALSE,
+				sizeof (struct fuzzy_peer_cmd),
+				MAX (cbdata->updates_pending->len, 1024));
 		cbdata->source = g_strdup (source);
-		rspamd_fuzzy_backend_process_updates (ctx->backend, ctx->updates_pending,
+		rspamd_fuzzy_backend_process_updates (ctx->backend,
+				cbdata->updates_pending,
 				source, rspamd_fuzzy_updates_cb, cbdata);
 	}
 }
@@ -772,6 +807,8 @@ rspamd_fuzzy_check_callback (struct rspamd_fuzzy_reply *result, void *ud)
 	struct fuzzy_session *session = ud;
 	gboolean encrypted = FALSE, is_shingle = FALSE;
 	struct rspamd_fuzzy_cmd *cmd = NULL;
+	const struct rspamd_shingle *shingle = NULL;
+	struct rspamd_shingle sgl_cpy;
 
 	switch (session->cmd_type) {
 	case CMD_NORMAL:
@@ -779,6 +816,8 @@ rspamd_fuzzy_check_callback (struct rspamd_fuzzy_reply *result, void *ud)
 		break;
 	case CMD_SHINGLE:
 		cmd = &session->cmd.shingle.basic;
+		memcpy (&sgl_cpy, &session->cmd.shingle.sgl, sizeof (sgl_cpy));
+		shingle = &sgl_cpy;
 		is_shingle = TRUE;
 		break;
 	case CMD_ENCRYPTED_NORMAL:
@@ -787,12 +826,60 @@ rspamd_fuzzy_check_callback (struct rspamd_fuzzy_reply *result, void *ud)
 		break;
 	case CMD_ENCRYPTED_SHINGLE:
 		cmd = &session->cmd.enc_shingle.cmd.basic;
+		memcpy (&sgl_cpy,  &session->cmd.enc_shingle.cmd.sgl, sizeof (sgl_cpy));
+		shingle = &sgl_cpy;
 		encrypted = TRUE;
 		is_shingle = TRUE;
 		break;
 	}
 
 	rspamd_fuzzy_make_reply (cmd, result, session, encrypted, is_shingle);
+
+	/* Refresh hash if found with strong confidence */
+	if (result->v1.prob > 0.9 && !session->ctx->read_only) {
+		struct fuzzy_peer_cmd up_cmd;
+		struct fuzzy_peer_request *up_req;
+
+		if (session->worker->index == 0 || session->ctx->peer_fd == -1) {
+			/* Just add to the queue */
+			memset (&up_cmd, 0, sizeof (up_cmd));
+			up_cmd.is_shingle = is_shingle;
+			memcpy (up_cmd.cmd.normal.digest, result->digest,
+					sizeof (up_cmd.cmd.normal.digest));
+			up_cmd.cmd.normal.flag = result->v1.flag;
+			up_cmd.cmd.normal.cmd = FUZZY_REFRESH;
+			up_cmd.cmd.normal.shingles_count = cmd->shingles_count;
+
+			if (is_shingle && shingle) {
+				memcpy (&up_cmd.cmd.shingle.sgl, shingle,
+						sizeof (up_cmd.cmd.shingle.sgl));
+			}
+
+			g_array_append_val (session->ctx->updates_pending, up_cmd);
+		}
+		else {
+			/* We need to send request to the peer */
+			up_req = g_malloc0 (sizeof (*up_req));
+			up_req->cmd.is_shingle = is_shingle;
+
+			memcpy (up_req->cmd.cmd.normal.digest, result->digest,
+					sizeof (up_req->cmd.cmd.normal.digest));
+			up_req->cmd.cmd.normal.flag = result->v1.flag;
+			up_req->cmd.cmd.normal.cmd = FUZZY_REFRESH;
+			up_req->cmd.cmd.normal.shingles_count = cmd->shingles_count;
+
+			if (is_shingle && shingle) {
+				memcpy (&up_req->cmd.cmd.shingle.sgl, shingle,
+						sizeof (up_req->cmd.cmd.shingle.sgl));
+			}
+
+			event_set (&up_req->io_ev, session->ctx->peer_fd, EV_WRITE,
+					fuzzy_peer_send_io, up_req);
+			event_base_set (session->ctx->ev_base, &up_req->io_ev);
+			event_add (&up_req->io_ev, NULL);
+		}
+	}
+
 	REF_RELEASE (session);
 }
 
@@ -1060,7 +1147,7 @@ rspamd_fuzzy_decrypt_command (struct fuzzy_session *s)
 
 	/* Now decrypt request */
 	if (!rspamd_cryptobox_decrypt_nm_inplace (payload, payload_len, hdr->nonce,
-			rspamd_pubkey_get_nm (rk),
+			rspamd_pubkey_get_nm (rk, key->key),
 			hdr->mac, RSPAMD_CRYPTOBOX_MODE_25519)) {
 		msg_err ("decryption failed");
 		rspamd_pubkey_unref (rk);
@@ -1068,7 +1155,7 @@ rspamd_fuzzy_decrypt_command (struct fuzzy_session *s)
 		return FALSE;
 	}
 
-	memcpy (s->nm, rspamd_pubkey_get_nm (rk), sizeof (s->nm));
+	memcpy (s->nm, rspamd_pubkey_get_nm (rk, key->key), sizeof (s->nm));
 	rspamd_pubkey_unref (rk);
 
 	return TRUE;
@@ -2406,42 +2493,6 @@ init_fuzzy (struct rspamd_config *cfg)
 	ctx->updates_maxfail = DEFAULT_UPDATES_MAXFAIL;
 	ctx->collection_id_file = RSPAMD_DBDIR "/fuzzy_collection.id";
 
-	rspamd_rcl_register_worker_option (cfg,
-			type,
-			"hashfile",
-			rspamd_rcl_parse_struct_string,
-			ctx,
-			G_STRUCT_OFFSET (struct rspamd_fuzzy_storage_ctx, hashfile),
-			0,
-			"Path to fuzzy database");
-
-	rspamd_rcl_register_worker_option (cfg,
-			type,
-			"hash_file",
-			rspamd_rcl_parse_struct_string,
-			ctx,
-			G_STRUCT_OFFSET (struct rspamd_fuzzy_storage_ctx, hashfile),
-			0,
-			"Path to fuzzy database (alias for hashfile)");
-
-	rspamd_rcl_register_worker_option (cfg,
-			type,
-			"file",
-			rspamd_rcl_parse_struct_string,
-			ctx,
-			G_STRUCT_OFFSET (struct rspamd_fuzzy_storage_ctx, hashfile),
-			0,
-			"Path to fuzzy database (alias for hashfile)");
-
-	rspamd_rcl_register_worker_option (cfg,
-			type,
-			"database",
-			rspamd_rcl_parse_struct_string,
-			ctx,
-			G_STRUCT_OFFSET (struct rspamd_fuzzy_storage_ctx, hashfile),
-			0,
-			"Path to fuzzy database (alias for hashfile)");
-
 	rspamd_rcl_register_worker_option (cfg,
 			type,
 			"sync",
@@ -2502,6 +2553,15 @@ init_fuzzy (struct rspamd_config *cfg)
 			0,
 			"Allow encrypted requests only (and forbid all unknown keys or plaintext requests)");
 
+	rspamd_rcl_register_worker_option (cfg,
+			type,
+			"read_only",
+			rspamd_rcl_parse_struct_boolean,
+			ctx,
+			G_STRUCT_OFFSET (struct rspamd_fuzzy_storage_ctx, read_only),
+			0,
+			"Work in read only mode");
+
 	rspamd_rcl_register_worker_option (cfg,
 			type,
 			"master_timeout",
@@ -2880,7 +2940,10 @@ start_fuzzy (struct rspamd_worker *worker)
 		struct rspamd_map *m;
 
 		if ((m = rspamd_map_add_from_ucl (cfg, ctx->skip_map,
-				"Skip hashes", rspamd_kv_list_read, rspamd_kv_list_fin,
+				"Skip hashes",
+				rspamd_kv_list_read,
+				rspamd_kv_list_fin,
+				rspamd_kv_list_dtor,
 				(void **)&ctx->skip_hashes)) == NULL) {
 			msg_warn_config ("cannot load hashes list from %s",
 					ucl_object_tostring (ctx->skip_map));
@@ -2900,7 +2963,7 @@ start_fuzzy (struct rspamd_worker *worker)
 	ctx->resolver = dns_resolver_init (worker->srv->logger,
 				ctx->ev_base,
 				worker->srv->cfg);
-	rspamd_map_watch (worker->srv->cfg, ctx->ev_base, ctx->resolver, 0);
+	rspamd_map_watch (worker->srv->cfg, ctx->ev_base, ctx->resolver, worker, 0);
 
 	/* Get peer pipe */
 	memset (&srv_cmd, 0, sizeof (srv_cmd));
@@ -2950,11 +3013,11 @@ start_fuzzy (struct rspamd_worker *worker)
 
 			close (fd);
 		}
-
-		g_array_free (ctx->updates_pending, TRUE);
 	}
 
-	rspamd_log_close (worker->srv->logger);
+	if (worker->index == 0) {
+		g_array_free (ctx->updates_pending, TRUE);
+	}
 
 	if (ctx->peer_fd != -1) {
 		if (worker->index == 0) {
@@ -2969,5 +3032,7 @@ start_fuzzy (struct rspamd_worker *worker)
 
 	REF_RELEASE (ctx->cfg);
 
+	rspamd_log_close (worker->srv->logger, TRUE);
+
 	exit (EXIT_SUCCESS);
 }
