@@ -23,6 +23,10 @@
 #include "message.h"
 #include <unicode/ucnv.h>
 #include <unicode/ucsdet.h>
+#if U_ICU_VERSION_MAJOR_NUM >= 44
+#include <unicode/unorm2.h>
+#endif
+#include <math.h>
 
 #define UTF8_CHARSET "UTF-8"
 
@@ -38,6 +42,10 @@
 static rspamd_regexp_t *utf_compatible_re = NULL;
 UConverter *utf8_converter = NULL;
 
+#if U_ICU_VERSION_MAJOR_NUM >= 44
+static const UNormalizer2 *norm = NULL;
+#endif
+
 struct rspamd_charset_substitution {
 	const gchar *input;
 	const gchar *canon;
@@ -82,7 +90,7 @@ rspamd_mime_get_converter_cached (const gchar *enc, UErrorCode *err)
 		if (conv != NULL) {
 			ucnv_setToUCallBack (conv,
 					UCNV_TO_U_CALLBACK_SUBSTITUTE,
-					UCNV_SUB_STOP_ON_ILLEGAL,
+					NULL,
 					NULL,
 					NULL,
 					err);
@@ -93,6 +101,36 @@ rspamd_mime_get_converter_cached (const gchar *enc, UErrorCode *err)
 	return conv;
 }
 
+static inline void
+rspamd_mime_utf8_conv_init (void)
+{
+	if (utf8_converter == NULL) {
+		UErrorCode uc_err = U_ZERO_ERROR;
+
+		utf8_converter = ucnv_open (UTF8_CHARSET, &uc_err);
+
+		if (!U_SUCCESS (uc_err)) {
+			msg_err ("FATAL error: cannot open converter for utf8: %s",
+					u_errorName (uc_err));
+
+			g_assert_not_reached ();
+		}
+
+		ucnv_setFromUCallBack (utf8_converter,
+				UCNV_FROM_U_CALLBACK_SUBSTITUTE,
+				NULL,
+				NULL,
+				NULL,
+				&uc_err);
+		ucnv_setToUCallBack (utf8_converter,
+				UCNV_TO_U_CALLBACK_SUBSTITUTE,
+				NULL,
+				NULL,
+				NULL,
+				&uc_err);
+	}
+}
+
 static void
 rspamd_mime_encoding_substitute_init (void)
 {
@@ -188,25 +226,7 @@ rspamd_mime_text_to_utf8 (rspamd_mempool_t *pool,
 	UErrorCode uc_err = U_ZERO_ERROR;
 	UConverter *conv;
 
-	if (utf8_converter == NULL) {
-		utf8_converter = ucnv_open (UTF8_CHARSET, &uc_err);
-
-		if (uc_err != U_ZERO_ERROR) {
-			g_set_error (err, rspamd_iconv_error_quark (), EINVAL,
-					"cannot open converter for utf8: %s",
-					u_errorName (uc_err));
-
-			return NULL;
-		}
-
-		ucnv_setFromUCallBack (utf8_converter,
-				UCNV_FROM_U_CALLBACK_SUBSTITUTE,
-				NULL,
-				NULL,
-				NULL,
-				&uc_err);
-	}
-
+	rspamd_mime_utf8_conv_init ();
 	conv = rspamd_mime_get_converter_cached (in_enc, &uc_err);
 
 	if (conv == NULL) {
@@ -221,7 +241,7 @@ rspamd_mime_text_to_utf8 (rspamd_mempool_t *pool,
 	uc_err = U_ZERO_ERROR;
 	r = ucnv_toUChars (conv, tmp_buf, len + 1, input, len, &uc_err);
 
-	if (uc_err != U_ZERO_ERROR) {
+	if (!U_SUCCESS (uc_err)) {
 		g_set_error (err, rspamd_iconv_error_quark (), EINVAL,
 					"cannot convert data to unicode from %s: %s",
 					in_enc, u_errorName (uc_err));
@@ -236,7 +256,7 @@ rspamd_mime_text_to_utf8 (rspamd_mempool_t *pool,
 	d = rspamd_mempool_alloc (pool, dlen);
 	r = ucnv_fromUChars (utf8_converter, d, dlen, tmp_buf, r, &uc_err);
 
-	if (uc_err != U_ZERO_ERROR) {
+	if (!U_SUCCESS (uc_err)) {
 		g_set_error (err, rspamd_iconv_error_quark (), EINVAL,
 				"cannot convert data from unicode from %s: %s",
 				in_enc, u_errorName (uc_err));
@@ -256,6 +276,186 @@ rspamd_mime_text_to_utf8 (rspamd_mempool_t *pool,
 	return d;
 }
 
+static void
+rspamd_mime_text_part_ucs_from_utf (struct rspamd_task *task,
+									struct rspamd_mime_text_part *text_part)
+{
+	GByteArray *utf;
+	UErrorCode uc_err = U_ZERO_ERROR;
+
+	rspamd_mime_utf8_conv_init ();
+	utf = text_part->utf_raw_content;
+	text_part->unicode_raw_content = g_array_sized_new (FALSE, FALSE,
+			sizeof (UChar), utf->len + 1);
+	text_part->unicode_raw_content->len = ucnv_toUChars (utf8_converter,
+			(UChar *)text_part->unicode_raw_content->data,
+			utf->len + 1,
+			utf->data,
+			utf->len,
+			&uc_err);
+
+	if (!U_SUCCESS (uc_err)) {
+		g_array_free (text_part->unicode_raw_content, TRUE);
+		text_part->unicode_raw_content = NULL;
+	}
+}
+
+static void
+rspamd_mime_text_part_normalise (struct rspamd_task *task,
+								 struct rspamd_mime_text_part *text_part)
+{
+#if U_ICU_VERSION_MAJOR_NUM >= 44
+	UErrorCode uc_err = U_ZERO_ERROR;
+	gint32 nsym, end;
+	UChar *src = NULL, *dest = NULL;
+
+	if (norm == NULL) {
+		norm = unorm2_getInstance (NULL, "nfkc", UNORM2_COMPOSE, &uc_err);
+	}
+
+	if (!text_part->unicode_raw_content) {
+		return;
+	}
+
+	src = (UChar *)text_part->unicode_raw_content->data;
+	nsym = text_part->unicode_raw_content->len;
+
+	/* We can now check if we need to decompose */
+	end = unorm2_spanQuickCheckYes (norm, src, nsym, &uc_err);
+
+	if (!U_SUCCESS (uc_err)) {
+		msg_warn_task ("cannot normalise URL, cannot check normalisation: %s",
+				u_errorName (uc_err));
+		return;
+	}
+
+	if (end == nsym) {
+		/* Already normalised */
+		return;
+	}
+
+	text_part->flags |= RSPAMD_MIME_TEXT_PART_HAS_SUBNORMAL;
+	dest = g_malloc (nsym * sizeof (*dest));
+	memcpy (dest, src, end * sizeof (*dest));
+	nsym = unorm2_normalizeSecondAndAppend (norm, dest, end, nsym,
+			src + end, nsym - end, &uc_err);
+
+	if (!U_SUCCESS (uc_err)) {
+		if (uc_err != U_BUFFER_OVERFLOW_ERROR) {
+			msg_warn_task ("cannot normalise URL: %s",
+					u_errorName (uc_err));
+		}
+	}
+	else {
+		/* Copy normalised back */
+		memcpy (text_part->unicode_raw_content->data, dest, nsym * sizeof (UChar));
+		text_part->unicode_raw_content->len = nsym;
+		text_part->flags |= RSPAMD_MIME_TEXT_PART_NORMALISED;
+	}
+
+	g_free (dest);
+#endif
+}
+
+/*
+ * Recode utf from normalised unichars if needed
+ */
+static void
+rspamd_mime_text_part_maybe_renormalise (struct rspamd_task *task,
+										 struct rspamd_mime_text_part *text_part)
+{
+	UErrorCode uc_err = U_ZERO_ERROR;
+	guint clen, dlen;
+	gint r;
+
+	rspamd_mime_utf8_conv_init ();
+
+	if ((text_part->flags & RSPAMD_MIME_TEXT_PART_NORMALISED) &&
+		text_part->unicode_raw_content) {
+		clen = ucnv_getMaxCharSize (utf8_converter);
+		dlen = UCNV_GET_MAX_BYTES_FOR_STRING (text_part->unicode_raw_content->len,
+				clen);
+		g_byte_array_set_size (text_part->utf_raw_content, dlen);
+		r = ucnv_fromUChars (utf8_converter,
+				text_part->utf_raw_content->data,
+				dlen,
+				(UChar *)text_part->unicode_raw_content->data,
+				text_part->unicode_raw_content->len,
+				&uc_err);
+		text_part->utf_raw_content->len = r;
+	}
+}
+
+
+static gboolean
+rspamd_mime_text_part_utf8_convert (struct rspamd_task *task,
+									struct rspamd_mime_text_part *text_part,
+									GByteArray *input,
+									const gchar *charset,
+									GError **err)
+{
+	gchar *d;
+	gint32 r, clen, dlen;
+
+	UErrorCode uc_err = U_ZERO_ERROR;
+	UConverter *conv;
+
+	rspamd_mime_utf8_conv_init ();
+	conv = rspamd_mime_get_converter_cached (charset, &uc_err);
+
+	if (conv == NULL) {
+		g_set_error (err, rspamd_iconv_error_quark (), EINVAL,
+				"cannot open converter for %s: %s",
+				charset, u_errorName (uc_err));
+
+		return FALSE;
+	}
+
+
+	text_part->unicode_raw_content = g_array_sized_new (FALSE, FALSE,
+			sizeof (UChar), input->len + 1);
+	r = ucnv_toUChars (conv,
+			(UChar *)text_part->unicode_raw_content->data,
+			input->len + 1,
+			input->data,
+			input->len,
+			&uc_err);
+
+	if (!U_SUCCESS (uc_err)) {
+		g_set_error (err, rspamd_iconv_error_quark (), EINVAL,
+				"cannot convert data to unicode from %s: %s",
+				charset, u_errorName (uc_err));
+		return FALSE;
+	}
+
+	text_part->unicode_raw_content->len = r;
+	rspamd_mime_text_part_normalise (task, text_part);
+
+	/* Now, convert to utf8 */
+	clen = ucnv_getMaxCharSize (utf8_converter);
+	dlen = UCNV_GET_MAX_BYTES_FOR_STRING (r, clen);
+	d = rspamd_mempool_alloc (task->task_pool, dlen);
+	r = ucnv_fromUChars (utf8_converter, d, dlen,
+			(UChar *)text_part->unicode_raw_content->data, r, &uc_err);
+
+	if (!U_SUCCESS (uc_err)) {
+		g_set_error (err, rspamd_iconv_error_quark (), EINVAL,
+				"cannot convert data from unicode from %s: %s",
+				charset, u_errorName (uc_err));
+
+		return FALSE;
+	}
+
+	msg_info_task ("converted from %s to UTF-8 inlen: %z, outlen: %d",
+			charset, input->len, r);
+	text_part->utf_raw_content = rspamd_mempool_alloc (task->task_pool,
+			sizeof (*text_part->utf_raw_content) + sizeof (gpointer) * 4);
+	text_part->utf_raw_content->data = d;
+	text_part->utf_raw_content->len = r;
+
+	return TRUE;
+}
+
 gboolean
 rspamd_mime_to_utf8_byte_array (GByteArray *in,
 		GByteArray *out,
@@ -277,24 +477,7 @@ rspamd_mime_to_utf8_byte_array (GByteArray *in,
 		return TRUE;
 	}
 
-	if (utf8_converter == NULL) {
-		utf8_converter = ucnv_open (UTF8_CHARSET, &uc_err);
-
-		if (uc_err != U_ZERO_ERROR) {
-			msg_warn ("cannot open converter for utf8: %s",
-					u_errorName (uc_err));
-
-			return FALSE;
-		}
-
-		ucnv_setFromUCallBack (utf8_converter,
-				UCNV_FROM_U_CALLBACK_SUBSTITUTE,
-				NULL,
-				NULL,
-				NULL,
-				&uc_err);
-	}
-
+	rspamd_mime_utf8_conv_init ();
 	conv = rspamd_mime_get_converter_cached (enc, &uc_err);
 
 	if (conv == NULL) {
@@ -305,7 +488,7 @@ rspamd_mime_to_utf8_byte_array (GByteArray *in,
 	uc_err = U_ZERO_ERROR;
 	r = ucnv_toUChars (conv, tmp_buf, in->len + 1, in->data, in->len, &uc_err);
 
-	if (uc_err != U_ZERO_ERROR) {
+	if (!U_SUCCESS (uc_err)) {
 		g_free (tmp_buf);
 
 		return FALSE;
@@ -317,7 +500,7 @@ rspamd_mime_to_utf8_byte_array (GByteArray *in,
 	g_byte_array_set_size (out, dlen);
 	r = ucnv_fromUChars (utf8_converter, out->data, dlen, tmp_buf, r, &uc_err);
 
-	if (uc_err != U_ZERO_ERROR) {
+	if (!U_SUCCESS (uc_err)) {
 		g_free (tmp_buf);
 
 		return FALSE;
@@ -372,6 +555,7 @@ rspamd_mime_charset_find_by_content (gchar *in, gsize inlen)
 	const UCharsetMatch **csm, *sel = NULL;
 	UErrorCode uc_err = U_ZERO_ERROR;
 	gint32 matches, i, max_conf = G_MININT32, conf;
+	gdouble mean = 0.0, stddev = 0.0;
 
 	if (csd == NULL) {
 		csd = ucsdet_open (&uc_err);
@@ -398,9 +582,13 @@ detect:
 			max_conf = conf;
 			sel = csm[i];
 		}
+
+		mean += (conf - mean) / (i + 1);
+		gdouble err = fabs (conf - mean);
+		stddev += (err - stddev) / (i + 1);
 	}
 
-	if (sel && max_conf > 50) {
+	if (sel && ((max_conf > 50) || (max_conf - mean > stddev * 1.25))) {
 		return ucsdet_getName (sel, &uc_err);
 	}
 
@@ -455,16 +643,14 @@ rspamd_mime_charset_utf_check (rspamd_ftok_t *charset,
 	return FALSE;
 }
 
-GByteArray *
+void
 rspamd_mime_text_part_maybe_convert (struct rspamd_task *task,
 		struct rspamd_mime_text_part *text_part)
 {
 	GError *err = NULL;
-	gsize write_bytes;
 	const gchar *charset = NULL;
 	gboolean checked = FALSE, need_charset_heuristic = TRUE;
-	gchar *res_str;
-	GByteArray *result_array, *part_content;
+	GByteArray *part_content;
 	rspamd_ftok_t charset_tok;
 	struct rspamd_mime_part *part = text_part->mime_part;
 
@@ -472,11 +658,12 @@ rspamd_mime_text_part_maybe_convert (struct rspamd_task *task,
 		text_part->flags |= RSPAMD_MIME_TEXT_PART_FLAG_8BIT;
 	}
 
-	part_content = rspamd_mempool_alloc0 (task->task_pool, sizeof (GByteArray));
-	part_content->data = rspamd_mempool_alloc (task->task_pool,
-			text_part->parsed.len);
+	/* Allocate copy storage */
+	part_content = g_byte_array_sized_new (text_part->parsed.len);
 	memcpy (part_content->data, text_part->parsed.begin, text_part->parsed.len);
 	part_content->len = text_part->parsed.len;
+	rspamd_mempool_add_destructor (task->task_pool,
+			(rspamd_mempool_destruct_t)g_byte_array_unref, part_content);
 
 	if (rspamd_str_has_8bit (text_part->parsed.begin, text_part->parsed.len)) {
 		text_part->flags |= RSPAMD_MIME_TEXT_PART_FLAG_8BIT_ENCODED;
@@ -488,8 +675,9 @@ rspamd_mime_text_part_maybe_convert (struct rspamd_task *task,
 
 	if (task->cfg && task->cfg->raw_mode) {
 		SET_PART_RAW (text_part);
+		text_part->utf_raw_content = part_content;
 
-		return part_content;
+		return;
 	}
 
 	if (part->ct->charset.len == 0) {
@@ -505,8 +693,12 @@ rspamd_mime_text_part_maybe_convert (struct rspamd_task *task,
 		}
 		else {
 			SET_PART_UTF (text_part);
+			text_part->utf_raw_content = part_content;
+			rspamd_mime_text_part_ucs_from_utf (task, text_part);
+			rspamd_mime_text_part_normalise (task, text_part);
+			rspamd_mime_text_part_maybe_renormalise (task, text_part);
 
-			return part_content;
+			return;
 		}
 	}
 	else {
@@ -524,8 +716,9 @@ rspamd_mime_text_part_maybe_convert (struct rspamd_task *task,
 	if (charset == NULL) {
 		msg_info_task ("<%s>: has invalid charset", task->message_id);
 		SET_PART_RAW (text_part);
+		text_part->utf_raw_content = part_content;
 
-		return NULL;
+		return;
 	}
 
 	RSPAMD_FTOK_FROM_STR (&charset_tok, charset);
@@ -533,18 +726,18 @@ rspamd_mime_text_part_maybe_convert (struct rspamd_task *task,
 	if (rspamd_mime_charset_utf_check (&charset_tok, part_content->data,
 			part_content->len, !checked)) {
 		SET_PART_UTF (text_part);
+		text_part->utf_raw_content = part_content;
+		rspamd_mime_text_part_ucs_from_utf (task, text_part);
+		rspamd_mime_text_part_normalise (task, text_part);
+		rspamd_mime_text_part_maybe_renormalise (task, text_part);
 
-		return part_content;
+		return;
 	}
 	else {
 		charset = charset_tok.begin;
-		res_str = rspamd_mime_text_to_utf8 (task->task_pool, part_content->data,
-				part_content->len,
-				charset,
-				&write_bytes,
-				&err);
 
-		if (res_str == NULL) {
+		if (!rspamd_mime_text_part_utf8_convert (task, text_part,
+				part_content, charset, &err)) {
 			msg_warn_task ("<%s>: cannot convert from %s to utf8: %s",
 					task->message_id,
 					charset,
@@ -552,14 +745,24 @@ rspamd_mime_text_part_maybe_convert (struct rspamd_task *task,
 			SET_PART_RAW (text_part);
 			g_error_free (err);
 
-			return NULL;
+			text_part->utf_raw_content = part_content;
+			return;
 		}
 	}
 
-	result_array = rspamd_mempool_alloc (task->task_pool, sizeof (GByteArray));
-	result_array->data = res_str;
-	result_array->len = write_bytes;
 	SET_PART_UTF (text_part);
+}
 
-	return result_array;
+void
+rspamd_utf_to_unicode (GByteArray *in, GArray *dest)
+{
+	UErrorCode uc_err = U_ZERO_ERROR;
+
+	g_array_set_size (dest, in->len + 1);
+	dest->len = ucnv_toUChars (utf8_converter,
+			(UChar *)dest->data,
+			in->len + 1,
+			in->data,
+			in->len,
+			&uc_err);
 }
