@@ -83,7 +83,7 @@ static gboolean rspamd_is_empty_body (struct rspamd_task *task,
 
 static rspamd_expression_atom_t * rspamd_mime_expr_parse (const gchar *line, gsize len,
 		rspamd_mempool_t *pool, gpointer ud, GError **err);
-static gdouble rspamd_mime_expr_process (gpointer input, rspamd_expression_atom_t *atom);
+static gdouble rspamd_mime_expr_process (struct rspamd_expr_process_data *process_data, rspamd_expression_atom_t *atom);
 static gint rspamd_mime_expr_priority (rspamd_expression_atom_t *atom);
 static void rspamd_mime_expr_destroy (rspamd_expression_atom_t *atom);
 
@@ -94,7 +94,10 @@ struct rspamd_regexp_atom {
 	enum rspamd_re_type type;                       /**< regexp type										*/
 	gchar *regexp_text;                             /**< regexp text representation							*/
 	rspamd_regexp_t *regexp;                        /**< regexp structure									*/
-	gchar *header;                                  /**< header name for header regexps						*/
+	union {
+		const gchar *header;                        /**< header name for header regexps						*/
+		const gchar *selector;                      /**< selector name for lua selector regexp				*/
+	} extra;
 	gboolean is_test;                               /**< true if this expression must be tested				*/
 	gboolean is_strong;                             /**< true if headers search must be case sensitive		*/
 	gboolean is_multiple;                           /**< true if we need to match all inclusions of atom	*/
@@ -236,6 +239,10 @@ rspamd_parse_long_option (const gchar *start, gsize len,
 		ret = TRUE;
 		a->type = RSPAMD_RE_SARAWBODY;
 	}
+	else if (TYPE_CHECK (start, "selector", len)) {
+		ret = TRUE;
+		a->type = RSPAMD_RE_SELECTOR;
+	}
 
 	return ret;
 }
@@ -248,7 +255,7 @@ rspamd_mime_expr_parse_regexp_atom (rspamd_mempool_t * pool, const gchar *line,
 		struct rspamd_config *cfg)
 {
 	const gchar *begin, *end, *p, *src, *start, *brace;
-	gchar *dbegin, *dend;
+	gchar *dbegin, *dend, *extra = NULL;
 	struct rspamd_regexp_atom *result;
 	GError *err = NULL;
 	GString *re_flags;
@@ -268,6 +275,9 @@ rspamd_mime_expr_parse_regexp_atom (rspamd_mempool_t * pool, const gchar *line,
 		msg_warn_pool ("got empty regexp");
 		return NULL;
 	}
+
+	result->type = RSPAMD_RE_MAX;
+
 	start = line;
 	/* First try to find header name */
 	begin = strchr (line, '/');
@@ -281,15 +291,15 @@ rspamd_mime_expr_parse_regexp_atom (rspamd_mempool_t * pool, const gchar *line,
 			}
 			p--;
 		}
+
 		if (end) {
-			result->header = rspamd_mempool_alloc (pool, end - line + 1);
-			rspamd_strlcpy (result->header, line, end - line + 1);
-			result->type = RSPAMD_RE_HEADER;
+			extra = rspamd_mempool_alloc (pool, end - line + 1);
+			rspamd_strlcpy (extra, line, end - line + 1);
 			line = end;
 		}
 	}
 	else {
-		result->header = rspamd_mempool_strdup (pool, line);
+		extra = rspamd_mempool_strdup (pool, line);
 		result->type = RSPAMD_RE_MAX;
 		line = start;
 	}
@@ -300,9 +310,9 @@ rspamd_mime_expr_parse_regexp_atom (rspamd_mempool_t * pool, const gchar *line,
 	if (*line != '\0') {
 		begin = line + 1;
 	}
-	else if (result->header == NULL) {
+	else if (extra == NULL) {
 		/* Assume that line without // is just a header name */
-		result->header = rspamd_mempool_strdup (pool, line);
+		extra = rspamd_mempool_strdup (pool, line);
 		result->type = RSPAMD_RE_HEADER;
 		return result;
 	}
@@ -382,6 +392,10 @@ rspamd_mime_expr_parse_regexp_atom (rspamd_mempool_t * pool, const gchar *line,
 			result->type = RSPAMD_RE_RAWHEADER;
 			p++;
 			break;
+		case '$':
+			result->type = RSPAMD_RE_SELECTOR;
+			p++;
+			break;
 		case '{':
 			/* Long definition */
 			if ((brace = strchr (p + 1, '}')) != NULL) {
@@ -419,16 +433,37 @@ rspamd_mime_expr_parse_regexp_atom (rspamd_mempool_t * pool, const gchar *line,
 	}
 
 	if (result->type >= RSPAMD_RE_MAX) {
-		msg_err_pool ("could not read regexp: %s, unknown type", src);
-		return NULL;
+		if (extra) {
+			/* Assume header regexp */
+			result->extra.header = extra;
+			result->type = RSPAMD_RE_HEADER;
+		}
+		else {
+			msg_err_pool ("could not read regexp: %s, unknown type", src);
+			return NULL;
+		}
 	}
 
 	if ((result->type == RSPAMD_RE_HEADER ||
 			result->type == RSPAMD_RE_RAWHEADER ||
-			result->type == RSPAMD_RE_MIMEHEADER) &&
-			result->header == NULL) {
-		msg_err_pool ("header regexp: '%s' has no header part", src);
-		return NULL;
+			result->type == RSPAMD_RE_MIMEHEADER)) {
+		if (extra == NULL) {
+			msg_err_pool ("header regexp: '%s' has no header part", src);
+			return NULL;
+		}
+		else {
+			result->extra.header = extra;
+		}
+	}
+
+	if (result->type == RSPAMD_RE_SELECTOR) {
+		if (extra == NULL) {
+			msg_err_pool ("selector regexp: '%s' has no selector part", src);
+			return NULL;
+		}
+		else {
+			result->extra.selector = extra;
+		}
 	}
 
 
@@ -655,7 +690,7 @@ rspamd_mime_expr_parse (const gchar *line, gsize len,
 				state = in_flags_brace;
 				p ++;
 			}
-			else if (!g_ascii_isalpha (t)) {
+			else if (!g_ascii_isalpha (t) && t != '$') {
 				state = end_atom;
 			}
 			else {
@@ -734,13 +769,13 @@ set:
 					mime_atom->d.re->type == RSPAMD_RE_RAWHEADER ||
 					mime_atom->d.re->type == RSPAMD_RE_MIMEHEADER) {
 
-				if (mime_atom->d.re->header != NULL) {
+				if (mime_atom->d.re->extra.header != NULL) {
 					own_re = mime_atom->d.re->regexp;
 					mime_atom->d.re->regexp = rspamd_re_cache_add (cfg->re_cache,
 							mime_atom->d.re->regexp,
 							mime_atom->d.re->type,
-							mime_atom->d.re->header,
-							strlen (mime_atom->d.re->header) + 1);
+							mime_atom->d.re->extra.header,
+							strlen (mime_atom->d.re->extra.header) + 1);
 					/* Pass ownership to the cache */
 					rspamd_regexp_unref (own_re);
 				}
@@ -749,7 +784,29 @@ set:
 					g_set_error (err,
 							rspamd_mime_expr_quark (),
 							200,
-							"no header name in /H regexp: '%s'",
+							"no header name in header regexp: '%s'",
+							mime_atom->str);
+					goto err;
+				}
+
+			}
+			else if (mime_atom->d.re->type == RSPAMD_RE_SELECTOR) {
+				if (mime_atom->d.re->extra.selector != NULL) {
+					own_re = mime_atom->d.re->regexp;
+					mime_atom->d.re->regexp = rspamd_re_cache_add (cfg->re_cache,
+							mime_atom->d.re->regexp,
+							mime_atom->d.re->type,
+							mime_atom->d.re->extra.selector,
+							strlen (mime_atom->d.re->extra.selector) + 1);
+					/* Pass ownership to the cache */
+					rspamd_regexp_unref (own_re);
+				}
+				else {
+					/* We have header regexp, but no header name is detected */
+					g_set_error (err,
+							rspamd_mime_expr_quark (),
+							200,
+							"no selector name in selector regexp: '%s'",
 							mime_atom->str);
 					goto err;
 				}
@@ -817,8 +874,16 @@ rspamd_mime_expr_process_regexp (struct rspamd_regexp_atom *re,
 		ret = rspamd_re_cache_process (task,
 				re->regexp,
 				re->type,
-				re->header,
-				strlen (re->header),
+				re->extra.header,
+				strlen (re->extra.header),
+				re->is_strong);
+	}
+	else if (re->type == RSPAMD_RE_SELECTOR) {
+		ret = rspamd_re_cache_process (task,
+				re->regexp,
+				re->type,
+				re->extra.selector,
+				strlen (re->extra.selector),
 				re->is_strong);
 	}
 	else {
@@ -913,9 +978,9 @@ rspamd_mime_expr_process_function (struct rspamd_function_atom * func,
 }
 
 static gdouble
-rspamd_mime_expr_process (gpointer input, rspamd_expression_atom_t *atom)
+rspamd_mime_expr_process (struct rspamd_expr_process_data *process_data, rspamd_expression_atom_t *atom)
 {
-	struct rspamd_task *task = input;
+	struct rspamd_task *task = process_data->task;
 	struct rspamd_mime_atom *mime_atom;
 	lua_State *L;
 	gdouble ret = 0;
@@ -1407,7 +1472,7 @@ match_smtp_data (struct rspamd_task *task,
 	const gchar *what, gsize len)
 {
 	rspamd_regexp_t *re;
-	gint r;
+	gint r = 0;
 
 	if (arg->type == EXPRESSION_ARGUMENT_REGEXP) {
 		/* This is a regexp */
@@ -1418,7 +1483,9 @@ match_smtp_data (struct rspamd_task *task,
 		}
 
 
-		r = rspamd_regexp_search (re, what, len, NULL, NULL, FALSE, NULL);
+		if (len > 0) {
+			r = rspamd_regexp_search (re, what, len, NULL, NULL, FALSE, NULL);
+		}
 
 		return r;
 	}
@@ -1598,9 +1665,12 @@ rspamd_content_type_compare_param (struct rspamd_task * task,
 				DL_FOREACH (found, cur) {
 					if (arg_pattern->type == EXPRESSION_ARGUMENT_REGEXP) {
 						re = arg_pattern->data;
-						r = rspamd_regexp_search (re,
-								cur->value.begin, cur->value.len,
-								NULL, NULL, FALSE, NULL);
+
+						if (cur->value.len > 0) {
+							r = rspamd_regexp_search (re,
+									cur->value.begin, cur->value.len,
+									NULL, NULL, FALSE, NULL);
+						}
 
 						if (r) {
 							return TRUE;
@@ -1694,7 +1764,7 @@ rspamd_content_type_check (struct rspamd_task *task,
 	rspamd_regexp_t *re;
 	struct expression_argument *arg1, *arg_pattern;
 	struct rspamd_content_type *ct;
-	gint r;
+	gint r = 0;
 	guint i;
 	gboolean recursive = FALSE;
 	struct rspamd_mime_part *cur_part;
@@ -1736,8 +1806,11 @@ rspamd_content_type_check (struct rspamd_task *task,
 
 		if (arg_pattern->type == EXPRESSION_ARGUMENT_REGEXP) {
 			re = arg_pattern->data;
-			r = rspamd_regexp_search (re, param_data->begin, param_data->len,
-					NULL, NULL, FALSE, NULL);
+
+			if (param_data->len > 0) {
+				r = rspamd_regexp_search (re, param_data->begin, param_data->len,
+						NULL, NULL, FALSE, NULL);
+			}
 
 			if (r) {
 				return TRUE;
@@ -1792,8 +1865,11 @@ compare_subtype (struct rspamd_task *task, struct rspamd_content_type *ct,
 	}
 	if (subtype->type == EXPRESSION_ARGUMENT_REGEXP) {
 		re = subtype->data;
-		r = rspamd_regexp_search (re, ct->subtype.begin, ct->subtype.len,
-				NULL, NULL, FALSE, NULL);
+
+		if (ct->subtype.len > 0) {
+			r = rspamd_regexp_search (re, ct->subtype.begin, ct->subtype.len,
+					NULL, NULL, FALSE, NULL);
+		}
 	}
 	else {
 		srch.begin = subtype->data;
@@ -1837,7 +1913,7 @@ common_has_content_part (struct rspamd_task * task,
 	struct rspamd_mime_part *part;
 	struct rspamd_content_type *ct;
 	rspamd_ftok_t srch;
-	gint r;
+	gint r = 0;
 	guint i;
 
 	for (i = 0; i < task->parts->len; i ++) {
@@ -1851,8 +1927,11 @@ common_has_content_part (struct rspamd_task * task,
 		if (param_type->type == EXPRESSION_ARGUMENT_REGEXP) {
 			re = param_type->data;
 
-			r = rspamd_regexp_search (re, ct->type.begin, ct->type.len,
-					NULL, NULL, FALSE, NULL);
+			if (ct->type.len > 0) {
+				r = rspamd_regexp_search (re, ct->type.begin, ct->type.len,
+						NULL, NULL, FALSE, NULL);
+			}
+
 			/* Also check subtype and length of the part */
 			if (r && param_subtype) {
 				r = compare_len (part, min_len, max_len) &&
