@@ -474,43 +474,57 @@ rspamd_url_add_static_matchers (struct url_match_scanner *sc)
 	g_array_append_vals (sc->matchers, static_matchers, n);
 }
 
+void
+rspamd_url_deinit (void)
+{
+	if (url_scanner != NULL) {
+		rspamd_multipattern_destroy (url_scanner->search_trie);
+		g_array_free (url_scanner->matchers, TRUE);
+		g_free (url_scanner);
+
+		url_scanner = NULL;
+	}
+}
+
 void
 rspamd_url_init (const gchar *tld_file)
 {
 	GError *err = NULL;
 
-	if (url_scanner == NULL) {
-		url_scanner = g_malloc (sizeof (struct url_match_scanner));
-
-		if (tld_file) {
-			/* Reserve larger multipattern */
-			url_scanner->matchers = g_array_sized_new (FALSE, TRUE,
-					sizeof (struct url_matcher), 13000);
-			url_scanner->search_trie = rspamd_multipattern_create_sized (13000,
-				RSPAMD_MULTIPATTERN_TLD | RSPAMD_MULTIPATTERN_ICASE);
-		}
-		else {
-			url_scanner->matchers = g_array_sized_new (FALSE, TRUE,
-					sizeof (struct url_matcher), 128);
-			url_scanner->search_trie = rspamd_multipattern_create_sized (128,
-					RSPAMD_MULTIPATTERN_TLD | RSPAMD_MULTIPATTERN_ICASE);
-		}
-
-		rspamd_url_add_static_matchers (url_scanner);
-
-		if (tld_file != NULL) {
-			rspamd_url_parse_tld_file (tld_file, url_scanner);
-		}
-
-		if (!rspamd_multipattern_compile (url_scanner->search_trie, &err)) {
-			msg_err ("cannot compile tld patterns, url matching will be "
-					"broken completely: %e", err);
-			g_error_free (err);
-		}
-
-		msg_debug ("initialized trie of %ud elements",
-				url_scanner->matchers->len);
+	if (url_scanner != NULL) {
+		rspamd_url_deinit ();
 	}
+
+	url_scanner = g_malloc (sizeof (struct url_match_scanner));
+
+	if (tld_file) {
+		/* Reserve larger multipattern */
+		url_scanner->matchers = g_array_sized_new (FALSE, TRUE,
+				sizeof (struct url_matcher), 13000);
+		url_scanner->search_trie = rspamd_multipattern_create_sized (13000,
+				RSPAMD_MULTIPATTERN_TLD | RSPAMD_MULTIPATTERN_ICASE);
+	}
+	else {
+		url_scanner->matchers = g_array_sized_new (FALSE, TRUE,
+				sizeof (struct url_matcher), 128);
+		url_scanner->search_trie = rspamd_multipattern_create_sized (128,
+				RSPAMD_MULTIPATTERN_TLD | RSPAMD_MULTIPATTERN_ICASE);
+	}
+
+	rspamd_url_add_static_matchers (url_scanner);
+
+	if (tld_file != NULL) {
+		rspamd_url_parse_tld_file (tld_file, url_scanner);
+	}
+
+	if (!rspamd_multipattern_compile (url_scanner->search_trie, &err)) {
+		msg_err ("cannot compile tld patterns, url matching will be "
+				 "broken completely: %e", err);
+		g_error_free (err);
+	}
+
+	msg_debug ("initialized trie of %ud elements",
+			url_scanner->matchers->len);
 }
 
 #define SET_U(u, field) do {                                                \
@@ -1012,13 +1026,46 @@ rspamd_web_parse (struct http_parser_url *u, const gchar *str, gsize len,
 				break;
 			case parse_port_password:
 				if (g_ascii_isdigit (t)) {
-					/* XXX: that breaks urls with passwords starting with number */
-					st = parse_port;
-					c = slash;
-					p--;
-					SET_U (u, UF_HOST);
-					p++;
-					c = p;
+					const gchar *tmp = p;
+
+					while (tmp < last) {
+						if (!g_ascii_isdigit (*tmp)) {
+							if (*tmp == '/' || *tmp == '#' || *tmp == '?' ||
+									is_url_end (*tmp) || g_ascii_isspace (*tmp)) {
+								/* Port + something */
+								st = parse_port;
+								c = slash;
+								p--;
+								SET_U (u, UF_HOST);
+								p++;
+								c = p;
+								break;
+							}
+							else {
+								/* Not a port, bad character at the end */
+								break;
+							}
+						}
+						tmp ++;
+					}
+
+					if (tmp == last) {
+						/* Host + port only */
+						st = parse_port;
+						c = slash;
+						p--;
+						SET_U (u, UF_HOST);
+						p++;
+						c = p;
+					}
+
+					if (st != parse_port) {
+						/* Fallback to user:password */
+						p = slash;
+						c = slash;
+						user_seen = TRUE;
+						st = parse_user;
+					}
 				}
 				else {
 					/* Rewind back */
@@ -1605,7 +1652,7 @@ rspamd_url_parse (struct rspamd_url *uri, gchar *uristring, gsize len,
 		memcpy (uri->string + u.field_data[UF_SCHEMA].len, "://", 3);
 		rspamd_strlcpy (uri->string + u.field_data[UF_SCHEMA].len + 3,
 			p + u.field_data[UF_SCHEMA].len + 1,
-				len - 1 - u.field_data[UF_SCHEMA].len);
+				len - 2 - u.field_data[UF_SCHEMA].len);
 		/* Compensate slashes added */
 		for (i = UF_SCHEMA + 1; i < UF_MAX; i++) {
 			if (u.field_set & (1 << i)) {
@@ -2577,7 +2624,7 @@ rspamd_url_text_extract (rspamd_mempool_t *pool,
 {
 	struct rspamd_url_mimepart_cbdata mcbd;
 
-	if (part->stripped_content == NULL || part->stripped_content->len == 0) {
+	if (part->utf_stripped_content == NULL || part->utf_stripped_content->len == 0) {
 		msg_warn_task ("got empty text part");
 		return;
 	}
@@ -2585,8 +2632,8 @@ rspamd_url_text_extract (rspamd_mempool_t *pool,
 	mcbd.task = task;
 	mcbd.part = part;
 
-	rspamd_url_find_multiple (task->task_pool, part->stripped_content->data,
-			part->stripped_content->len, is_html, part->newlines,
+	rspamd_url_find_multiple (task->task_pool, part->utf_stripped_content->data,
+			part->utf_stripped_content->len, is_html, part->newlines,
 			rspamd_url_text_part_callback, &mcbd);
 }
 
