@@ -105,6 +105,7 @@ rspamd_stat_init (struct rspamd_config *cfg, struct event_base *ev_base)
 	const gchar *cache_name = NULL;
 	lua_State *L = cfg->lua_state;
 	guint lua_classifiers_cnt = 0, i;
+	gboolean skip_cache = FALSE;
 
 	if (stat_ctx == NULL) {
 		stat_ctx = g_malloc0 (sizeof (*stat_ctx));
@@ -217,17 +218,22 @@ rspamd_stat_init (struct rspamd_config *cfg, struct event_base *ev_base)
 			cache_obj = ucl_object_lookup (clf->opts, "cache");
 			cache_name_obj = NULL;
 
-			if (cache_obj) {
-				cache_name_obj = ucl_object_lookup_any (cache_obj,
-						"name", "type", NULL);
+			if (cache_obj && ucl_object_type (cache_obj) == UCL_NULL) {
+				skip_cache = TRUE;
 			}
+			else {
+				if (cache_obj) {
+					cache_name_obj = ucl_object_lookup_any (cache_obj,
+							"name", "type", NULL);
+				}
 
-			if (cache_name_obj) {
-				cache_name = ucl_object_tostring (cache_name_obj);
+				if (cache_name_obj) {
+					cache_name = ucl_object_tostring (cache_name_obj);
+				}
 			}
 		}
 
-		if (cache_name == NULL) {
+		if (cache_name == NULL && !skip_cache) {
 			/* We assume that learn cache is the same as backend */
 			cache_name = clf->backend;
 		}
@@ -252,7 +258,7 @@ rspamd_stat_init (struct rspamd_config *cfg, struct event_base *ev_base)
 			}
 
 			/* XXX: bad hack to pass statfiles configuration to cache */
-			if (cl->cache == NULL) {
+			if (cl->cache == NULL && !skip_cache) {
 				cl->cache = rspamd_stat_get_cache (cache_name);
 				g_assert (cl->cache != NULL);
 				cl->cachecf = cl->cache->init (stat_ctx, cfg, st, cache_obj);
@@ -475,17 +481,23 @@ rspamd_stat_ctx_register_async (rspamd_stat_async_handler handler,
 	elt->ud = d;
 	elt->timeout = timeout;
 	/* Enabled by default */
-	elt->enabled = TRUE;
 
-	event_set (&elt->timer_ev, -1, EV_TIMEOUT, rspamd_async_elt_on_timer, elt);
-	event_base_set (st_ctx->ev_base, &elt->timer_ev);
-	/*
-	 * First we set timeval to zero as we want cb to be executed as
-	 * fast as possible
-	 */
-	elt->tv.tv_sec = 0;
-	elt->tv.tv_usec = 0;
-	event_add (&elt->timer_ev, &elt->tv);
+
+	if (st_ctx->ev_base) {
+		elt->enabled = TRUE;
+		event_set (&elt->timer_ev, -1, EV_TIMEOUT, rspamd_async_elt_on_timer, elt);
+		event_base_set (st_ctx->ev_base, &elt->timer_ev);
+		/*
+		 * First we set timeval to zero as we want cb to be executed as
+		 * fast as possible
+		 */
+		elt->tv.tv_sec = 0;
+		elt->tv.tv_usec = 0;
+		event_add (&elt->timer_ev, &elt->tv);
+	}
+	else {
+		elt->enabled = FALSE;
+	}
 
 	g_queue_push_tail (st_ctx->async_elts, elt);
 
