@@ -81,6 +81,7 @@ rspamd_stat_tokenize_parts_metadata (struct rspamd_stat_ctx *st_ctx,
 	gchar tmpbuf[128];
 	lua_State *L = task->cfg->lua_state;
 	const gchar *headers_hash;
+	struct rspamd_mime_header *hdr;
 
 	ar = g_array_sized_new (FALSE, FALSE, sizeof (elt), 16);
 	elt.flags = RSPAMD_STAT_TOKEN_FLAG_META;
@@ -183,6 +184,19 @@ rspamd_stat_tokenize_parts_metadata (struct rspamd_stat_ctx *st_ctx,
 		g_array_append_val (ar, elt);
 	}
 
+	/* Use more precise headers order */
+	cur = g_list_first (task->headers_order->head);
+	while (cur) {
+		hdr = cur->data;
+
+		if (hdr->name && hdr->type != RSPAMD_HEADER_RECEIVED) {
+			elt.begin = hdr->name;
+			elt.len = strlen (hdr->name);
+			g_array_append_val (ar, elt);
+		}
+
+		cur = g_list_next (cur);
+	}
 
 	/* Use metatokens plugin from Lua */
 	lua_getglobal (L, "rspamd_plugins");
@@ -293,7 +307,7 @@ rspamd_stat_tokenize_parts_metadata (struct rspamd_stat_ctx *st_ctx,
 /*
  * Tokenize task using the tokenizer specified
  */
-static void
+void
 rspamd_stat_process_tokenize (struct rspamd_stat_ctx *st_ctx,
 		struct rspamd_task *task)
 {
@@ -308,11 +322,17 @@ rspamd_stat_process_tokenize (struct rspamd_stat_ctx *st_ctx,
 	guchar hout[rspamd_cryptobox_HASHBYTES];
 	gchar *b32_hout;
 
+	if (st_ctx == NULL) {
+		st_ctx = rspamd_stat_get_ctx ();
+	}
+
+	g_assert (st_ctx != NULL);
+
 	for (i = 0; i < task->text_parts->len; i++) {
 		part = g_ptr_array_index (task->text_parts, i);
 
-		if (!IS_PART_EMPTY (part) && part->normalized_words != NULL) {
-			reserved_len += part->normalized_words->len;
+		if (!IS_PART_EMPTY (part) && part->utf_words != NULL) {
+			reserved_len += part->utf_words->len;
 		}
 		/* XXX: normal window size */
 		reserved_len += 5;
@@ -326,9 +346,9 @@ rspamd_stat_process_tokenize (struct rspamd_stat_ctx *st_ctx,
 	for (i = 0; i < task->text_parts->len; i ++) {
 		part = g_ptr_array_index (task->text_parts, i);
 
-		if (!IS_PART_EMPTY (part) && part->normalized_words != NULL) {
+		if (!IS_PART_EMPTY (part) && part->utf_words != NULL) {
 			st_ctx->tokenizer->tokenize_func (st_ctx, task->task_pool,
-					part->normalized_words, IS_PART_UTF (part),
+					part->utf_words, IS_PART_UTF (part),
 					NULL, task->tokens);
 		}
 
@@ -345,8 +365,18 @@ rspamd_stat_process_tokenize (struct rspamd_stat_ctx *st_ctx,
 	}
 
 	if (sub != NULL) {
-		words = rspamd_tokenize_text (sub, strlen (sub), TRUE, NULL, NULL, FALSE,
-				NULL);
+		UText utxt = UTEXT_INITIALIZER;
+		UErrorCode uc_err = U_ZERO_ERROR;
+		gsize slen = strlen (sub);
+
+		utext_openUTF8 (&utxt,
+				sub,
+				slen,
+				&uc_err);
+
+		words = rspamd_tokenize_text (sub, slen, &utxt, RSPAMD_TOKENIZE_UTF,
+				NULL, NULL, NULL);
+
 		if (words != NULL) {
 
 			for (i = 0; i < words->len; i ++) {
@@ -364,6 +394,8 @@ rspamd_stat_process_tokenize (struct rspamd_stat_ctx *st_ctx,
 			rspamd_mempool_add_destructor (task->task_pool,
 					rspamd_array_free_hard, words);
 		}
+
+		utext_close (&utxt);
 	}
 
 	rspamd_stat_tokenize_parts_metadata (st_ctx, task);
@@ -395,7 +427,10 @@ rspamd_stat_preprocess (struct rspamd_stat_ctx *st_ctx,
 	struct rspamd_statfile *st;
 	gpointer bk_run;
 
-	rspamd_stat_process_tokenize (st_ctx, task);
+	if (task->tokens == NULL) {
+		rspamd_stat_process_tokenize (st_ctx, task);
+	}
+
 	task->stat_runtimes = g_ptr_array_sized_new (st_ctx->statfiles->len);
 	g_ptr_array_set_size (task->stat_runtimes, st_ctx->statfiles->len);
 	rspamd_mempool_add_destructor (task->task_pool,
@@ -505,12 +540,12 @@ rspamd_stat_classifiers_process (struct rspamd_stat_ctx *st_ctx,
 	 * Do not classify a message if some class is missing
 	 */
 	if (!(task->flags & RSPAMD_TASK_FLAG_HAS_SPAM_TOKENS)) {
-		msg_warn_task ("skip statistics as SPAM class is missing");
+		msg_info_task ("skip statistics as SPAM class is missing");
 
 		return;
 	}
 	if (!(task->flags & RSPAMD_TASK_FLAG_HAS_HAM_TOKENS)) {
-		msg_warn_task ("skip statistics as HAM class is missing");
+		msg_info_task ("skip statistics as HAM class is missing");
 
 		return;
 	}
@@ -1070,7 +1105,7 @@ rspamd_stat_has_classifier_symbols (struct rspamd_task *task,
 		id = g_array_index (cl->statfiles_ids, gint, i);
 		st = g_ptr_array_index (st_ctx->statfiles, id);
 
-		if (g_hash_table_lookup (mres->symbols, st->stcf->symbol)) {
+		if (rspamd_task_find_symbol_result (task, st->stcf->symbol)) {
 			if (is_spam == !!st->stcf->is_spam) {
 				msg_debug_task ("do not autolearn %s as symbol %s is already "
 						"added", is_spam ? "spam" : "ham", st->stcf->symbol);
