@@ -349,7 +349,6 @@ rspamd_parse_unix_path (rspamd_inet_addr_t **target, const char *src)
 	bool has_group = false;
 
 	tokens = g_strsplit_set (src, " ,", -1);
-
 	addr = rspamd_inet_addr_create (AF_UNIX);
 
 	rspamd_strlcpy (addr->u.un->addr.sun_path, tokens[0],
@@ -422,6 +421,7 @@ rspamd_parse_unix_path (rspamd_inet_addr_t **target, const char *src)
 	}
 
 	g_free (pwbuf);
+	g_strfreev (tokens);
 
 	if (target) {
 		rspamd_ip_validate_af (addr);
@@ -435,6 +435,7 @@ rspamd_parse_unix_path (rspamd_inet_addr_t **target, const char *src)
 
 err:
 
+	g_strfreev (tokens);
 	g_free (pwbuf);
 	rspamd_inet_address_free (addr);
 	return FALSE;
@@ -492,7 +493,7 @@ gboolean
 rspamd_parse_inet_address_ip6 (const guchar *text, gsize len, gpointer target)
 {
 	guchar t, *zero = NULL, *s, *d,  *addr = target;
-	const guchar *p, *digit = NULL;
+	const guchar *p, *digit = NULL, *percent;
 	gsize len4 = 0;
 	guint n = 8, nibbles = 0, word = 0;
 
@@ -512,6 +513,11 @@ rspamd_parse_inet_address_ip6 (const guchar *text, gsize len, gpointer target)
 		p = text;
 	}
 
+	/* Check IPv6 scope */
+	if ((percent = memchr (p, '%', len)) != NULL && percent > p) {
+		len = percent - p; /* Ignore scope */
+	}
+
 	for (/* void */; len; len--) {
 		t = *p++;
 
@@ -1752,9 +1758,7 @@ rspamd_inet_library_init (void)
 void
 rspamd_inet_library_destroy (void)
 {
-	if (local_addrs != NULL) {
-		rspamd_map_helper_destroy_radix (local_addrs);
-	}
+	/* Ugly: local_addrs will actually be freed by config object */
 }
 
 gsize
