@@ -21,7 +21,7 @@ struct f_str_s;
 
 #define MEMPOOL_TAG_LEN 20
 #define MEMPOOL_UID_LEN 20
-#define MEM_ALIGNMENT   8
+#define MEM_ALIGNMENT   16
 #define align_ptr(p, a)                                                   \
     (guint8 *) (((uintptr_t) (p) + ((uintptr_t) a - 1)) & ~((uintptr_t) a - 1))
 
@@ -223,6 +223,13 @@ void rspamd_mempool_add_destructor_full (rspamd_mempool_t *pool,
 void rspamd_mempool_replace_destructor (rspamd_mempool_t *pool,
 	rspamd_mempool_destruct_t func, void *old_data, void *new_data);
 
+/**
+ * Calls all destructors associated with the specific memory pool without removing
+ * of the pool itself
+ * @param pool
+ */
+void rspamd_mempool_destructors_enforce (rspamd_mempool_t *pool);
+
 /**
  * Delete pool, free all its chunks and call destructors chain
  * @param pool memory pool object
