@@ -330,7 +330,7 @@ rspamd_gstring_icase_hash (gconstpointer key)
 #define HASZERO(x) ~(((((x) & ZEROMASK) + ZEROMASK) | (x)) | ZEROMASK)
 
 gsize
-rspamd_strlcpy (gchar *dst, const gchar *src, gsize siz)
+rspamd_strlcpy_fast (gchar *dst, const gchar *src, gsize siz)
 {
 	gchar *d = dst;
 	const gchar *s = src;
@@ -371,6 +371,30 @@ rspamd_strlcpy (gchar *dst, const gchar *src, gsize siz)
 	return (d - dst);
 }
 
+size_t
+rspamd_strlcpy_safe (gchar *dst, const gchar *src, gsize siz)
+{
+	gchar *d = dst;
+	gsize nleft = siz;
+
+	if (nleft != 0) {
+		while (--nleft != 0) {
+			if ((*d++ = *src++) == '\0') {
+				d --;
+				break;
+			}
+		}
+	}
+
+	if (nleft == 0) {
+		if (siz != 0) {
+			*d = '\0';
+		}
+	}
+
+	return (d - dst);
+}
+
 /*
  * Try to convert string of length to long
  */
@@ -866,9 +890,8 @@ rspamd_strings_levenshtein_distance (const gchar *s1, gsize s1len,
 		const gchar *s2, gsize s2len,
 		guint replace_cost)
 {
-	guint x, y, lastdiag, olddiag;
-	gchar c1, c2;
-	guint *column;
+	gchar c1, c2, last_c2, last_c1;
+	static GArray *current_row = NULL, *prev_row = NULL, *transp_row = NULL;
 	gint eq;
 	static const guint max_cmp = 8192;
 	gint ret;
@@ -885,31 +908,79 @@ rspamd_strings_levenshtein_distance (const gchar *s1, gsize s1len,
 
 	if (MAX(s1len, s2len) > max_cmp) {
 		/* Cannot compare too many characters */
-		return 0;
+		return max_cmp;
 	}
 
-	column = g_malloc0 ((s1len + 1) * sizeof (guint));
+	if (s1len > s2len) {
+		/* Exchange s1 and s2 */
+		const gchar *tmp;
+		gsize tmplen;
 
-	for (y = 1; y <= s1len; y++) {
-		column[y] = y;
+		tmp = s2;
+		s2 = s1;
+		s1 = tmp;
+
+		tmplen = s2len;
+		s2len = s1len;
+		s1len = tmplen;
 	}
 
-	for (x = 1; x <= s2len; x++) {
-		column[0] = x;
+	/* Adjust static space */
+	if (current_row == NULL) {
+		current_row = g_array_sized_new (FALSE, FALSE, sizeof (gint), s1len + 1);
+		prev_row = g_array_sized_new (FALSE, FALSE, sizeof (gint), s1len + 1);
+		transp_row = g_array_sized_new (FALSE, FALSE, sizeof (gint), s1len + 1);
+		g_array_set_size (current_row, s1len + 1);
+		g_array_set_size (prev_row, s1len + 1);
+		g_array_set_size (transp_row, s1len + 1);
+	}
+	else if (current_row->len < s1len + 1) {
+		g_array_set_size (current_row, s1len + 1);
+		g_array_set_size (prev_row, s1len + 1);
+		g_array_set_size (transp_row, s1len + 1);
+	}
 
-		for (y = 1, lastdiag = x - 1; y <= s1len; y++) {
-			olddiag = column[y];
-			c1 = s1[y - 1];
-			c2 = s2[x - 1];
-			eq = (c1 == c2) ? 0 : replace_cost;
-			column[y] = MIN3 (column[y] + 1, column[y - 1] + 1,
-					lastdiag + (eq));
-			lastdiag = olddiag;
+	memset (current_row->data, 0, (s1len + 1) * sizeof (gint));
+	memset (transp_row->data, 0, (s1len + 1) * sizeof (gint));
+
+	for (gint i = 0; i <= s1len; i++) {
+		g_array_index (prev_row, gint, i) = i;
+	}
+
+	last_c2 = '\0';
+
+	for (gint i = 1; i <= s2len; i++) {
+		c2 = s2[i - 1];
+		g_array_index (current_row, gint, 0) = i;
+		last_c1 = '\0';
+
+		for (gint j = 1; j <= s1len; j++) {
+			c1 = s1[j - 1];
+			eq = c1 == c2 ? 0 : replace_cost;
+			ret = MIN3 (g_array_index (current_row, gint, j - 1) + 1, /* Insert */
+					g_array_index (prev_row, gint, j) + 1, /* Remove */
+					g_array_index (prev_row, gint, j - 1) + eq /* Replace */);
+
+			/* Take reordering into account */
+			if (c1 == last_c2 && c2 == last_c1 && j >= 2) {
+				ret = MIN (ret, g_array_index (transp_row, gint, j - 2) + eq);
+			}
+
+			g_array_index (current_row, gint, j) = ret;
+			last_c1 = c1;
 		}
+
+		last_c2 = c2;
+
+		/* Exchange pointers */
+		GArray *tmp;
+		tmp = transp_row;
+		transp_row = prev_row;
+		prev_row = current_row;
+		current_row = tmp;
 	}
 
-	ret = column[s1len];
-	g_free (column);
+	ret = g_array_index (prev_row, gint, s1len);
 
 	return ret;
 }
@@ -1214,7 +1285,7 @@ rspamd_substring_preprocess_kmp (const gchar *pat, gsize len, goffset *fsm,
 		i++;
 		j++;
 
-		if (f(pat[i], pat[j])) {
+		if (i < len && j < len && f(pat[i], pat[j])) {
 			fsm[i] = fsm[j];
 		}
 		else {
@@ -1341,10 +1412,7 @@ rspamd_substring_search_caseless (const gchar *in, gsize inlen,
 				rspamd_substring_casecmp_func);
 	}
 	else if (inlen == srchlen) {
-		return rspamd_lc_cmp (srch, in, srchlen) == 0;
-	}
-	else {
-		return (-1);
+		return rspamd_lc_cmp (srch, in, srchlen) == 0 ? 0 : (-1);
 	}
 
 	return (-1);
@@ -1408,6 +1476,10 @@ rspamd_string_find_eoh (GString *input, goffset *body_start)
 				p++;
 				state = got_lf;
 			}
+			else if (g_ascii_isspace (*p)) {
+				/* We have \r<space>*, allow to stay in this state */
+				p ++;
+			}
 			else {
 				p++;
 				state = skip_char;
@@ -1424,6 +1496,10 @@ rspamd_string_find_eoh (GString *input, goffset *body_start)
 			else if (*p == '\r') {
 				state = got_linebreak;
 			}
+			else if (g_ascii_isspace (*p)) {
+				/* We have \n<space>*, allow to stay in this state */
+				p ++;
+			}
 			else {
 				p++;
 				state = skip_char;
@@ -1440,6 +1516,10 @@ rspamd_string_find_eoh (GString *input, goffset *body_start)
 				p++;
 				state = got_linebreak_lf;
 			}
+			else if (g_ascii_isspace (*p)) {
+				/* We have <linebreak><space>*, allow to stay in this state */
+				p ++;
+			}
 			else {
 				p++;
 				state = skip_char;
@@ -1455,6 +1535,10 @@ rspamd_string_find_eoh (GString *input, goffset *body_start)
 				state = got_linebreak_lf;
 				p++;
 			}
+			else if (g_ascii_isspace (*p)) {
+				/* We have \r\n<space>*, allow to keep in this state */
+				p ++;
+			}
 			else {
 				p++;
 				state = skip_char;
@@ -1691,12 +1775,19 @@ decode:
 
 #define BITOP(a,b,op) \
 		((a)[(gsize)(b)/(8*sizeof *(a))] op (gsize)1<<((gsize)(b)%(8*sizeof *(a))))
+
+
 gsize
 rspamd_memcspn (const gchar *s, const gchar *e, gsize len)
 {
 	gsize byteset[32 / sizeof(gsize)];
 	const gchar *p = s, *end = s + len;
 
+	if (!e[1]) {
+		for (; p < end && *p != *e; p++);
+		return p - s;
+	}
+
 	memset (byteset, 0, sizeof byteset);
 
 	for (; *e && BITOP (byteset, *(guchar *)e, |=); e++);
@@ -1705,6 +1796,25 @@ rspamd_memcspn (const gchar *s, const gchar *e, gsize len)
 	return p - s;
 }
 
+gsize
+rspamd_memspn (const gchar *s, const gchar *e, gsize len)
+{
+	gsize byteset[32 / sizeof(gsize)];
+	const gchar *p = s, *end = s + len;
+
+	if (!e[1]) {
+		for (; p < end && *p == *e; p++);
+		return p - s;
+	}
+
+	memset (byteset, 0, sizeof byteset);
+
+	for (; *e && BITOP (byteset, *(guchar *)e, |=); e++);
+	for (; p < end && BITOP (byteset, *(guchar *)p, &); p++);
+
+	return p - s;
+}
+
 gssize
 rspamd_decode_qp2047_buf (const gchar *in, gsize inlen,
 		gchar *out, gsize outlen)
