@@ -77,4 +77,8 @@
 #include <libgen.h>
 #endif
 
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 #endif
