@@ -505,14 +505,14 @@ LUA_FUNCTION_DEF (config, replace_regexp);
 LUA_FUNCTION_DEF (config, register_worker_script);
 
 /***
- * @method rspamd_config:add_on_load(function(cfg, ev_base) ... end)
+ * @method rspamd_config:add_on_load(function(cfg, ev_base, worker) ... end)
  * Registers the following script to be executed when configuration is completely loaded
  * @param {function} script function to be executed
  * @example
-rspamd_config:add_on_load(function(cfg, ev_base)
+rspamd_config:add_on_load(function(cfg, ev_base, worker)
 	rspamd_config:add_periodic(ev_base, 1.0, function(cfg, ev_base)
 		local logger = require "rspamd_logger"
-		logger.infox(cfg, "periodic function")
+		logger.infox(cfg, "periodic function in worker %s", worker:get_name())
 		return true
 	end)
 end)
@@ -1739,14 +1739,21 @@ lua_config_register_symbol (lua_State * L)
 				FALSE,
 				no_squeeze);
 
-		if (!isnan (score)) {
+		if (!isnan (score) || group) {
 			if (one_shot) {
 				nshots = 1;
 			}
 
-			rspamd_config_add_symbol (cfg, name,
-					score, description, group, flags,
-					(guint) priority, nshots);
+			if (!isnan (score)) {
+				rspamd_config_add_symbol (cfg, name,
+						score, description, group, flags,
+						(guint) priority, nshots);
+			}
+			else {
+				rspamd_config_add_symbol (cfg, name,
+						0.0, description, group, flags,
+						(guint) priority, nshots);
+			}
 
 			lua_pushstring (L, "groups");
 			lua_gettable (L, 2);
@@ -2385,7 +2392,7 @@ lua_config_newindex (lua_State *L)
 		}
 		else if (lua_type (L, 3) == LUA_TTABLE) {
 			gint type = SYMBOL_TYPE_NORMAL, priority = 0, idx;
-			gdouble weight = 1.0, score;
+			gdouble weight = 1.0, score = NAN;
 			const char *type_str, *group = NULL, *description = NULL;
 			guint flags = 0;
 
@@ -2500,22 +2507,23 @@ lua_config_newindex (lua_State *L)
 			 */
 			if (g_hash_table_lookup (cfg->symbols, name) == NULL) {
 				nshots = cfg->default_max_shots;
+
 				lua_pushstring (L, "score");
 				lua_gettable (L, -2);
-
 				if (lua_type (L, -1) == LUA_TNUMBER) {
 					score = lua_tonumber (L, -1);
-					lua_pop (L, 1);
 
-					/* If score defined, then we can check other metric fields */
-					lua_pushstring (L, "group");
-					lua_gettable (L, -2);
+				}
+				lua_pop (L, 1);
 
-					if (lua_type (L, -1) == LUA_TSTRING) {
-						group = lua_tostring (L, -1);
-					}
-					lua_pop (L, 1);
+				lua_pushstring (L, "group");
+				lua_gettable (L, -2);
+				if (lua_type (L, -1) == LUA_TSTRING) {
+					group = lua_tostring (L, -1);
+				}
+				lua_pop (L, 1);
 
+				if (!isnan (score) || group != NULL) {
 					lua_pushstring (L, "description");
 					lua_gettable (L, -2);
 
@@ -2548,8 +2556,15 @@ lua_config_newindex (lua_State *L)
 					 * Do not override the existing symbols (using zero priority),
 					 * since we are defining default values here
 					 */
-					rspamd_config_add_symbol (cfg, name, score,
-							description, group, flags, 0, nshots);
+					if (!isnan (score)) {
+						rspamd_config_add_symbol (cfg, name, score,
+								description, group, flags, 0, nshots);
+					}
+					else if (group) {
+						/* Add with zero score */
+						rspamd_config_add_symbol (cfg, name, 0.0,
+								description, group, flags, 0, nshots);
+					}
 
 					lua_pushstring (L, "groups");
 					lua_gettable (L, -2);
@@ -2568,9 +2583,6 @@ lua_config_newindex (lua_State *L)
 
 					lua_pop (L, 1);
 				}
-				else {
-					lua_pop (L, 1);
-				}
 			}
 
 			/* Remove table from stack */
@@ -2861,7 +2873,6 @@ lua_periodic_callback (gint unused_fd, short what, gpointer ud)
 	*pev_base = periodic->ev_base;
 
 	event_del (&periodic->ev);
-
 	lua_thread_call (thread, 2);
 }
 
@@ -2876,6 +2887,10 @@ lua_periodic_callback_finish (struct thread_entry *thread, int ret)
 
 	L = thread->lua_state;
 
+#ifdef HAVE_EVENT_NO_CACHE_TIME_FUNC
+	event_base_update_cache_time (periodic->ev_base);
+#endif
+
 	if (ret == 0) {
 		if (lua_type (L, -1) == LUA_TBOOLEAN) {
 			plan_more = lua_toboolean (L, -1);
@@ -3441,6 +3456,7 @@ lua_config_load_ucl (lua_State *L)
 
 		if (lua_istable (L, -1)) {
 			LUA_TABLE_TO_HASH(paths, RSPAMD_CONFDIR_INDEX);
+			LUA_TABLE_TO_HASH(paths, RSPAMD_LOCAL_CONFDIR_INDEX);
 			LUA_TABLE_TO_HASH(paths, RSPAMD_RUNDIR_INDEX);
 			LUA_TABLE_TO_HASH(paths, RSPAMD_DBDIR_INDEX);
 			LUA_TABLE_TO_HASH(paths, RSPAMD_LOGDIR_INDEX);
