@@ -27,6 +27,9 @@
  */
 
 #include "lua_common.h"
+#include "libcryptobox/cryptobox.h"
+#include "libcryptobox/keypair.h"
+#include "libcryptobox/keypair_private.h"
 #include "unix-std.h"
 
 struct rspamd_lua_cryptobox_hash {
@@ -42,6 +45,10 @@ LUA_FUNCTION_DEF (cryptobox_pubkey,	 gc);
 LUA_FUNCTION_DEF (cryptobox_keypair,	 load);
 LUA_FUNCTION_DEF (cryptobox_keypair,	 create);
 LUA_FUNCTION_DEF (cryptobox_keypair,	 gc);
+LUA_FUNCTION_DEF (cryptobox_keypair,	 totable);
+LUA_FUNCTION_DEF (cryptobox_keypair,	 get_type);
+LUA_FUNCTION_DEF (cryptobox_keypair,	 get_alg);
+LUA_FUNCTION_DEF (cryptobox_keypair,	 get_pk);
 LUA_FUNCTION_DEF (cryptobox_signature, create);
 LUA_FUNCTION_DEF (cryptobox_signature, load);
 LUA_FUNCTION_DEF (cryptobox_signature, save);
@@ -60,16 +67,24 @@ LUA_FUNCTION_DEF (cryptobox_hash, base32);
 LUA_FUNCTION_DEF (cryptobox_hash, base64);
 LUA_FUNCTION_DEF (cryptobox_hash, bin);
 LUA_FUNCTION_DEF (cryptobox_hash, gc);
-LUA_FUNCTION_DEF (cryptobox,			 verify_memory);
-LUA_FUNCTION_DEF (cryptobox,			 verify_file);
-LUA_FUNCTION_DEF (cryptobox,			 sign_file);
-LUA_FUNCTION_DEF (cryptobox,			 sign_memory);
+LUA_FUNCTION_DEF (cryptobox, verify_memory);
+LUA_FUNCTION_DEF (cryptobox, verify_file);
+LUA_FUNCTION_DEF (cryptobox, sign_file);
+LUA_FUNCTION_DEF (cryptobox, sign_memory);
+LUA_FUNCTION_DEF (cryptobox, encrypt_memory);
+LUA_FUNCTION_DEF (cryptobox, encrypt_file);
+LUA_FUNCTION_DEF (cryptobox, decrypt_memory);
+LUA_FUNCTION_DEF (cryptobox, decrypt_file);
 
 static const struct luaL_reg cryptoboxlib_f[] = {
 	LUA_INTERFACE_DEF (cryptobox, verify_memory),
 	LUA_INTERFACE_DEF (cryptobox, verify_file),
 	LUA_INTERFACE_DEF (cryptobox, sign_memory),
 	LUA_INTERFACE_DEF (cryptobox, sign_file),
+	LUA_INTERFACE_DEF (cryptobox, encrypt_memory),
+	LUA_INTERFACE_DEF (cryptobox, encrypt_file),
+	LUA_INTERFACE_DEF (cryptobox, decrypt_memory),
+	LUA_INTERFACE_DEF (cryptobox, decrypt_file),
 	{NULL, NULL}
 };
 
@@ -93,6 +108,13 @@ static const struct luaL_reg cryptoboxkeypairlib_f[] = {
 
 static const struct luaL_reg cryptoboxkeypairlib_m[] = {
 	{"__tostring", rspamd_lua_class_tostring},
+	{"totable", lua_cryptobox_keypair_totable},
+	{"get_type", lua_cryptobox_keypair_get_type},
+	{"get_alg", lua_cryptobox_keypair_get_alg},
+	{"type", lua_cryptobox_keypair_get_type},
+	{"alg", lua_cryptobox_keypair_get_alg},
+	{"pk", lua_cryptobox_keypair_get_pk},
+	{"pubkey", lua_cryptobox_keypair_get_pk},
 	{"__gc", lua_cryptobox_keypair_gc},
 	{NULL, NULL}
 };
@@ -181,6 +203,7 @@ lua_check_cryptobox_hash (lua_State * L, int pos)
 static gint
 lua_cryptobox_pubkey_load (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_pubkey *pkey = NULL, **ppkey;
 	const gchar *filename, *arg;
 	gint type = RSPAMD_KEYPAIR_SIGN;
@@ -256,6 +279,7 @@ lua_cryptobox_pubkey_load (lua_State *L)
 static gint
 lua_cryptobox_pubkey_create (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_pubkey *pkey = NULL, **ppkey;
 	const gchar *buf, *arg;
 	gsize len;
@@ -310,6 +334,7 @@ lua_cryptobox_pubkey_create (lua_State *L)
 static gint
 lua_cryptobox_pubkey_gc (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_pubkey *pkey = lua_check_cryptobox_pubkey (L, 1);
 
 	if (pkey != NULL) {
@@ -320,65 +345,15 @@ lua_cryptobox_pubkey_gc (lua_State *L)
 }
 
 /***
- * @function rspamd_cryptobox_keypair.load(file)
- * Loads public key from UCL file
+ * @function rspamd_cryptobox_keypair.load(file|table)
+ * Loads public key from UCL file or directly from Lua
  * @param {string} file filename to load
  * @return {cryptobox_keypair} new keypair
  */
 static gint
 lua_cryptobox_keypair_load (lua_State *L)
 {
-	struct rspamd_cryptobox_keypair *kp, **pkp;
-	const gchar *filename;
-	struct ucl_parser *parser;
-	ucl_object_t *obj;
-
-	filename = luaL_checkstring (L, 1);
-	if (filename != NULL) {
-		parser = ucl_parser_new (0);
-
-		if (!ucl_parser_add_file (parser, filename)) {
-			msg_err ("cannot open keypair from file: %s, %s",
-				filename,
-				ucl_parser_get_error (parser));
-			ucl_parser_free (parser);
-			lua_pushnil (L);
-		}
-		else {
-			obj = ucl_parser_get_object (parser);
-			kp = rspamd_keypair_from_ucl (obj);
-			ucl_parser_free (parser);
-
-			if (kp == NULL) {
-				msg_err ("cannot open keypair from file: %s",
-						filename);
-				ucl_object_unref (obj);
-				lua_pushnil (L);
-			}
-			else {
-				pkp = lua_newuserdata (L, sizeof (gpointer));
-				*pkp = kp;
-				rspamd_lua_setclass (L, "rspamd{cryptobox_keypair}", -1);
-				ucl_object_unref (obj);
-			}
-		}
-	}
-	else {
-		return luaL_error (L, "bad input arguments");
-	}
-
-	return 1;
-}
-
-/***
- * @function rspamd_cryptobox_keypair.create(ucl_data)
- * Loads public key from UCL data
- * @param {string} ucl_data ucl to load
- * @return {cryptobox_keypair} new keypair
- */
-static gint
-lua_cryptobox_keypair_create (lua_State *L)
-{
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_keypair *kp, **pkp;
 	const gchar *buf;
 	gsize len;
@@ -439,9 +414,62 @@ lua_cryptobox_keypair_create (lua_State *L)
 	return 1;
 }
 
+/***
+ * @function rspamd_cryptobox_keypair.create([type='encryption'[, alg='curve25519']])
+ * Generates new keypair
+ * @param {string} type type of keypair: 'encryption' (default) or 'sign'
+ * @param {string} alg algorithm of keypair: 'curve25519' (default) or 'nist'
+ * @return {cryptobox_keypair} new keypair
+ */
+static gint
+lua_cryptobox_keypair_create (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp, **pkp;
+	enum rspamd_cryptobox_keypair_type type = RSPAMD_KEYPAIR_KEX;
+	enum rspamd_cryptobox_mode alg = RSPAMD_CRYPTOBOX_MODE_25519;
+
+	if (lua_isstring (L, 1)) {
+		const gchar *str = lua_tostring (L, 1);
+
+		if (strcmp (str, "sign") == 0) {
+			type = RSPAMD_KEYPAIR_SIGN;
+		}
+		else if (strcmp (str, "encryption") == 0) {
+			type = RSPAMD_KEYPAIR_KEX;
+		}
+		else {
+			return luaL_error (L, "invalid keypair type: %s", str);
+		}
+	}
+
+	if (lua_isstring (L, 2)) {
+		const gchar *str = lua_tostring (L, 2);
+
+		if (strcmp (str, "nist") == 0 || strcmp (str, "openssl") == 0) {
+			alg = RSPAMD_CRYPTOBOX_MODE_NIST;
+		}
+		else if (strcmp (str, "curve25519") == 0 || strcmp (str, "default") == 0) {
+			alg = RSPAMD_CRYPTOBOX_MODE_25519;
+		}
+		else {
+			return luaL_error (L, "invalid keypair algorithm: %s", str);
+		}
+	}
+
+	kp = rspamd_keypair_new (type, alg);
+
+	pkp = lua_newuserdata (L, sizeof (gpointer));
+	*pkp = kp;
+	rspamd_lua_setclass (L, "rspamd{cryptobox_keypair}", -1);
+
+	return 1;
+}
+
 static gint
 lua_cryptobox_keypair_gc (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_keypair *kp = lua_check_cryptobox_keypair (L, 1);
 
 	if (kp != NULL) {
@@ -452,7 +480,122 @@ lua_cryptobox_keypair_gc (lua_State *L)
 }
 
 /***
- * @function rspamd_cryptobox_signature.load(file)
+ * @method keypair:totable([hex=false]])
+ * Converts keypair to table (not very safe due to memory leftovers)
+ */
+static gint
+lua_cryptobox_keypair_totable (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp = lua_check_cryptobox_keypair (L, 1);
+	ucl_object_t *obj;
+	gboolean hex = FALSE;
+	gint ret = 1;
+
+	if (kp != NULL) {
+
+		if (lua_isboolean (L, 2)) {
+			hex = lua_toboolean (L, 2);
+		}
+
+		obj = rspamd_keypair_to_ucl (kp, hex);
+
+		ret = ucl_object_push_lua (L, obj, true);
+		ucl_object_unref (obj);
+	}
+	else {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	return ret;
+}
+/***
+ * @method keypair:type()
+ * Returns type of keypair as a string: 'encryption' or 'sign'
+ * @return {string} type of keypair as a string
+ */
+static gint
+lua_cryptobox_keypair_get_type (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp = lua_check_cryptobox_keypair (L, 1);
+
+	if (kp) {
+		if (kp->type == RSPAMD_KEYPAIR_KEX) {
+			lua_pushstring (L, "encryption");
+		}
+		else {
+			lua_pushstring (L, "sign");
+		}
+	}
+	else {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	return 1;
+}
+
+/***
+ * @method keypair:alg()
+ * Returns algorithm of keypair as a string: 'encryption' or 'sign'
+ * @return {string} type of keypair as a string
+ */
+static gint
+lua_cryptobox_keypair_get_alg (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp = lua_check_cryptobox_keypair (L, 1);
+
+	if (kp) {
+		if (kp->alg == RSPAMD_CRYPTOBOX_MODE_25519) {
+			lua_pushstring (L, "curve25519");
+		}
+		else {
+			lua_pushstring (L, "nist");
+		}
+	}
+	else {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	return 1;
+}
+
+/***
+ * @method keypair:pk()
+ * Returns pubkey for a specific keypair
+ * @return {rspamd_pubkey} pubkey for a keypair
+ */
+static gint
+lua_cryptobox_keypair_get_pk (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp = lua_check_cryptobox_keypair (L, 1);
+	struct rspamd_cryptobox_pubkey *pk, **ppk;
+	const guchar *data;
+	guint dlen;
+
+	if (kp) {
+		data = rspamd_keypair_component (kp, RSPAMD_KEYPAIR_COMPONENT_PK, &dlen);
+		pk = rspamd_pubkey_from_bin (data, dlen, kp->type, kp->alg);
+
+		if (pk == NULL) {
+			return luaL_error (L, "invalid keypair");
+		}
+
+		ppk = lua_newuserdata (L, sizeof (*ppk));
+		*ppk = pk;
+		rspamd_lua_setclass (L, "rspamd{cryptobox_pubkey}", -1);
+	}
+	else {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	return 1;
+}
+
+/***
+ * @function rspamd_cryptobox_signature.load(file, [alg = 'curve25519'])
  * Loads signature from raw file
  * @param {string} file filename to load
  * @return {cryptobox_signature} new signature
@@ -460,11 +603,13 @@ lua_cryptobox_keypair_gc (lua_State *L)
 static gint
 lua_cryptobox_signature_load (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig, **psig;
 	const gchar *filename;
 	gpointer data;
 	int fd;
 	struct stat st;
+	enum rspamd_cryptobox_mode alg = RSPAMD_CRYPTOBOX_MODE_25519;
 
 	filename = luaL_checkstring (L, 1);
 	if (filename != NULL) {
@@ -475,7 +620,6 @@ lua_cryptobox_signature_load (lua_State *L)
 			lua_pushnil (L);
 		}
 		else {
-			sig = g_malloc (sizeof (rspamd_fstring_t));
 			if (fstat (fd, &st) == -1 ||
 				(data =
 				mmap (NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0))
@@ -484,8 +628,20 @@ lua_cryptobox_signature_load (lua_State *L)
 				lua_pushnil (L);
 			}
 			else {
-				if (st.st_size == rspamd_cryptobox_signature_bytes (
-						RSPAMD_CRYPTOBOX_MODE_25519)) {
+				if (lua_isstring (L, 2)) {
+					const gchar *str = lua_tostring (L, 2);
+
+					if (strcmp (str, "nist") == 0 || strcmp (str, "openssl") == 0) {
+						alg = RSPAMD_CRYPTOBOX_MODE_NIST;
+					}
+					else if (strcmp (str, "curve25519") == 0 || strcmp (str, "default") == 0) {
+						alg = RSPAMD_CRYPTOBOX_MODE_25519;
+					}
+					else {
+						return luaL_error (L, "invalid keypair algorithm: %s", str);
+					}
+				}
+				if (st.st_size > 0) {
 					sig = rspamd_fstring_new_init (data, st.st_size);
 					psig = lua_newuserdata (L, sizeof (rspamd_fstring_t *));
 					rspamd_lua_setclass (L, "rspamd{cryptobox_signature}", -1);
@@ -494,7 +650,7 @@ lua_cryptobox_signature_load (lua_State *L)
 				else {
 					msg_err ("size of %s mismatches: %d while %d is expected",
 							filename, (int)st.st_size,
-							rspamd_cryptobox_signature_bytes (RSPAMD_CRYPTOBOX_MODE_25519));
+							rspamd_cryptobox_signature_bytes (alg));
 					lua_pushnil (L);
 				}
 
@@ -519,6 +675,7 @@ lua_cryptobox_signature_load (lua_State *L)
 static gint
 lua_cryptobox_signature_save (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig;
 	gint fd, flags;
 	const gchar *filename;
@@ -582,6 +739,7 @@ lua_cryptobox_signature_save (lua_State *L)
 static gint
 lua_cryptobox_signature_create (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig, **psig;
 	struct rspamd_lua_text *t;
 	const gchar *data;
@@ -624,6 +782,7 @@ lua_cryptobox_signature_create (lua_State *L)
 static gint
 lua_cryptobox_signature_hex (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig = lua_check_cryptobox_sign (L, 1);
 	gchar *encoded;
 
@@ -647,6 +806,7 @@ lua_cryptobox_signature_hex (lua_State *L)
 static gint
 lua_cryptobox_signature_base32 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig = lua_check_cryptobox_sign (L, 1);
 	gchar *encoded;
 
@@ -670,6 +830,7 @@ lua_cryptobox_signature_base32 (lua_State *L)
 static gint
 lua_cryptobox_signature_base64 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig = lua_check_cryptobox_sign (L, 1);
 	gsize dlen;
 	gchar *encoded;
@@ -694,6 +855,7 @@ lua_cryptobox_signature_base64 (lua_State *L)
 static gint
 lua_cryptobox_signature_bin (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig = lua_check_cryptobox_sign (L, 1);
 
 	if (sig) {
@@ -709,6 +871,7 @@ lua_cryptobox_signature_bin (lua_State *L)
 static gint
 lua_cryptobox_signature_gc (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	rspamd_fstring_t *sig = lua_check_cryptobox_sign (L, 1);
 
 	rspamd_fstring_free (sig);
@@ -792,6 +955,7 @@ ret:
 static gint
 lua_cryptobox_hash_create (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h, **ph;
 	const gchar *s = NULL;
 	struct rspamd_lua_text *t;
@@ -834,6 +998,7 @@ lua_cryptobox_hash_create (lua_State *L)
 static gint
 lua_cryptobox_hash_create_specific (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h, **ph;
 	const gchar *s = NULL, *type = luaL_checkstring (L, 1);
 	gsize len = 0;
@@ -879,6 +1044,7 @@ lua_cryptobox_hash_create_specific (lua_State *L)
 static gint
 lua_cryptobox_hash_create_keyed (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h, **ph;
 	const gchar *key, *s = NULL;
 	struct rspamd_lua_text *t;
@@ -928,6 +1094,7 @@ lua_cryptobox_hash_create_keyed (lua_State *L)
 static gint
 lua_cryptobox_hash_update (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h = lua_check_cryptobox_hash (L, 1);
 	const gchar *data;
 	struct rspamd_lua_text *t;
@@ -975,6 +1142,7 @@ lua_cryptobox_hash_update (lua_State *L)
 static gint
 lua_cryptobox_hash_reset (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h = lua_check_cryptobox_hash (L, 1);
 
 	if (h) {
@@ -1002,6 +1170,7 @@ lua_cryptobox_hash_reset (lua_State *L)
 static gint
 lua_cryptobox_hash_hex (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h = lua_check_cryptobox_hash (L, 1);
 	guchar out[rspamd_cryptobox_HASHBYTES],
 		out_hex[rspamd_cryptobox_HASHBYTES * 2 + 1];
@@ -1038,6 +1207,7 @@ lua_cryptobox_hash_hex (lua_State *L)
 static gint
 lua_cryptobox_hash_base32 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h = lua_check_cryptobox_hash (L, 1);
 	guchar out[rspamd_cryptobox_HASHBYTES],
 		out_b32[rspamd_cryptobox_HASHBYTES * 2];
@@ -1073,6 +1243,7 @@ lua_cryptobox_hash_base32 (lua_State *L)
 static gint
 lua_cryptobox_hash_base64 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h = lua_check_cryptobox_hash (L, 1);
 	guchar out[rspamd_cryptobox_HASHBYTES], *b64;
 	gsize len;
@@ -1108,6 +1279,7 @@ lua_cryptobox_hash_base64 (lua_State *L)
 static gint
 lua_cryptobox_hash_bin (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h = lua_check_cryptobox_hash (L, 1);
 	guchar out[rspamd_cryptobox_HASHBYTES];
 	guint dlen;
@@ -1135,6 +1307,7 @@ lua_cryptobox_hash_bin (lua_State *L)
 static gint
 lua_cryptobox_hash_gc (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_cryptobox_hash *h = lua_check_cryptobox_hash (L, 1);
 
 	if (h->is_ssl) {
@@ -1156,7 +1329,7 @@ lua_cryptobox_hash_gc (lua_State *L)
 }
 
 /***
- * @function rspamd_cryptobox.verify_memory(pk, sig, data)
+ * @function rspamd_cryptobox.verify_memory(pk, sig, data, [alg = 'curve25519'])
  * Check memory using specified cryptobox key and signature
  * @param {pubkey} pk public key to verify
  * @param {sig} signature to check
@@ -1166,10 +1339,12 @@ lua_cryptobox_hash_gc (lua_State *L)
 static gint
 lua_cryptobox_verify_memory (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_pubkey *pk;
 	rspamd_fstring_t *signature;
 	struct rspamd_lua_text *t;
 	const gchar *data;
+	enum rspamd_cryptobox_mode alg = RSPAMD_CRYPTOBOX_MODE_25519;
 	gsize len;
 	gint ret;
 
@@ -1190,9 +1365,23 @@ lua_cryptobox_verify_memory (lua_State *L)
 		data = luaL_checklstring (L, 3, &len);
 	}
 
+	if (lua_isstring (L, 4)) {
+		const gchar *str = lua_tostring (L, 4);
+
+		if (strcmp (str, "nist") == 0 || strcmp (str, "openssl") == 0) {
+			alg = RSPAMD_CRYPTOBOX_MODE_NIST;
+		}
+		else if (strcmp (str, "curve25519") == 0 || strcmp (str, "default") == 0) {
+			alg = RSPAMD_CRYPTOBOX_MODE_25519;
+		}
+		else {
+			return luaL_error (L, "invalid algorithm: %s", str);
+		}
+	}
+
 	if (pk != NULL && signature != NULL && data != NULL) {
-		ret = rspamd_cryptobox_verify (signature->str, data, len,
-				rspamd_pubkey_get_pk (pk, NULL), RSPAMD_CRYPTOBOX_MODE_25519);
+		ret = rspamd_cryptobox_verify (signature->str, signature->len, data, len,
+				rspamd_pubkey_get_pk (pk, NULL), alg);
 
 		if (ret) {
 			lua_pushboolean (L, 1);
@@ -1209,7 +1398,7 @@ lua_cryptobox_verify_memory (lua_State *L)
 }
 
 /***
- * @function rspamd_cryptobox.verify_file(pk, sig, file)
+ * @function rspamd_cryptobox.verify_file(pk, sig, file, [alg = 'curve25519'])
  * Check file using specified cryptobox key and signature
  * @param {pubkey} pk public key to verify
  * @param {sig} signature to check
@@ -1219,10 +1408,12 @@ lua_cryptobox_verify_memory (lua_State *L)
 static gint
 lua_cryptobox_verify_file (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *fname;
 	struct rspamd_cryptobox_pubkey *pk;
 	rspamd_fstring_t *signature;
 	guchar *map = NULL;
+	enum rspamd_cryptobox_mode alg = RSPAMD_CRYPTOBOX_MODE_25519;
 	gsize len;
 	gint ret;
 
@@ -1230,11 +1421,26 @@ lua_cryptobox_verify_file (lua_State *L)
 	signature = lua_check_cryptobox_sign (L, 2);
 	fname = luaL_checkstring (L, 3);
 
+	if (lua_isstring (L, 4)) {
+		const gchar *str = lua_tostring (L, 4);
+
+		if (strcmp (str, "nist") == 0 || strcmp (str, "openssl") == 0) {
+			alg = RSPAMD_CRYPTOBOX_MODE_NIST;
+		}
+		else if (strcmp (str, "curve25519") == 0 || strcmp (str, "default") == 0) {
+			alg = RSPAMD_CRYPTOBOX_MODE_25519;
+		}
+		else {
+			return luaL_error (L, "invalid algorithm: %s", str);
+		}
+	}
+
 	map = rspamd_file_xmap (fname, PROT_READ, &len, TRUE);
 
 	if (map != NULL && pk != NULL && signature != NULL) {
-		ret = rspamd_cryptobox_verify (signature->str, map, len,
-				rspamd_pubkey_get_pk (pk, NULL), RSPAMD_CRYPTOBOX_MODE_25519);
+		ret = rspamd_cryptobox_verify (signature->str, signature->len,
+				map, len,
+				rspamd_pubkey_get_pk (pk, NULL), alg);
 
 		if (ret) {
 			lua_pushboolean (L, 1);
@@ -1268,6 +1474,7 @@ lua_cryptobox_verify_file (lua_State *L)
 static gint
 lua_cryptobox_sign_memory (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_keypair *kp;
 	const gchar *data;
 	struct rspamd_lua_text *t;
@@ -1291,7 +1498,7 @@ lua_cryptobox_sign_memory (lua_State *L)
 	}
 
 
-	if (!kp || !data) {
+	if (!kp || !data || kp->type == RSPAMD_KEYPAIR_KEX) {
 		return luaL_error (L, "invalid arguments");
 	}
 
@@ -1318,6 +1525,7 @@ lua_cryptobox_sign_memory (lua_State *L)
 static gint
 lua_cryptobox_sign_file (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_cryptobox_keypair *kp;
 	const gchar *filename;
 	gchar *data;
@@ -1353,6 +1561,267 @@ lua_cryptobox_sign_file (lua_State *L)
 	return 1;
 }
 
+/***
+ * @function rspamd_cryptobox.encrypt_memory(kp, data[, nist=false])
+ * Encrypt data using specified keypair/pubkey
+ * @param {keypair|string} kp keypair or pubkey in base32 to use
+ * @param {string|text} data
+ * @return {rspamd_text} encrypted text
+ */
+static gint
+lua_cryptobox_encrypt_memory (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp = NULL;
+	struct rspamd_cryptobox_pubkey *pk = NULL;
+	const gchar *data;
+	guchar *out = NULL;
+	struct rspamd_lua_text *t, *res;
+	gsize len = 0, outlen = 0;
+	GError *err = NULL;
+
+	if (lua_type (L, 1) == LUA_TUSERDATA) {
+		if (rspamd_lua_check_udata_maybe (L, 1, "rspamd{cryptobox_keypair}")) {
+			kp = lua_check_cryptobox_keypair (L, 1);
+		}
+		else if (rspamd_lua_check_udata_maybe (L, 1, "rspamd{cryptobox_pubkey}")) {
+			pk = lua_check_cryptobox_pubkey (L, 1);
+		}
+	}
+	else if (lua_type (L, 1) == LUA_TSTRING) {
+		const gchar *b32;
+		gsize blen;
+
+		b32 = lua_tolstring (L, 1, &blen);
+		pk = rspamd_pubkey_from_base32 (b32, blen, RSPAMD_KEYPAIR_KEX,
+				lua_toboolean (L, 3) ?
+				RSPAMD_CRYPTOBOX_MODE_NIST : RSPAMD_CRYPTOBOX_MODE_25519);
+	}
+
+	if (lua_isuserdata (L, 2)) {
+		t = lua_check_text (L, 2);
+
+		if (!t) {
+			return luaL_error (L, "invalid arguments");
+		}
+
+		data = t->start;
+		len = t->len;
+	}
+	else {
+		data = luaL_checklstring (L, 2, &len);
+	}
+
+
+	if (!(kp || pk) || !data) {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	if (kp) {
+		if (!rspamd_keypair_encrypt (kp, data, len, &out, &outlen, &err)) {
+			gint ret = luaL_error (L, "cannot encrypt data: %s", err->message);
+			g_error_free (err);
+
+			return ret;
+		}
+	}
+	else if (pk) {
+		if (!rspamd_pubkey_encrypt (pk, data, len, &out, &outlen, &err)) {
+			gint ret = luaL_error (L, "cannot encrypt data: %s", err->message);
+			g_error_free (err);
+
+			return ret;
+		}
+	}
+
+	res = lua_newuserdata (L, sizeof (*res));
+	res->flags = RSPAMD_TEXT_FLAG_OWN;
+	res->start = out;
+	res->len = outlen;
+	rspamd_lua_setclass (L, "rspamd{text}", -1);
+
+	return 1;
+}
+
+/***
+ * @function rspamd_cryptobox.encrypt_file(kp|pk_string, filename[, nist=false])
+ * Encrypt data using specified keypair/pubkey
+ * @param {keypair|string} kp keypair or pubkey in base32 to use
+ * @param {string} filename
+ * @return {rspamd_text} encrypted text
+ */
+static gint
+lua_cryptobox_encrypt_file (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp = NULL;
+	struct rspamd_cryptobox_pubkey *pk = NULL;
+	const gchar *filename;
+	gchar *data;
+	guchar *out = NULL;
+	struct rspamd_lua_text *res;
+	gsize len = 0, outlen = 0;
+	GError *err = NULL;
+
+	if (lua_type (L, 1) == LUA_TUSERDATA) {
+		if (rspamd_lua_check_udata_maybe (L, 1, "rspamd{cryptobox_keypair}")) {
+			kp = lua_check_cryptobox_keypair (L, 1);
+		}
+		else if (rspamd_lua_check_udata_maybe (L, 1, "rspamd{cryptobox_pubkey}")) {
+			pk = lua_check_cryptobox_pubkey (L, 1);
+		}
+	}
+	else if (lua_type (L, 1) == LUA_TSTRING) {
+		const gchar *b32;
+		gsize blen;
+
+		b32 = lua_tolstring (L, 1, &blen);
+		pk = rspamd_pubkey_from_base32 (b32, blen, RSPAMD_KEYPAIR_KEX,
+				lua_toboolean (L, 3) ?
+				RSPAMD_CRYPTOBOX_MODE_NIST : RSPAMD_CRYPTOBOX_MODE_25519);
+	}
+
+	filename = luaL_checkstring (L, 2);
+	data = rspamd_file_xmap (filename, PROT_READ, &len, TRUE);
+
+	if (!(kp || pk) || !data) {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	if (kp) {
+		if (!rspamd_keypair_encrypt (kp, data, len, &out, &outlen, &err)) {
+			gint ret = luaL_error (L, "cannot encrypt file %s: %s", filename,
+					err->message);
+			g_error_free (err);
+			munmap (data, len);
+
+			return ret;
+		}
+	}
+	else if (pk) {
+		if (!rspamd_pubkey_encrypt (pk, data, len, &out, &outlen, &err)) {
+			gint ret = luaL_error (L, "cannot encrypt file %s: %s", filename,
+					err->message);
+			g_error_free (err);
+			munmap (data, len);
+
+			return ret;
+		}
+	}
+
+	res = lua_newuserdata (L, sizeof (*res));
+	res->flags = RSPAMD_TEXT_FLAG_OWN;
+	res->start = out;
+	res->len = outlen;
+	rspamd_lua_setclass (L, "rspamd{text}", -1);
+	munmap (data, len);
+
+	return 1;
+}
+
+/***
+ * @function rspamd_cryptobox.decrypt_memory(kp, data[, nist = false])
+ * Encrypt data using specified keypair
+ * @param {keypair} kp keypair to use
+ * @param {string} data
+ * @return status,{rspamd_text}|error status is boolean variable followed by either unencrypted data or an error message
+ */
+static gint
+lua_cryptobox_decrypt_memory (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp;
+	const gchar *data;
+	guchar *out;
+	struct rspamd_lua_text *t, *res;
+	gsize len = 0, outlen;
+	GError *err = NULL;
+
+	kp = lua_check_cryptobox_keypair (L, 1);
+
+	if (lua_isuserdata (L, 2)) {
+		t = lua_check_text (L, 2);
+
+		if (!t) {
+			return luaL_error (L, "invalid arguments");
+		}
+
+		data = t->start;
+		len = t->len;
+	}
+	else {
+		data = luaL_checklstring (L, 2, &len);
+	}
+
+
+	if (!kp || !data) {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	if (!rspamd_keypair_decrypt (kp, data, len, &out, &outlen, &err)) {
+		lua_pushboolean (L, false);
+		lua_pushstring (L, err->message);
+		g_error_free (err);
+	}
+	else {
+		lua_pushboolean (L, true);
+		res = lua_newuserdata (L, sizeof (*res));
+		res->flags = RSPAMD_TEXT_FLAG_OWN;
+		res->start = out;
+		res->len = outlen;
+		rspamd_lua_setclass (L, "rspamd{text}", -1);
+	}
+
+	return 2;
+}
+
+/***
+ * @function rspamd_cryptobox.decrypt_file(kp, filename)
+ * Encrypt data using specified keypair
+ * @param {keypair} kp keypair to use
+ * @param {string} filename
+ * @return status,{rspamd_text}|error status is boolean variable followed by either unencrypted data or an error message
+ */
+static gint
+lua_cryptobox_decrypt_file (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_cryptobox_keypair *kp;
+	const gchar *filename;
+	gchar *data;
+	guchar *out;
+	struct rspamd_lua_text *res;
+	gsize len = 0, outlen;
+	GError *err = NULL;
+
+	kp = lua_check_cryptobox_keypair (L, 1);
+	filename = luaL_checkstring (L, 2);
+	data = rspamd_file_xmap (filename, PROT_READ, &len, TRUE);
+
+
+	if (!kp || !data) {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	if (!rspamd_keypair_decrypt (kp, data, len, &out, &outlen, &err)) {
+		lua_pushboolean (L, false);
+		lua_pushstring (L, err->message);
+		g_error_free (err);
+	}
+	else {
+		lua_pushboolean (L, true);
+		res = lua_newuserdata (L, sizeof (*res));
+		res->flags = RSPAMD_TEXT_FLAG_OWN;
+		res->start = out;
+		res->len = outlen;
+		rspamd_lua_setclass (L, "rspamd{text}", -1);
+	}
+
+	munmap (data, len);
+
+	return 2;
+}
+
 static gint
 lua_load_pubkey (lua_State * L)
 {
