@@ -260,6 +260,7 @@ lua_html_push_image (lua_State *L, struct html_image *img)
 {
 	LUA_TRACE_POINT;
 	struct html_tag **ptag;
+	struct rspamd_url **purl;
 
 	lua_newtable (L);
 
@@ -269,6 +270,14 @@ lua_html_push_image (lua_State *L, struct html_image *img)
 		lua_settable (L, -3);
 	}
 
+	if (img->url) {
+		lua_pushstring (L, "url");
+		purl = lua_newuserdata (L, sizeof (gpointer));
+		*purl = img->url;
+		rspamd_lua_setclass (L, "rspamd{url}", -1);
+		lua_settable (L, -3);
+	}
+
 	if (img->tag) {
 		lua_pushstring (L, "tag");
 		ptag = lua_newuserdata (L, sizeof (gpointer));
