@@ -140,6 +140,16 @@ LUA_FUNCTION_DEF (logger, debugm);
  */
 LUA_FUNCTION_DEF (logger, slog);
 
+/***
+ * @function logger.logx(level, module, id, fmt[, args)
+ * Extended interface to make a generic log message on any level
+ * @param {number} log level as a number (see GLogLevelFlags enum for values)
+ * @param {task|cfg|pool|string} id id to log
+ * @param {string} fmt format string, arguments are encoded as %<number>
+ * @param {any} args list of arguments to be replaced in %<number> positions
+ */
+LUA_FUNCTION_DEF (logger, logx);
+
 static const struct luaL_reg loggerlib_f[] = {
 		LUA_INTERFACE_DEF (logger, err),
 		LUA_INTERFACE_DEF (logger, warn),
@@ -155,18 +165,23 @@ static const struct luaL_reg loggerlib_f[] = {
 		LUA_INTERFACE_DEF (logger, debugx),
 		LUA_INTERFACE_DEF (logger, debugm),
 		LUA_INTERFACE_DEF (logger, slog),
+		LUA_INTERFACE_DEF (logger, logx),
 		{"__tostring", rspamd_lua_class_tostring},
 		{NULL, NULL}
 };
 
 static void
-lua_common_log_line (GLogLevelFlags level, lua_State *L,
-	   const gchar *msg, const gchar *uid, const gchar *module)
+lua_common_log_line (GLogLevelFlags level,
+					 lua_State *L,
+					 const gchar *msg,
+					 const gchar *uid,
+					 const gchar *module,
+					 gint stack_level)
 {
 	lua_Debug d;
 	gchar func_buf[128], *p;
 
-	if (lua_getstack (L, 1, &d) == 1) {
+	if (lua_getstack (L, stack_level, &d) == 1) {
 		(void) lua_getinfo (L, "Sl", &d);
 		if ((p = strrchr (d.short_src, '/')) == NULL) {
 			p = d.short_src;
@@ -184,44 +199,22 @@ lua_common_log_line (GLogLevelFlags level, lua_State *L,
 					d.currentline);
 		}
 
-		if (level == G_LOG_LEVEL_DEBUG) {
-			rspamd_conditional_debug (NULL,
-					NULL,
-					module,
-					uid,
-					func_buf,
-					"%s",
-					msg);
-		}
-		else {
-			rspamd_common_log_function (NULL,
-					level,
-					module,
-					uid,
-					func_buf,
-					"%s",
-					msg);
-		}
+		rspamd_common_log_function (NULL,
+				level,
+				module,
+				uid,
+				func_buf,
+				"%s",
+				msg);
 	}
 	else {
-		if (level == G_LOG_LEVEL_DEBUG) {
-			rspamd_conditional_debug (NULL,
-					NULL,
-					module,
-					uid,
-					G_STRFUNC,
-					"%s",
-					msg);
-		}
-		else {
-			rspamd_common_log_function (NULL,
-					level,
-					module,
-					uid,
-					G_STRFUNC,
-					"%s",
-					msg);
-		}
+		rspamd_common_log_function (NULL,
+				level,
+				module,
+				uid,
+				G_STRFUNC,
+				"%s",
+				msg);
 	}
 }
 
@@ -229,50 +222,56 @@ lua_common_log_line (GLogLevelFlags level, lua_State *L,
 static gint
 lua_logger_err (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *msg;
 	msg = luaL_checkstring (L, 1);
-	lua_common_log_line (G_LOG_LEVEL_CRITICAL, L, msg, NULL, NULL);
+	lua_common_log_line (G_LOG_LEVEL_CRITICAL, L, msg, NULL, NULL, 1);
 	return 0;
 }
 
 static gint
 lua_logger_warn (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *msg;
 	msg = luaL_checkstring (L, 1);
-	lua_common_log_line (G_LOG_LEVEL_WARNING, L, msg, NULL, NULL);
+	lua_common_log_line (G_LOG_LEVEL_WARNING, L, msg, NULL, NULL, 1);
 	return 0;
 }
 
 static gint
 lua_logger_info (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *msg;
 	msg = luaL_checkstring (L, 1);
-	lua_common_log_line (G_LOG_LEVEL_INFO, L, msg, NULL, NULL);
+	lua_common_log_line (G_LOG_LEVEL_INFO, L, msg, NULL, NULL, 1);
 	return 0;
 }
 
 static gint
 lua_logger_message (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *msg;
 	msg = luaL_checkstring (L, 1);
-	lua_common_log_line (G_LOG_LEVEL_MESSAGE, L, msg, NULL, NULL);
+	lua_common_log_line (G_LOG_LEVEL_MESSAGE, L, msg, NULL, NULL, 1);
 	return 0;
 }
 
 static gint
 lua_logger_debug (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *msg;
 	msg = luaL_checkstring (L, 1);
-	lua_common_log_line (G_LOG_LEVEL_DEBUG, L, msg, NULL, NULL);
+	lua_common_log_line (G_LOG_LEVEL_DEBUG, L, msg, NULL, NULL, 1);
 	return 0;
 }
 
 static gsize
-lua_logger_out_str (lua_State *L, gint pos, gchar *outbuf, gsize len)
+lua_logger_out_str (lua_State *L, gint pos, gchar *outbuf, gsize len,
+					struct lua_logger_trace *trace)
 {
 	gsize slen;
 	const gchar *str = lua_tolstring (L, pos, &slen);
@@ -286,7 +285,8 @@ lua_logger_out_str (lua_State *L, gint pos, gchar *outbuf, gsize len)
 }
 
 static gsize
-lua_logger_out_num (lua_State *L, gint pos, gchar *outbuf, gsize len)
+lua_logger_out_num (lua_State *L, gint pos, gchar *outbuf, gsize len,
+					struct lua_logger_trace *trace)
 {
 	gdouble num = lua_tonumber (L, pos);
 	glong inum;
@@ -304,7 +304,8 @@ lua_logger_out_num (lua_State *L, gint pos, gchar *outbuf, gsize len)
 }
 
 static gsize
-lua_logger_out_boolean (lua_State *L, gint pos, gchar *outbuf, gsize len)
+lua_logger_out_boolean (lua_State *L, gint pos, gchar *outbuf, gsize len,
+						struct lua_logger_trace *trace)
 {
 	gboolean val = lua_toboolean (L, pos);
 	gsize r = 0;
@@ -315,10 +316,12 @@ lua_logger_out_boolean (lua_State *L, gint pos, gchar *outbuf, gsize len)
 }
 
 static gsize
-lua_logger_out_userdata (lua_State *L, gint pos, gchar *outbuf, gsize len)
+lua_logger_out_userdata (lua_State *L, gint pos, gchar *outbuf, gsize len,
+						 struct lua_logger_trace *trace)
 {
 	gint r, top;
 	const gchar *str = NULL;
+	gboolean converted_to_str = FALSE;
 
 	top = lua_gettop (L);
 
@@ -348,6 +351,10 @@ lua_logger_out_userdata (lua_State *L, gint pos, gchar *outbuf, gsize len)
 		}
 
 		str = lua_tostring (L, -1);
+
+		if (str) {
+			converted_to_str = TRUE;
+		}
 	}
 	else {
 		lua_pushstring (L, "class");
@@ -362,7 +369,14 @@ lua_logger_out_userdata (lua_State *L, gint pos, gchar *outbuf, gsize len)
 		str = lua_tostring (L, -1);
 	}
 
-	r = rspamd_snprintf (outbuf, len + 1, "%s(%p)", str, lua_touserdata (L, pos));
+	if (converted_to_str) {
+		r = rspamd_snprintf (outbuf, len + 1, "%s", str);
+	}
+	else {
+		/* Print raw pointer */
+		r = rspamd_snprintf (outbuf, len + 1, "%s(%p)", str, lua_touserdata (L, pos));
+	}
+
 	lua_settop (L, top);
 
 	return r;
@@ -373,17 +387,34 @@ lua_logger_out_userdata (lua_State *L, gint pos, gchar *outbuf, gsize len)
     if ((remain) == 0) { lua_pop (L, 1); break; }
 
 static gsize
-lua_logger_out_table (lua_State *L, gint pos, gchar *outbuf, gsize len)
+lua_logger_out_table (lua_State *L, gint pos, gchar *outbuf, gsize len,
+					  struct lua_logger_trace *trace)
 {
 	gchar *d = outbuf;
 	gsize remain = len, r;
 	gboolean first = TRUE;
-	gint i;
+	gconstpointer self = NULL;
+	gint i, tpos;
 
 	if (!lua_istable (L, pos) || remain == 0) {
 		return 0;
 	}
 
+	self = lua_topointer (L, pos);
+
+	/* Check if we have seen this pointer */
+	for (i = 0; i < TRACE_POINTS; i ++) {
+		if (trace->traces[i] == self) {
+			r = rspamd_snprintf (d, remain + 1, "ref(%p)", self);
+
+			d += r;
+
+			return (d - outbuf);
+		}
+	}
+
+	trace->traces[trace->cur_level % TRACE_POINTS] = self;
+
 	lua_pushvalue (L, pos);
 	r = rspamd_snprintf (d, remain + 1, "{");
 	remain -= r;
@@ -405,7 +436,14 @@ lua_logger_out_table (lua_State *L, gint pos, gchar *outbuf, gsize len)
 
 		r = rspamd_snprintf (d, remain + 1, "[%d] = ", i);
 		MOVE_BUF(d, remain, r);
-		r = lua_logger_out_type (L, lua_gettop (L), d, remain);
+		tpos = lua_gettop (L);
+
+		if (lua_topointer (L, tpos) == self) {
+			r = rspamd_snprintf (d, remain + 1, "__self");
+		}
+		else {
+			r = lua_logger_out_type (L, tpos, d, remain, trace);
+		}
 		MOVE_BUF(d, remain, r);
 
 		first = FALSE;
@@ -428,7 +466,14 @@ lua_logger_out_table (lua_State *L, gint pos, gchar *outbuf, gsize len)
 		r = rspamd_snprintf (d, remain + 1, "[%s] = ",
 				lua_tostring (L, -2));
 		MOVE_BUF(d, remain, r);
-		r = lua_logger_out_type (L, lua_gettop (L), d, remain);
+		tpos = lua_gettop (L);
+
+		if (lua_topointer (L, tpos) == self) {
+			r = rspamd_snprintf (d, remain + 1, "__self");
+		}
+		else {
+			r = lua_logger_out_type (L, tpos, d, remain, trace);
+		}
 		MOVE_BUF(d, remain, r);
 
 		first = FALSE;
@@ -445,7 +490,8 @@ lua_logger_out_table (lua_State *L, gint pos, gchar *outbuf, gsize len)
 #undef MOVE_BUF
 
 gsize
-lua_logger_out_type (lua_State *L, gint pos, gchar *outbuf, gsize len)
+lua_logger_out_type (lua_State *L, gint pos, gchar *outbuf, gsize len,
+					 struct lua_logger_trace *trace)
 {
 	gint type;
 	gsize r = 0;
@@ -455,19 +501,20 @@ lua_logger_out_type (lua_State *L, gint pos, gchar *outbuf, gsize len)
 	}
 
 	type = lua_type (L, pos);
+	trace->cur_level ++;
 
 	switch (type) {
 		case LUA_TNUMBER:
-			r = lua_logger_out_num (L, pos, outbuf, len);
+			r = lua_logger_out_num (L, pos, outbuf, len, trace);
 			break;
 		case LUA_TBOOLEAN:
-			r = lua_logger_out_boolean (L, pos, outbuf, len);
+			r = lua_logger_out_boolean (L, pos, outbuf, len, trace);
 			break;
 		case LUA_TTABLE:
-			r = lua_logger_out_table (L, pos, outbuf, len);
+			r = lua_logger_out_table (L, pos, outbuf, len, trace);
 			break;
 		case LUA_TUSERDATA:
-			r = lua_logger_out_userdata (L, pos, outbuf, len);
+			r = lua_logger_out_userdata (L, pos, outbuf, len, trace);
 			break;
 		case LUA_TFUNCTION:
 			r = rspamd_snprintf (outbuf, len + 1, "function");
@@ -480,15 +527,17 @@ lua_logger_out_type (lua_State *L, gint pos, gchar *outbuf, gsize len)
 			break;
 		default:
 			/* Try to push everything as string using tostring magic */
-			r = lua_logger_out_str (L, pos, outbuf, len);
+			r = lua_logger_out_str (L, pos, outbuf, len, trace);
 			break;
 	}
 
+	trace->cur_level --;
+
 	return r;
 }
 
 static const gchar *
-lua_logger_get_id (lua_State *L, gint pos)
+lua_logger_get_id (lua_State *L, gint pos, GError **err)
 {
 	const gchar *uid = NULL, *clsname;
 
@@ -508,6 +557,10 @@ lua_logger_get_id (lua_State *L, gint pos)
 			if (task) {
 				uid = task->task_pool->tag.uid;
 			}
+			else {
+				g_set_error (err, g_quark_from_static_string ("lua_logger"),
+						EINVAL, "invalid rspamd{task}");
+			}
 		}
 		else if (strcmp (clsname, "rspamd{mempool}") == 0) {
 			rspamd_mempool_t  *pool;
@@ -517,6 +570,10 @@ lua_logger_get_id (lua_State *L, gint pos)
 			if (pool) {
 				uid = pool->tag.uid;
 			}
+			else {
+				g_set_error (err, g_quark_from_static_string ("lua_logger"),
+						EINVAL, "invalid rspamd{mempool}");
+			}
 		}
 		else if (strcmp (clsname, "rspamd{config}") == 0) {
 			struct rspamd_config *cfg;
@@ -524,7 +581,13 @@ lua_logger_get_id (lua_State *L, gint pos)
 			cfg = lua_check_config (L, pos);
 
 			if (cfg) {
-				uid = cfg->checksum;
+				if (cfg->checksum) {
+					uid = cfg->checksum;
+				}
+			}
+			else {
+				g_set_error (err, g_quark_from_static_string ("lua_logger"),
+						EINVAL, "invalid rspamd{config}");
 			}
 		}
 		else if (strcmp (clsname, "rspamd{map}") == 0) {
@@ -540,12 +603,24 @@ lua_logger_get_id (lua_State *L, gint pos)
 					uid = "embedded";
 				}
 			}
+			else {
+				g_set_error (err, g_quark_from_static_string ("lua_logger"),
+						EINVAL, "invalid rspamd{map}");
+			}
+		}
+		else {
+			g_set_error (err, g_quark_from_static_string ("lua_logger"),
+					EINVAL, "unknown class: %s", clsname);
 		}
 
 
 		/* Metatable, __index, classname */
 		lua_pop (L, 3);
 	}
+	else {
+		g_set_error (err, g_quark_from_static_string ("lua_logger"),
+				EINVAL, "no metatable found for userdata");
+	}
 
 	return uid;
 }
@@ -559,6 +634,7 @@ lua_logger_log_format (lua_State *L, gint fmt_pos, gboolean is_string,
 	gsize r, cpylen = 0;
 	guint arg_num = 0, cur_arg;
 	bool num_arg = false;
+	struct lua_logger_trace tr;
 	enum {
 		copy_char = 0,
 		got_percent,
@@ -627,7 +703,8 @@ lua_logger_log_format (lua_State *L, gint fmt_pos, gboolean is_string,
 					return FALSE;
 				}
 
-				r = lua_logger_out_type (L, arg_num + 1, d, remain);
+				memset (&tr, 0, sizeof (tr));
+				r = lua_logger_out_type (L, arg_num + 1, d, remain, &tr);
 				g_assert (r <= remain);
 				remain -= r;
 				d += r;
@@ -654,7 +731,8 @@ lua_logger_log_format (lua_State *L, gint fmt_pos, gboolean is_string,
 			return FALSE;
 		}
 
-		r = lua_logger_out_type (L, arg_num + 1, d, remain);
+		memset (&tr, 0, sizeof (tr));
+		r = lua_logger_out_type (L, arg_num + 1, d, remain, &tr);
 		g_assert (r <= remain);
 		remain -= r;
 		d += r;
@@ -673,32 +751,40 @@ lua_logger_log_format (lua_State *L, gint fmt_pos, gboolean is_string,
 }
 
 static gint
-lua_logger_logx (lua_State *L, GLogLevelFlags level, gboolean is_string)
+lua_logger_do_log (lua_State *L,
+				   GLogLevelFlags level,
+				   gboolean is_string,
+				   gint start_pos)
 {
 	gchar logbuf[RSPAMD_LOGBUF_SIZE - 128];
 	const gchar *uid = NULL;
-	gint fmt_pos = 1;
-	gboolean ret;
+	gint fmt_pos = start_pos;
+	gint ret;
+	GError *err = NULL;
 
-	if (lua_type (L, 1) == LUA_TSTRING) {
-		fmt_pos = 1;
+	if (lua_type (L, start_pos) == LUA_TSTRING) {
+		fmt_pos = start_pos;
 	}
-	else if (lua_type (L, 1) == LUA_TUSERDATA) {
-		fmt_pos = 2;
+	else if (lua_type (L, start_pos) == LUA_TUSERDATA) {
+		fmt_pos = start_pos + 1;
 
-		uid = lua_logger_get_id (L, 1);
+		uid = lua_logger_get_id (L, start_pos, &err);
 
 		if (uid == NULL) {
-			return luaL_error (L, "bad userdata for logging");
+			ret = luaL_error (L, "bad userdata for logging: %s",
+					err ? err->message : "unknown error");
+
+			if (err) {
+				g_error_free (err);
+			}
+
+			return ret;
 		}
 	}
 	else {
 		/* Bad argument type */
-		msg_err ("bad format string type: %s", lua_typename (L, lua_type (L,
-				1)));
-		lua_error (L);
-
-		return 0;
+		return luaL_error (L, "bad format string type: %s",
+				lua_typename (L, lua_type (L, start_pos)));
 	}
 
 	ret = lua_logger_log_format (L, fmt_pos, is_string,
@@ -710,7 +796,7 @@ lua_logger_logx (lua_State *L, GLogLevelFlags level, gboolean is_string)
 			return 1;
 		}
 		else {
-			lua_common_log_line (level, L, logbuf, uid, "lua");
+			lua_common_log_line (level, L, logbuf, uid, "lua", 1);
 		}
 	}
 	else {
@@ -727,38 +813,86 @@ lua_logger_logx (lua_State *L, GLogLevelFlags level, gboolean is_string)
 static gint
 lua_logger_errx (lua_State *L)
 {
-	return lua_logger_logx (L, G_LOG_LEVEL_CRITICAL, FALSE);
+	LUA_TRACE_POINT;
+	return lua_logger_do_log (L, G_LOG_LEVEL_CRITICAL, FALSE, 1);
 }
 
 static gint
 lua_logger_warnx (lua_State *L)
 {
-	return lua_logger_logx (L, G_LOG_LEVEL_WARNING, FALSE);
+	LUA_TRACE_POINT;
+	return lua_logger_do_log (L, G_LOG_LEVEL_WARNING, FALSE, 1);
 }
 
 static gint
 lua_logger_infox (lua_State *L)
 {
-	return lua_logger_logx (L, G_LOG_LEVEL_INFO, FALSE);
+	LUA_TRACE_POINT;
+	return lua_logger_do_log (L, G_LOG_LEVEL_INFO, FALSE, 1);
 }
 
 static gint
 lua_logger_messagex (lua_State *L)
 {
-	return lua_logger_logx (L, G_LOG_LEVEL_MESSAGE, FALSE);
+	LUA_TRACE_POINT;
+	return lua_logger_do_log (L, G_LOG_LEVEL_MESSAGE, FALSE, 1);
 }
 
 static gint
 lua_logger_debugx (lua_State *L)
 {
-	return lua_logger_logx (L, G_LOG_LEVEL_DEBUG, FALSE);
+	LUA_TRACE_POINT;
+	return lua_logger_do_log (L, G_LOG_LEVEL_DEBUG, FALSE, 1);
 }
 
+static gint
+lua_logger_logx (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	GLogLevelFlags flags = lua_tonumber (L, 1);
+	const gchar *modname = lua_tostring (L, 2), *uid = NULL;
+	gchar logbuf[RSPAMD_LOGBUF_SIZE - 128];
+	gboolean ret;
+	gint stack_pos = 1;
+
+	if (lua_type (L, 3) == LUA_TSTRING) {
+		uid = luaL_checkstring (L, 3);
+	}
+	else {
+		uid = lua_logger_get_id (L, 3, NULL);
+	}
+
+	if (uid && modname) {
+		if (lua_type (L, 4) == LUA_TSTRING) {
+			ret = lua_logger_log_format (L, 4, FALSE, logbuf, sizeof (logbuf) - 1);
+		}
+		else if (lua_type (L, 4) == LUA_TNUMBER) {
+			stack_pos = lua_tonumber (L, 4);
+			ret = lua_logger_log_format (L, 5, FALSE, logbuf, sizeof (logbuf) - 1);
+		}
+		else {
+			return luaL_error (L, "invalid argument on pos 4");
+		}
+
+		if (ret) {
+			lua_common_log_line (flags, L, logbuf, uid, modname, stack_pos);
+		}
+	}
+	else {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	return 0;
+}
+
+
 static gint
 lua_logger_debugm (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gchar logbuf[RSPAMD_LOGBUF_SIZE - 128];
 	const gchar *uid = NULL, *module = NULL;
+	gint stack_pos = 1;
 	gboolean ret;
 
 	module = luaL_checkstring (L, 1);
@@ -767,14 +901,23 @@ lua_logger_debugm (lua_State *L)
 		uid = luaL_checkstring (L, 2);
 	}
 	else {
-		uid = lua_logger_get_id (L, 2);
+		uid = lua_logger_get_id (L, 2, NULL);
 	}
 
-	if (uid && module && lua_type (L, 3) == LUA_TSTRING) {
-		ret = lua_logger_log_format (L, 3, FALSE, logbuf, sizeof (logbuf) - 1);
+	if (uid && module) {
+		if (lua_type (L, 3) == LUA_TSTRING) {
+			ret = lua_logger_log_format (L, 3, FALSE, logbuf, sizeof (logbuf) - 1);
+		}
+		else if (lua_type (L, 3) == LUA_TNUMBER) {
+			stack_pos = lua_tonumber (L, 3);
+			ret = lua_logger_log_format (L, 4, FALSE, logbuf, sizeof (logbuf) - 1);
+		}
+		else {
+			return luaL_error (L, "invalid argument on pos 3");
+		}
 
 		if (ret) {
-			lua_common_log_line (G_LOG_LEVEL_DEBUG, L, logbuf, uid, module);
+			lua_common_log_line (G_LOG_LEVEL_DEBUG, L, logbuf, uid, module, stack_pos);
 		}
 	}
 	else {
@@ -789,7 +932,7 @@ lua_logger_debugm (lua_State *L)
 static gint
 lua_logger_slog (lua_State *L)
 {
-	return lua_logger_logx (L, 0, TRUE);
+	return lua_logger_do_log (L, 0, TRUE, 1);
 }
 
 /*** Init functions ***/
