@@ -141,6 +141,7 @@ lua_check_map (lua_State * L, gint pos)
 gint
 lua_config_add_radix_map (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	const gchar *map_line, *description;
 	struct rspamd_lua_map *map, **pmap;
@@ -156,6 +157,7 @@ lua_config_add_radix_map (lua_State *L)
 		if ((m = rspamd_map_add (cfg, map_line, description,
 				rspamd_radix_read,
 				rspamd_radix_fin,
+				rspamd_radix_dtor,
 				(void **)&map->data.radix)) == NULL) {
 			msg_warn_config ("invalid radix map %s", map_line);
 			lua_pushnil (L);
@@ -180,6 +182,7 @@ lua_config_add_radix_map (lua_State *L)
 gint
 lua_config_radix_from_config (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	const gchar *mname, *optname;
 	const ucl_object_t *obj;
@@ -211,6 +214,7 @@ lua_config_radix_from_config (lua_State *L)
 			if ((m = rspamd_map_add_from_ucl (cfg, fake_obj, "static radix map",
 					rspamd_radix_read,
 					rspamd_radix_fin,
+					rspamd_radix_dtor,
 					(void **)&map->data.radix)) == NULL) {
 				msg_err_config ("invalid radix map static");
 				lua_pushnil (L);
@@ -244,6 +248,7 @@ lua_config_radix_from_config (lua_State *L)
 gint
 lua_config_radix_from_ucl (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	ucl_object_t *obj;
 	struct rspamd_lua_map *map, **pmap;
@@ -270,15 +275,18 @@ lua_config_radix_from_ucl (lua_State *L)
 		if ((m = rspamd_map_add_from_ucl (cfg, fake_obj, "static radix map",
 				rspamd_radix_read,
 				rspamd_radix_fin,
+				rspamd_radix_dtor,
 				(void **)&map->data.radix)) == NULL) {
 			msg_err_config ("invalid radix map static");
 			lua_pushnil (L);
 			ucl_object_unref (fake_obj);
+			ucl_object_unref (obj);
 
 			return 1;
 		}
 
 		ucl_object_unref (fake_obj);
+		ucl_object_unref (obj);
 		pmap = lua_newuserdata (L, sizeof (void *));
 		map->map = m;
 		m->lua_map = map;
@@ -296,6 +304,7 @@ lua_config_radix_from_ucl (lua_State *L)
 gint
 lua_config_add_hash_map (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	const gchar *map_line, *description;
 	struct rspamd_lua_map *map, **pmap;
@@ -311,6 +320,7 @@ lua_config_add_hash_map (lua_State *L)
 		if ((m = rspamd_map_add (cfg, map_line, description,
 				rspamd_kv_list_read,
 				rspamd_kv_list_fin,
+				rspamd_kv_list_dtor,
 				(void **)&map->data.hash)) == NULL) {
 			msg_warn_config ("invalid set map %s", map_line);
 			lua_pushnil (L);
@@ -334,6 +344,7 @@ lua_config_add_hash_map (lua_State *L)
 gint
 lua_config_add_kv_map (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	const gchar *map_line, *description;
 	struct rspamd_lua_map *map, **pmap;
@@ -349,6 +360,7 @@ lua_config_add_kv_map (lua_State *L)
 		if ((m = rspamd_map_add (cfg, map_line, description,
 				rspamd_kv_list_read,
 				rspamd_kv_list_fin,
+				rspamd_kv_list_dtor,
 				(void **)&map->data.hash)) == NULL) {
 			msg_warn_config ("invalid hash map %s", map_line);
 			lua_pushnil (L);
@@ -441,9 +453,27 @@ lua_map_fin (struct map_cb_data *data)
 	cbdata->data = rspamd_fstring_assign (cbdata->data, "", 0);
 }
 
+static void
+lua_map_dtor (struct map_cb_data *data)
+{
+	struct lua_map_callback_data *cbdata;
+
+	if (data->cur_data) {
+		cbdata = (struct lua_map_callback_data *)data->cur_data;
+		if (cbdata->ref != -1) {
+			luaL_unref (cbdata->L, LUA_REGISTRYINDEX, cbdata->ref);
+		}
+
+		if (cbdata->data) {
+			rspamd_fstring_free (cbdata->data);
+		}
+	}
+}
+
 gint
 lua_config_add_map (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	const char *description = NULL;
 	const gchar *type = NULL;
@@ -488,7 +518,9 @@ lua_config_add_map (lua_State *L)
 			cbdata->ref = cbidx;
 
 			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
-					lua_map_read, lua_map_fin,
+					lua_map_read,
+					lua_map_fin,
+					lua_map_dtor,
 					(void **)&map->data.cbdata)) == NULL) {
 
 				if (cbidx != -1) {
@@ -513,6 +545,7 @@ lua_config_add_map (lua_State *L)
 			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
 					rspamd_kv_list_read,
 					rspamd_kv_list_fin,
+					rspamd_kv_list_dtor,
 					(void **)&map->data.hash)) == NULL) {
 				lua_pushnil (L);
 				ucl_object_unref (map_obj);
@@ -529,6 +562,7 @@ lua_config_add_map (lua_State *L)
 			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
 					rspamd_kv_list_read,
 					rspamd_kv_list_fin,
+					rspamd_kv_list_dtor,
 					(void **)&map->data.hash)) == NULL) {
 				lua_pushnil (L);
 				ucl_object_unref (map_obj);
@@ -545,6 +579,7 @@ lua_config_add_map (lua_State *L)
 			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
 					rspamd_radix_read,
 					rspamd_radix_fin,
+					rspamd_radix_dtor,
 					(void **)&map->data.radix)) == NULL) {
 				lua_pushnil (L);
 				ucl_object_unref (map_obj);
@@ -561,6 +596,7 @@ lua_config_add_map (lua_State *L)
 			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
 					rspamd_regexp_list_read_single,
 					rspamd_regexp_list_fin,
+					rspamd_regexp_list_dtor,
 					(void **) &map->data.re_map)) == NULL) {
 				lua_pushnil (L);
 				ucl_object_unref (map_obj);
@@ -577,6 +613,7 @@ lua_config_add_map (lua_State *L)
 			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
 					rspamd_regexp_list_read_multiple,
 					rspamd_regexp_list_fin,
+					rspamd_regexp_list_dtor,
 					(void **) &map->data.re_map)) == NULL) {
 				lua_pushnil (L);
 				ucl_object_unref (map_obj);
@@ -593,6 +630,24 @@ lua_config_add_map (lua_State *L)
 			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
 					rspamd_glob_list_read_single,
 					rspamd_regexp_list_fin,
+					rspamd_regexp_list_dtor,
+					(void **) &map->data.re_map)) == NULL) {
+				lua_pushnil (L);
+				ucl_object_unref (map_obj);
+
+				return 1;
+			}
+			m->lua_map = map;
+		}
+		else if (strcmp (type, "glob_multi") == 0) {
+			map = rspamd_mempool_alloc0 (cfg->cfg_pool, sizeof (*map));
+			map->data.re_map = NULL;
+			map->type = RSPAMD_LUA_MAP_REGEXP_MULTIPLE;
+
+			if ((m = rspamd_map_add_from_ucl (cfg, map_obj, description,
+					rspamd_glob_list_read_multiple,
+					rspamd_regexp_list_fin,
+					rspamd_regexp_list_dtor,
 					(void **) &map->data.re_map)) == NULL) {
 				lua_pushnil (L);
 				ucl_object_unref (map_obj);
@@ -625,6 +680,7 @@ lua_config_add_map (lua_State *L)
 gint
 lua_config_get_maps (lua_State*L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	struct rspamd_lua_map *map, **pmap;
 	struct rspamd_map *m;
@@ -701,6 +757,7 @@ lua_map_process_string_key (lua_State *L, gint pos, gsize *len)
 static gint
 lua_map_get_key (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	struct rspamd_radix_map_helper *radix;
 	struct rspamd_lua_ip *addr = NULL;
@@ -854,7 +911,7 @@ lua_map_traverse_cb (gconstpointer key,
 	lua_State *L = (lua_State *)ud;
 
 	lua_pushstring (L, key);
-	lua_pushnumber (L, hits);
+	lua_pushinteger (L, hits);
 	lua_settable (L, -3);
 
 	return TRUE;
@@ -863,6 +920,7 @@ lua_map_traverse_cb (gconstpointer key,
 static gint
 lua_map_get_stats (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	gboolean do_reset = FALSE;
 
@@ -887,6 +945,7 @@ lua_map_get_stats (lua_State * L)
 static gint
 lua_map_get_data_digest (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	gchar numbuf[64];
 
@@ -904,10 +963,11 @@ lua_map_get_data_digest (lua_State * L)
 static gint
 lua_map_get_nelts (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 
 	if (map != NULL) {
-		lua_pushnumber (L, map->map->nelts);
+		lua_pushinteger (L, map->map->nelts);
 	}
 	else {
 		return luaL_error (L, "invalid arguments");
@@ -919,6 +979,7 @@ lua_map_get_nelts (lua_State * L)
 static int
 lua_map_is_signed (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	gboolean ret = FALSE;
 	struct rspamd_map_backend *bk;
@@ -946,6 +1007,7 @@ lua_map_is_signed (lua_State *L)
 static int
 lua_map_get_proto (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	const gchar *ret = "undefined";
 	struct rspamd_map_backend *bk;
@@ -982,6 +1044,7 @@ lua_map_get_proto (lua_State *L)
 static int
 lua_map_get_sign_key (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	struct rspamd_map_backend *bk;
 	guint i;
@@ -1018,6 +1081,7 @@ lua_map_get_sign_key (lua_State *L)
 static int
 lua_map_set_sign_key (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	struct rspamd_map_backend *bk;
 	const gchar *pk_str;
@@ -1057,6 +1121,7 @@ lua_map_set_sign_key (lua_State *L)
 static int
 lua_map_set_callback (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 
 	if (!map || map->type != RSPAMD_LUA_MAP_CALLBACK || map->data.cbdata == NULL) {
@@ -1077,6 +1142,7 @@ lua_map_set_callback (lua_State *L)
 static int
 lua_map_get_uri (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_map *map = lua_check_map (L, 1);
 	const gchar *ret = "undefined";
 	struct rspamd_map_backend *bk;
