@@ -18,6 +18,8 @@
 #include "libmime/message.h"
 #include "libmime/lang_detection.h"
 #include "libstat/stat_api.h"
+#include "libcryptobox/cryptobox.h"
+#include "libutil/shingles.h"
 
 /* Textpart methods */
 /***
@@ -168,6 +170,17 @@ LUA_FUNCTION_DEF (textpart, get_language);
  * @return {array|tables} all languages detected for the part
  */
 LUA_FUNCTION_DEF (textpart, get_languages);
+/***
+ * @method text_part:get_fuzzy_hashes(mempool)
+ * @param {rspamd_mempool} mempool - memory pool (usually task pool)
+ * Returns direct hash + array of shingles being calculated as following:
+ * - [1] - fuzzy digest as a string
+ * - [2..33] - fuzzy hashes as the following tables:
+ *   - [1] - 64 bit integer represented as a string
+ *   - [2..4] - strings used to generate this hash
+ * @return {string,array|tables} fuzzy hashes calculated
+ */
+LUA_FUNCTION_DEF (textpart, get_fuzzy_hashes);
 /***
  * @method text_part:get_mimepart()
  * Returns the mime part object corresponding to this text part
@@ -195,6 +208,7 @@ static const struct luaL_reg textpartlib_m[] = {
 	LUA_INTERFACE_DEF (textpart, get_languages),
 	LUA_INTERFACE_DEF (textpart, get_mimepart),
 	LUA_INTERFACE_DEF (textpart, get_stats),
+	LUA_INTERFACE_DEF (textpart, get_fuzzy_hashes),
 	{"__tostring", rspamd_lua_class_tostring},
 	{NULL, NULL}
 };
@@ -262,12 +276,27 @@ function check_header_delimiter_tab(task, header_name)
 end
  */
 LUA_FUNCTION_DEF (mimepart, get_header_full);
+/***
+ * @method mimepart:get_header_count(name[, case_sensitive])
+ * Lightweight version if you need just a header's count
+ *  * By default headers are searched in caseless matter.
+ * @param {string} name name of header to get
+ * @param {boolean} case_sensitive case sensitiveness flag to search for a header
+ * @return {number} number of header's occurrencies or 0 if not found
+ */
+LUA_FUNCTION_DEF (mimepart, get_header_count);
 /***
  * @method mime_part:get_content()
- * Get the raw content of part
+ * Get the parsed content of part
  * @return {text} opaque text object (zero-copy if not casted to lua string)
  */
 LUA_FUNCTION_DEF (mimepart, get_content);
+/***
+ * @method mime_part:get_raw_content()
+ * Get the raw content of part
+ * @return {text} opaque text object (zero-copy if not casted to lua string)
+ */
+LUA_FUNCTION_DEF (mimepart, get_raw_content);
 /***
  * @method mime_part:get_length()
  * Get length of the content of the part
@@ -395,6 +424,7 @@ LUA_FUNCTION_DEF (mimepart, headers_foreach);
 
 static const struct luaL_reg mimepartlib_m[] = {
 	LUA_INTERFACE_DEF (mimepart, get_content),
+	LUA_INTERFACE_DEF (mimepart, get_raw_content),
 	LUA_INTERFACE_DEF (mimepart, get_length),
 	LUA_INTERFACE_DEF (mimepart, get_type),
 	LUA_INTERFACE_DEF (mimepart, get_type_full),
@@ -403,6 +433,7 @@ static const struct luaL_reg mimepartlib_m[] = {
 	LUA_INTERFACE_DEF (mimepart, get_header),
 	LUA_INTERFACE_DEF (mimepart, get_header_raw),
 	LUA_INTERFACE_DEF (mimepart, get_header_full),
+	LUA_INTERFACE_DEF (mimepart, get_header_count),
 	LUA_INTERFACE_DEF (mimepart, is_image),
 	LUA_INTERFACE_DEF (mimepart, get_image),
 	LUA_INTERFACE_DEF (mimepart, is_archive),
@@ -439,6 +470,7 @@ lua_check_mimepart (lua_State * L)
 static gint
 lua_textpart_is_utf (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part == NULL || IS_PART_EMPTY (part)) {
@@ -455,6 +487,7 @@ lua_textpart_is_utf (lua_State * L)
 static gint
 lua_textpart_has_8bit_raw (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part) {
@@ -475,6 +508,7 @@ lua_textpart_has_8bit_raw (lua_State * L)
 static gint
 lua_textpart_has_8bit (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part) {
@@ -496,6 +530,7 @@ lua_textpart_has_8bit (lua_State * L)
 static gint
 lua_textpart_get_content (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	struct rspamd_lua_text *t;
 	gsize len;
@@ -514,16 +549,16 @@ lua_textpart_get_content (lua_State * L)
 	rspamd_lua_setclass (L, "rspamd{text}", -1);
 
 	if (!type) {
-		start = part->content->data;
-		len = part->content->len;
+		start = part->utf_content->data;
+		len = part->utf_content->len;
 	}
 	else if (strcmp (type, "content") == 0) {
-		start = part->content->data;
-		len = part->content->len;
+		start = part->utf_content->data;
+		len = part->utf_content->len;
 	}
 	else if (strcmp (type, "content_oneline") == 0) {
-		start = part->stripped_content->data;
-		len = part->stripped_content->len;
+		start = part->utf_stripped_content->data;
+		len = part->utf_stripped_content->len;
 	}
 	else if (strcmp (type, "raw_parsed") == 0) {
 		start = part->parsed.begin;
@@ -551,6 +586,7 @@ lua_textpart_get_content (lua_State * L)
 static gint
 lua_textpart_get_raw_content (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	struct rspamd_lua_text *t;
 
@@ -571,6 +607,7 @@ lua_textpart_get_raw_content (lua_State * L)
 static gint
 lua_textpart_get_content_oneline (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	struct rspamd_lua_text *t;
 
@@ -581,8 +618,8 @@ lua_textpart_get_content_oneline (lua_State * L)
 
 	t = lua_newuserdata (L, sizeof (*t));
 	rspamd_lua_setclass (L, "rspamd{text}", -1);
-	t->start = part->stripped_content->data;
-	t->len = part->stripped_content->len;
+	t->start = part->utf_stripped_content->data;
+	t->len = part->utf_stripped_content->len;
 	t->flags = 0;
 
 	return 1;
@@ -591,6 +628,7 @@ lua_textpart_get_content_oneline (lua_State * L)
 static gint
 lua_textpart_get_length (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part == NULL) {
@@ -598,11 +636,11 @@ lua_textpart_get_length (lua_State * L)
 		return 1;
 	}
 
-	if (IS_PART_EMPTY (part) || part->content == NULL) {
-		lua_pushnumber (L, 0);
+	if (IS_PART_EMPTY (part) || part->utf_content == NULL) {
+		lua_pushinteger (L, 0);
 	}
 	else {
-		lua_pushnumber (L, part->content->len);
+		lua_pushinteger (L, part->utf_content->len);
 	}
 
 	return 1;
@@ -611,6 +649,7 @@ lua_textpart_get_length (lua_State * L)
 static gint
 lua_textpart_get_raw_length (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part == NULL) {
@@ -618,7 +657,7 @@ lua_textpart_get_raw_length (lua_State * L)
 		return 1;
 	}
 
-	lua_pushnumber (L, part->raw.len);
+	lua_pushinteger (L, part->raw.len);
 
 	return 1;
 }
@@ -626,6 +665,7 @@ lua_textpart_get_raw_length (lua_State * L)
 static gint
 lua_textpart_get_urls_length (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	GList *cur;
 	guint total = 0;
@@ -644,7 +684,7 @@ lua_textpart_get_urls_length (lua_State * L)
 		}
 	}
 
-	lua_pushnumber (L, total);
+	lua_pushinteger (L, total);
 
 	return 1;
 }
@@ -652,6 +692,7 @@ lua_textpart_get_urls_length (lua_State * L)
 static gint
 lua_textpart_get_lines_count (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part == NULL) {
@@ -660,10 +701,10 @@ lua_textpart_get_lines_count (lua_State * L)
 	}
 
 	if (IS_PART_EMPTY (part)) {
-		lua_pushnumber (L, 0);
+		lua_pushinteger (L, 0);
 	}
 	else {
-		lua_pushnumber (L, part->nlines);
+		lua_pushinteger (L, part->nlines);
 	}
 
 	return 1;
@@ -672,6 +713,7 @@ lua_textpart_get_lines_count (lua_State * L)
 static gint
 lua_textpart_get_words_count (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part == NULL) {
@@ -679,11 +721,11 @@ lua_textpart_get_words_count (lua_State *L)
 		return 1;
 	}
 
-	if (IS_PART_EMPTY (part) || part->normalized_words == NULL) {
-		lua_pushnumber (L, 0);
+	if (IS_PART_EMPTY (part) || part->utf_words == NULL) {
+		lua_pushinteger (L, 0);
 	}
 	else {
-		lua_pushnumber (L, part->normalized_words->len);
+		lua_pushinteger (L, part->utf_words->len);
 	}
 
 	return 1;
@@ -692,6 +734,7 @@ lua_textpart_get_words_count (lua_State *L)
 static gint
 lua_textpart_get_words (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	rspamd_stat_token_t *w;
 	guint i;
@@ -700,14 +743,14 @@ lua_textpart_get_words (lua_State *L)
 		return luaL_error (L, "invalid arguments");
 	}
 
-	if (IS_PART_EMPTY (part) || part->normalized_words == NULL) {
+	if (IS_PART_EMPTY (part) || part->utf_words == NULL) {
 		lua_createtable (L, 0, 0);
 	}
 	else {
-		lua_createtable (L, part->normalized_words->len, 0);
+		lua_createtable (L, part->utf_words->len, 0);
 
-		for (i = 0; i < part->normalized_words->len; i ++) {
-			w = &g_array_index (part->normalized_words, rspamd_stat_token_t, i);
+		for (i = 0; i < part->utf_words->len; i ++) {
+			w = &g_array_index (part->utf_words, rspamd_stat_token_t, i);
 
 			lua_pushlstring (L, w->begin, w->len);
 			lua_rawseti (L, -2, i + 1);
@@ -720,6 +763,7 @@ lua_textpart_get_words (lua_State *L)
 static gint
 lua_textpart_is_empty (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part == NULL) {
@@ -735,6 +779,7 @@ lua_textpart_is_empty (lua_State * L)
 static gint
 lua_textpart_is_html (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part == NULL) {
@@ -750,6 +795,7 @@ lua_textpart_is_html (lua_State * L)
 static gint
 lua_textpart_get_html (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	struct html_content **phc;
 
@@ -768,6 +814,7 @@ lua_textpart_get_html (lua_State * L)
 static gint
 lua_textpart_get_language (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part != NULL) {
@@ -789,6 +836,7 @@ lua_textpart_get_language (lua_State * L)
 static gint
 lua_textpart_get_languages (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	guint i;
 	struct rspamd_lang_detector_res *cur;
@@ -820,9 +868,127 @@ lua_textpart_get_languages (lua_State * L)
 	return 1;
 }
 
+struct lua_shingle_data {
+	guint64 hash;
+	rspamd_ftok_t t1;
+	rspamd_ftok_t t2;
+	rspamd_ftok_t t3;
+};
+
+#define STORE_TOKEN(i, t) do { \
+    if ((i) < part->utf_words->len) { \
+        word = &g_array_index (part->utf_words, rspamd_stat_token_t, (i)); \
+        sd->t.begin = word->begin; \
+        sd->t.len = word->len; \
+    } \
+    }while (0)
+
+static guint64
+lua_shingles_filter (guint64 *input, gsize count,
+					 gint shno, const guchar *key, gpointer ud)
+{
+	guint64 minimal = G_MAXUINT64;
+	gsize i, min_idx = 0;
+	struct lua_shingle_data *sd;
+	rspamd_stat_token_t *word;
+	struct rspamd_mime_text_part *part = (struct rspamd_mime_text_part *)ud;
+
+	for (i = 0; i < count; i ++) {
+		if (minimal > input[i]) {
+			minimal = input[i];
+			min_idx = i;
+		}
+	}
+
+	sd = g_malloc0 (sizeof (*sd));
+	sd->hash = minimal;
+
+
+	STORE_TOKEN (min_idx, t1);
+	STORE_TOKEN (min_idx + 1, t2);
+	STORE_TOKEN (min_idx + 2, t3);
+
+	return GPOINTER_TO_SIZE (sd);
+}
+
+#undef STORE_TOKEN
+
+static gint
+lua_textpart_get_fuzzy_hashes (lua_State * L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_mime_text_part *part = lua_check_textpart (L);
+	rspamd_mempool_t *pool = rspamd_lua_check_mempool (L, 2);
+	guchar key[rspamd_cryptobox_HASHBYTES], digest[rspamd_cryptobox_HASHBYTES],
+			hexdigest[rspamd_cryptobox_HASHBYTES * 2 + 1], numbuf[64];
+	struct rspamd_shingle *sgl;
+	guint i;
+	struct lua_shingle_data *sd;
+	rspamd_cryptobox_hash_state_t st;
+	rspamd_stat_token_t *word;
+
+	if (part && pool) {
+		/* TODO: add keys and algorithms support */
+		rspamd_cryptobox_hash (key, "rspamd", strlen ("rspamd"), NULL, 0);
+
+		/* TODO: add short text support */
+
+		/* Calculate direct hash */
+		rspamd_cryptobox_hash_init (&st, key, rspamd_cryptobox_HASHKEYBYTES);
+
+		for (i = 0; i < part->utf_words->len; i ++) {
+			word = &g_array_index (part->utf_words, rspamd_stat_token_t, i);
+			rspamd_cryptobox_hash_update (&st, word->begin, word->len);
+		}
+
+		rspamd_cryptobox_hash_final (&st, digest);
+
+		rspamd_encode_hex_buf (digest, sizeof (digest), hexdigest,
+				sizeof (hexdigest));
+		lua_pushlstring (L, hexdigest, sizeof (hexdigest) - 1);
+
+		sgl = rspamd_shingles_from_text (part->utf_words, key,
+				pool, lua_shingles_filter, part, RSPAMD_SHINGLES_MUMHASH);
+
+		if (sgl == NULL) {
+			lua_pushnil (L);
+		}
+		else {
+			lua_createtable (L, G_N_ELEMENTS (sgl->hashes), 0);
+
+			for (i = 0; i < G_N_ELEMENTS (sgl->hashes); i ++) {
+				sd = GSIZE_TO_POINTER (sgl->hashes[i]);
+
+				lua_createtable (L, 4, 0);
+				rspamd_snprintf (numbuf, sizeof (numbuf), "%uL", sd->hash);
+				lua_pushstring (L, numbuf);
+				lua_rawseti (L, -2, 1);
+
+				/* Tokens */
+				lua_pushlstring (L, sd->t1.begin, sd->t1.len);
+				lua_rawseti (L, -2, 2);
+
+				lua_pushlstring (L, sd->t2.begin, sd->t2.len);
+				lua_rawseti (L, -2, 3);
+
+				lua_pushlstring (L, sd->t3.begin, sd->t3.len);
+				lua_rawseti (L, -2, 4);
+
+				lua_rawseti (L, -2, i + 1); /* Store table */
+			}
+		}
+	}
+	else {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	return 2;
+}
+
 static gint
 lua_textpart_get_mimepart (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 	struct rspamd_mime_part **pmime;
 
@@ -855,37 +1021,38 @@ lua_textpart_get_mimepart (lua_State * L)
 static gint
 lua_textpart_get_stats (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_text_part *part = lua_check_textpart (L);
 
 	if (part != NULL) {
 		lua_createtable (L, 0, 9);
 
 		lua_pushstring (L, "lines");
-		lua_pushnumber (L, part->nlines);
+		lua_pushinteger (L, part->nlines);
 		lua_settable (L, -3);
 		lua_pushstring (L, "empty_lines");
-		lua_pushnumber (L, part->empty_lines);
+		lua_pushinteger (L, part->empty_lines);
 		lua_settable (L, -3);
 		lua_pushstring (L, "spaces");
-		lua_pushnumber (L, part->spaces);
+		lua_pushinteger (L, part->spaces);
 		lua_settable (L, -3);
 		lua_pushstring (L, "non_spaces");
-		lua_pushnumber (L, part->non_spaces);
+		lua_pushinteger (L, part->non_spaces);
 		lua_settable (L, -3);
 		lua_pushstring (L, "double_spaces");
-		lua_pushnumber (L, part->double_spaces);
+		lua_pushinteger (L, part->double_spaces);
 		lua_settable (L, -3);
 		lua_pushstring (L, "ascii_characters");
-		lua_pushnumber (L, part->ascii_chars);
+		lua_pushinteger (L, part->ascii_chars);
 		lua_settable (L, -3);
 		lua_pushstring (L, "non_ascii_characters");
-		lua_pushnumber (L, part->non_ascii_chars);
+		lua_pushinteger (L, part->non_ascii_chars);
 		lua_settable (L, -3);
 		lua_pushstring (L, "capital_letters");
-		lua_pushnumber (L, part->capital_letters);
+		lua_pushinteger (L, part->capital_letters);
 		lua_settable (L, -3);
 		lua_pushstring (L, "numeric_characters");
-		lua_pushnumber (L, part->numeric_characters);
+		lua_pushinteger (L, part->numeric_characters);
 		lua_settable (L, -3);
 	}
 	else {
@@ -900,6 +1067,7 @@ lua_textpart_get_stats (lua_State * L)
 static gint
 lua_mimepart_get_content (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 	struct rspamd_lua_text *t;
 
@@ -917,9 +1085,31 @@ lua_mimepart_get_content (lua_State * L)
 	return 1;
 }
 
+static gint
+lua_mimepart_get_raw_content (lua_State * L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_mime_part *part = lua_check_mimepart (L);
+	struct rspamd_lua_text *t;
+
+	if (part == NULL) {
+		lua_pushnil (L);
+		return 1;
+	}
+
+	t = lua_newuserdata (L, sizeof (*t));
+	rspamd_lua_setclass (L, "rspamd{text}", -1);
+	t->start = part->raw_data.begin;
+	t->len = part->raw_data.len;
+	t->flags = 0;
+
+	return 1;
+}
+
 static gint
 lua_mimepart_get_length (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL) {
@@ -927,7 +1117,7 @@ lua_mimepart_get_length (lua_State * L)
 		return 1;
 	}
 
-	lua_pushnumber (L, part->parsed_data.len);
+	lua_pushinteger (L, part->parsed_data.len);
 
 	return 1;
 }
@@ -989,18 +1179,21 @@ lua_mimepart_get_type_common (lua_State * L, gboolean full)
 static gint
 lua_mimepart_get_type (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	return lua_mimepart_get_type_common (L, FALSE);
 }
 
 static gint
 lua_mimepart_get_type_full (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	return lua_mimepart_get_type_common (L, TRUE);
 }
 
 static gint
 lua_mimepart_get_cte (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL) {
@@ -1016,6 +1209,7 @@ lua_mimepart_get_cte (lua_State * L)
 static gint
 lua_mimepart_get_filename (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL || part->cd == NULL || part->cd->filename.len == 0) {
@@ -1029,7 +1223,7 @@ lua_mimepart_get_filename (lua_State * L)
 }
 
 static gint
-lua_mimepart_get_header_common (lua_State *L, gboolean full, gboolean raw)
+lua_mimepart_get_header_common (lua_State *L, enum rspamd_lua_task_header_type how)
 {
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 	const gchar *name;
@@ -1042,7 +1236,7 @@ lua_mimepart_get_header_common (lua_State *L, gboolean full, gboolean raw)
 		ar = rspamd_message_get_header_from_hash (part->raw_headers, NULL,
 				name, FALSE);
 
-		return rspamd_lua_push_header_array (L, ar, full, raw);
+		return rspamd_lua_push_header_array (L, ar, how);
 	}
 
 	lua_pushnil (L);
@@ -1053,24 +1247,35 @@ lua_mimepart_get_header_common (lua_State *L, gboolean full, gboolean raw)
 static gint
 lua_mimepart_get_header_full (lua_State * L)
 {
-	return lua_mimepart_get_header_common (L, TRUE, TRUE);
+	LUA_TRACE_POINT;
+	return lua_mimepart_get_header_common (L, RSPAMD_TASK_HEADER_PUSH_FULL);
 }
 
 static gint
 lua_mimepart_get_header (lua_State * L)
 {
-	return lua_mimepart_get_header_common (L, FALSE, FALSE);
+	LUA_TRACE_POINT;
+	return lua_mimepart_get_header_common (L, RSPAMD_TASK_HEADER_PUSH_SIMPLE);
 }
 
 static gint
 lua_mimepart_get_header_raw (lua_State * L)
 {
-	return lua_mimepart_get_header_common (L, FALSE, TRUE);
+	LUA_TRACE_POINT;
+	return lua_mimepart_get_header_common (L, RSPAMD_TASK_HEADER_PUSH_RAW);
+}
+
+static gint
+lua_mimepart_get_header_count (lua_State * L)
+{
+	LUA_TRACE_POINT;
+	return lua_mimepart_get_header_common (L, RSPAMD_TASK_HEADER_PUSH_COUNT);
 }
 
 static gint
 lua_mimepart_is_image (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL) {
@@ -1085,6 +1290,7 @@ lua_mimepart_is_image (lua_State * L)
 static gint
 lua_mimepart_is_archive (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL) {
@@ -1099,6 +1305,7 @@ lua_mimepart_is_archive (lua_State * L)
 static gint
 lua_mimepart_is_multipart (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL) {
@@ -1113,6 +1320,7 @@ lua_mimepart_is_multipart (lua_State * L)
 static gint
 lua_mimepart_is_text (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL) {
@@ -1127,6 +1335,7 @@ lua_mimepart_is_text (lua_State * L)
 static gint
 lua_mimepart_is_broken (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 
 	if (part == NULL) {
@@ -1147,6 +1356,7 @@ lua_mimepart_is_broken (lua_State * L)
 static gint
 lua_mimepart_get_image (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 	struct rspamd_image **pimg;
 
@@ -1169,6 +1379,7 @@ lua_mimepart_get_image (lua_State * L)
 static gint
 lua_mimepart_get_archive (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 	struct rspamd_archive **parch;
 
@@ -1191,6 +1402,7 @@ lua_mimepart_get_archive (lua_State * L)
 static gint
 lua_mimepart_get_children (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 	struct rspamd_mime_part **pcur, *cur;
 	guint i;
@@ -1220,6 +1432,7 @@ lua_mimepart_get_children (lua_State * L)
 static gint
 lua_mimepart_get_text (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 	struct rspamd_mime_text_part **ppart;
 
@@ -1242,6 +1455,7 @@ lua_mimepart_get_text (lua_State * L)
 static gint
 lua_mimepart_get_digest (lua_State * L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
 	gchar digestbuf[rspamd_cryptobox_HASHBYTES * 2 + 1];
 
@@ -1260,8 +1474,9 @@ lua_mimepart_get_digest (lua_State * L)
 static gint
 lua_mimepart_headers_foreach (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
-	gboolean full = FALSE, raw = FALSE;
+	enum rspamd_lua_task_header_type how = RSPAMD_TASK_HEADER_PUSH_SIMPLE;
 	struct rspamd_lua_regexp *re = NULL;
 	GList *cur;
 	struct rspamd_mime_header *hdr;
@@ -1272,8 +1487,8 @@ lua_mimepart_headers_foreach (lua_State *L)
 			lua_pushstring (L, "full");
 			lua_gettable (L, 3);
 
-			if (lua_isboolean (L, -1)) {
-				full = lua_toboolean (L, -1);
+			if (lua_isboolean (L, -1) && lua_toboolean (L, -1)) {
+				how = RSPAMD_TASK_HEADER_PUSH_FULL;
 			}
 
 			lua_pop (L, 1);
@@ -1281,8 +1496,8 @@ lua_mimepart_headers_foreach (lua_State *L)
 			lua_pushstring (L, "raw");
 			lua_gettable (L, 3);
 
-			if (lua_isboolean (L, -1)) {
-				raw = lua_toboolean (L, -1);
+			if (lua_isboolean (L, -1) && lua_toboolean (L, -1)) {
+				how = RSPAMD_TASK_HEADER_PUSH_RAW;
 			}
 
 			lua_pop (L, 1);
@@ -1315,7 +1530,7 @@ lua_mimepart_headers_foreach (lua_State *L)
 				old_top = lua_gettop (L);
 				lua_pushvalue (L, 2);
 				lua_pushstring (L, hdr->name);
-				rspamd_lua_push_header (L, hdr, full, raw);
+				rspamd_lua_push_header (L, hdr, how);
 
 				if (lua_pcall (L, 2, LUA_MULTRET, 0) != 0) {
 					msg_err ("call to header_foreach failed: %s",
