@@ -328,6 +328,7 @@ lua_rsa_privkey_load_base64 (lua_State *L)
 {
 	RSA *rsa = NULL, **prsa;
 	BIO *b;
+	EVP_PKEY *evp = NULL;
 	struct rspamd_lua_text *t;
 	const gchar *data;
 	guchar *decoded;
@@ -357,21 +358,31 @@ lua_rsa_privkey_load_base64 (lua_State *L)
 		}
 
 		b = BIO_new_mem_buf (decoded, dec_len);
-		g_free (decoded);
-		rsa = d2i_RSAPrivateKey_bio (b, NULL);
 
-		if (rsa == NULL) {
-			msg_err ("cannot open private key from data, %s",
+		if (d2i_PrivateKey_bio (b, &evp) != NULL) {
+			rsa = EVP_PKEY_get1_RSA (evp);
+
+			if (rsa == NULL) {
+				msg_err ("cannot open RSA private key from data, %s",
+						ERR_error_string (ERR_get_error (), NULL));
+				lua_pushnil (L);
+			}
+			else {
+				prsa = lua_newuserdata (L, sizeof (RSA *));
+				rspamd_lua_setclass (L, "rspamd{rsa_privkey}", -1);
+				*prsa = rsa;
+			}
+
+			EVP_PKEY_free (evp);
+		}
+		else {
+			msg_err ("cannot open EVP private key from data, %s",
 					ERR_error_string (ERR_get_error (), NULL));
 			lua_pushnil (L);
 		}
-		else {
-			prsa = lua_newuserdata (L, sizeof (RSA *));
-			rspamd_lua_setclass (L, "rspamd{rsa_privkey}", -1);
-			*prsa = rsa;
-		}
 
 		BIO_free (b);
+		g_free (decoded);
 	}
 	else {
 		return luaL_error (L, "invalid arguments");
