@@ -22,6 +22,7 @@
 #include "linenoise.h"
 #include <math.h>
 #include <glob.h>
+#include <zlib.h>
 
 #include "unicode/uspoof.h"
 
@@ -44,7 +45,7 @@ LUA_FUNCTION_DEF (util, create_event_base);
  */
 LUA_FUNCTION_DEF (util, load_rspamd_config);
 /***
- * @function util.config_from_ucl(any)
+ * @function util.config_from_ucl(any, string)
  * Load rspamd config from ucl reperesented by any lua table
  * @return {confg} new configuration object suitable for access
  */
@@ -355,6 +356,24 @@ LUA_FUNCTION_DEF (util, zstd_compress);
  */
 LUA_FUNCTION_DEF (util, zstd_decompress);
 
+/***
+ * @function util.gzip_decompress(data)
+ * Decompresses input using gzip algorithm
+ *
+ * @param {string/rspamd_text} data compressed data
+ * @return {error,rspamd_text} pair of error + decompressed text
+ */
+LUA_FUNCTION_DEF (util, gzip_decompress);
+
+/***
+ * @function util.gzip_compress(data)
+ * Compresses input using gzip compression
+ *
+ * @param {string/rspamd_text} data input data
+ * @return {rspamd_text} compressed data
+ */
+LUA_FUNCTION_DEF (util, gzip_compress);
+
 /***
  * @function util.normalize_prob(prob, [bias = 0.5])
  * Normalize probabilities using polynom
@@ -395,7 +414,7 @@ LUA_FUNCTION_DEF (util, readpassphrase);
 /***
  * @function util.file_exists(file)
  * Checks if a specified file exists and is available for reading
- * @return {boolean} true if file exists
+ * @return {boolean,string} true if file exists + string error if not
  */
 LUA_FUNCTION_DEF (util, file_exists);
 
@@ -558,6 +577,8 @@ static const struct luaL_reg utillib_f[] = {
 	LUA_INTERFACE_DEF (util, random_hex),
 	LUA_INTERFACE_DEF (util, zstd_compress),
 	LUA_INTERFACE_DEF (util, zstd_decompress),
+	LUA_INTERFACE_DEF (util, gzip_compress),
+	LUA_INTERFACE_DEF (util, gzip_decompress),
 	LUA_INTERFACE_DEF (util, normalize_prob),
 	LUA_INTERFACE_DEF (util, caseless_hash),
 	LUA_INTERFACE_DEF (util, caseless_hash_fast),
@@ -608,6 +629,7 @@ lua_check_int64 (lua_State * L, gint pos)
 static gint
 lua_util_create_event_base (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct event_base **pev_base;
 
 	pev_base = lua_newuserdata (L, sizeof (struct event_base *));
@@ -620,15 +642,17 @@ lua_util_create_event_base (lua_State *L)
 static gint
 lua_util_load_rspamd_config (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg, **pcfg;
 	const gchar *cfg_name;
 
 	cfg_name = luaL_checkstring (L, 1);
 
 	if (cfg_name) {
-		cfg = rspamd_config_new ();
+		cfg = rspamd_config_new (RSPAMD_CONFIG_INIT_SKIP_LUA);
+		cfg->lua_state = L;
 
-		if (rspamd_config_read (cfg, cfg_name, NULL, NULL, NULL, NULL)) {
+		if (rspamd_config_read (cfg, cfg_name, NULL, NULL, NULL)) {
 			msg_err_config ("cannot load config from %s", cfg_name);
 			lua_pushnil (L);
 		}
@@ -643,23 +667,75 @@ lua_util_load_rspamd_config (lua_State *L)
 	return 1;
 }
 
+static gint
+parse_config_options (const char *str_options)
+{
+	gint ret = 0;
+	gchar **vec;
+	const gchar *str;
+	guint i, l;
+
+	vec = g_strsplit_set (str_options, ",;", -1);
+	if (vec) {
+		l = g_strv_length (vec);
+		for (i = 0; i < l; i ++) {
+			str = vec[i];
+
+			if (g_ascii_strcasecmp (str, "INIT_URL") == 0) {
+				ret |= RSPAMD_CONFIG_INIT_URL;
+			} else if (g_ascii_strcasecmp (str, "INIT_LIBS") == 0) {
+				ret |= RSPAMD_CONFIG_INIT_LIBS;
+			} else if (g_ascii_strcasecmp (str, "INIT_SYMCACHE") == 0) {
+				ret |= RSPAMD_CONFIG_INIT_SYMCACHE;
+			} else if (g_ascii_strcasecmp (str, "INIT_VALIDATE") == 0) {
+				ret |= RSPAMD_CONFIG_INIT_VALIDATE;
+			} else if (g_ascii_strcasecmp (str, "INIT_NO_TLD") == 0) {
+				ret |= RSPAMD_CONFIG_INIT_NO_TLD;
+			} else if (g_ascii_strcasecmp (str, "INIT_PRELOAD_MAPS") == 0) {
+				ret |= RSPAMD_CONFIG_INIT_PRELOAD_MAPS;
+			} else {
+				msg_warn ("bad type: %s", str);
+			}
+		}
+
+		g_strfreev (vec);
+	}
+
+	return ret;
+}
+
 static gint
 lua_util_config_from_ucl (lua_State *L)
 {
-	struct rspamd_config *cfg, **pcfg;
+	LUA_TRACE_POINT;
+	struct rspamd_config *cfg = NULL, **pcfg;
 	struct rspamd_rcl_section *top;
 	GError *err = NULL;
 	ucl_object_t *obj;
+	const char *str_options = NULL;
+	gint int_options = 0;
+
 
 	obj = ucl_object_lua_import (L, 1);
+	if (lua_gettop (L) == 2) {
+		if (lua_type (L, 2) == LUA_TSTRING) {
+			str_options = lua_tostring (L, 2);
+			int_options = parse_config_options(str_options);
+		}
+		else {
+			msg_err_config ("config_from_ucl: second parameter is expected to be string");
+			ucl_object_unref (obj);
+			lua_pushnil (L);
+		}
+	}
 
 	if (obj) {
-		cfg = g_malloc0 (sizeof (struct rspamd_config));
-		cfg = rspamd_config_new ();
+		cfg = rspamd_config_new (RSPAMD_CONFIG_INIT_SKIP_LUA);
+		cfg->lua_state = L;
 
 		cfg->rcl_obj = obj;
 		cfg->cache = rspamd_symbols_cache_new (cfg);
-		top = rspamd_rcl_config_init (cfg);
+		top = rspamd_rcl_config_init (cfg, NULL);
 
 		if (!rspamd_rcl_parse (top, cfg, cfg, cfg->cfg_pool, cfg->rcl_obj, &err)) {
 			msg_err_config ("rcl parse error: %s", err->message);
@@ -667,7 +743,7 @@ lua_util_config_from_ucl (lua_State *L)
 			lua_pushnil (L);
 		}
 		else {
-			rspamd_config_post_load (cfg, 0);
+			rspamd_config_post_load (cfg, int_options);
 			pcfg = lua_newuserdata (L, sizeof (struct rspamd_config *));
 			rspamd_lua_setclass (L, "rspamd{config}", -1);
 			*pcfg = cfg;
@@ -691,6 +767,7 @@ lua_util_task_fin (struct rspamd_task *task, void *ud)
 static gint
 lua_util_process_message (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config (L, 1);
 	const gchar *message;
 	gsize mlen;
@@ -751,6 +828,7 @@ lua_util_process_message (lua_State *L)
 static gint
 lua_util_encode_base64 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 	const gchar *s = NULL;
 	gchar *out;
@@ -823,6 +901,7 @@ lua_util_encode_base64 (lua_State *L)
 static gint
 lua_util_decode_base64 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 	const gchar *s = NULL;
 	gsize inlen, outlen;
@@ -879,6 +958,7 @@ lua_util_decode_base64 (lua_State *L)
 static gint
 lua_util_encode_base32 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 	const gchar *s = NULL;
 	gchar *out;
@@ -922,6 +1002,7 @@ lua_util_encode_base32 (lua_State *L)
 static gint
 lua_util_decode_base32 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 	const gchar *s = NULL;
 	gsize inlen, outlen;
@@ -955,6 +1036,7 @@ lua_util_decode_base32 (lua_State *L)
 static gint
 lua_util_decode_url (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 	const gchar *s = NULL;
 	gsize inlen;
@@ -990,14 +1072,15 @@ lua_util_decode_url (lua_State *L)
 static gint
 lua_util_tokenize_text (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *in = NULL;
 	gsize len, pos, ex_len, i;
 	GList *exceptions = NULL, *cur;
 	struct rspamd_lua_text *t;
 	struct rspamd_process_exception *ex;
+	UText utxt = UTEXT_INITIALIZER;
 	GArray *res;
 	rspamd_stat_token_t *w;
-	gboolean compat = FALSE;
 
 	if (lua_type (L, 1) == LUA_TSTRING) {
 		in = luaL_checklstring (L, 1, &len);
@@ -1043,15 +1126,20 @@ lua_util_tokenize_text (lua_State *L)
 		lua_pop (L, 1);
 	}
 
-	if (lua_gettop (L) > 2 && lua_type (L, 3) == LUA_TBOOLEAN) {
-		compat = lua_toboolean (L, 3);
-	}
-
 	if (exceptions) {
 		exceptions = g_list_reverse (exceptions);
 	}
 
-	res = rspamd_tokenize_text ((gchar *)in, len, TRUE, NULL, exceptions, compat,
+	UErrorCode uc_err = U_ZERO_ERROR;
+	utext_openUTF8 (&utxt,
+			in,
+			len,
+			&uc_err);
+
+	res = rspamd_tokenize_text ((gchar *)in, len,
+			&utxt,
+			RSPAMD_TOKENIZE_UTF, NULL,
+			exceptions,
 			NULL);
 
 	if (res == NULL) {
@@ -1075,6 +1163,7 @@ lua_util_tokenize_text (lua_State *L)
 	}
 
 	g_list_free (exceptions);
+	utext_close (&utxt);
 
 	return 1;
 }
@@ -1082,6 +1171,7 @@ lua_util_tokenize_text (lua_State *L)
 static gint
 lua_util_tanh (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gdouble in = luaL_checknumber (L, 1);
 
 	lua_pushnumber (L, tanh (in));
@@ -1092,6 +1182,7 @@ lua_util_tanh (lua_State *L)
 static gint
 lua_util_parse_html (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 	const gchar *start = NULL;
 	gsize len;
@@ -1139,6 +1230,7 @@ lua_util_parse_html (lua_State *L)
 static gint
 lua_util_levenshtein_distance (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *s1, *s2;
 	gsize s1len, s2len;
 	gint dist = 0;
@@ -1156,7 +1248,7 @@ lua_util_levenshtein_distance (lua_State *L)
 				replace_cost);
 	}
 
-	lua_pushnumber (L, dist);
+	lua_pushinteger (L, dist);
 
 	return 1;
 }
@@ -1164,6 +1256,7 @@ lua_util_levenshtein_distance (lua_State *L)
 static gint
 lua_util_parse_addr (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	GPtrArray *addrs;
 	gsize len;
 	const gchar *str = luaL_checklstring (L, 1, &len);
@@ -1207,6 +1300,7 @@ lua_util_parse_addr (lua_State *L)
 static gint
 lua_util_fold_header (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *name, *value, *how, *stop_chars = NULL;
 	GString *folded;
 
@@ -1256,6 +1350,7 @@ lua_util_fold_header (lua_State *L)
 static gint
 lua_util_is_uppercase (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *str;
 	gsize sz;
 	gint32 i = 0;
@@ -1294,6 +1389,7 @@ lua_util_is_uppercase (lua_State *L)
 static gint
 lua_util_humanize_number (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gdouble number = luaL_checknumber (L, 1);
 	gchar numbuf[32];
 
@@ -1307,6 +1403,7 @@ lua_util_humanize_number (lua_State *L)
 static gint
 lua_util_get_tld (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *host;
 	gsize hostlen;
 	rspamd_ftok_t tld;
@@ -1332,6 +1429,7 @@ lua_util_get_tld (lua_State *L)
 static gint
 lua_util_glob (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *pattern;
 	glob_t gl;
 	gint top, i, flags;
@@ -1363,6 +1461,7 @@ lua_util_glob (lua_State *L)
 static gint
 lua_util_parse_mail_address (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	GPtrArray *addrs;
 	gsize len;
 	const gchar *str = luaL_checklstring (L, 1, &len);
@@ -1406,6 +1505,7 @@ lua_util_parse_mail_address (lua_State *L)
 static gint
 lua_util_strlen_utf8 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *str, *end;
 	gsize len;
 
@@ -1420,7 +1520,7 @@ lua_util_strlen_utf8 (lua_State *L)
 					+ (len - (end - str)) /* raw part */;
 		}
 
-		lua_pushnumber (L, len);
+		lua_pushinteger (L, len);
 	}
 	else {
 		return luaL_error (L, "invalid arguments");
@@ -1432,6 +1532,7 @@ lua_util_strlen_utf8 (lua_State *L)
 static gint
 lua_util_strcasecmp_utf8 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *str1, *str2;
 	gsize len1, len2;
 	gint ret = -1;
@@ -1452,13 +1553,14 @@ lua_util_strcasecmp_utf8 (lua_State *L)
 		return luaL_error (L, "invalid arguments");
 	}
 
-	lua_pushnumber (L, ret);
+	lua_pushinteger (L, ret);
 	return 1;
 }
 
 static gint
 lua_util_strcasecmp_ascii (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *str1, *str2;
 	gsize len1, len2;
 	gint ret = -1;
@@ -1479,13 +1581,14 @@ lua_util_strcasecmp_ascii (lua_State *L)
 		return luaL_error (L, "invalid arguments");
 	}
 
-	lua_pushnumber (L, ret);
+	lua_pushinteger (L, ret);
 	return 1;
 }
 
 static gint
 lua_util_strequal_caseless (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *str1, *str2;
 	gsize len1, len2;
 	gint ret = -1;
@@ -1513,6 +1616,7 @@ lua_util_strequal_caseless (lua_State *L)
 static gint
 lua_util_get_ticks (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gdouble ticks;
 	gboolean rdtsc = FALSE;
 
@@ -1529,6 +1633,7 @@ lua_util_get_ticks (lua_State *L)
 static gint
 lua_util_get_time (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gdouble seconds;
 	struct timeval tv;
 
@@ -1547,6 +1652,7 @@ lua_util_get_time (lua_State *L)
 static gint
 lua_util_time_to_string (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gdouble seconds;
 	struct timeval tv;
 	char timebuf[128];
@@ -1572,6 +1678,7 @@ lua_util_time_to_string (lua_State *L)
 static gint
 lua_util_stat (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *fpath;
 	struct stat st;
 
@@ -1587,11 +1694,11 @@ lua_util_stat (lua_State *L)
 			lua_createtable (L, 0, 3);
 
 			lua_pushstring (L, "size");
-			lua_pushnumber (L, st.st_size);
+			lua_pushinteger (L, st.st_size);
 			lua_settable (L, -3);
 
 			lua_pushstring (L, "mtime");
-			lua_pushnumber (L, st.st_mtime);
+			lua_pushinteger (L, st.st_mtime);
 			lua_settable (L, -3);
 
 			lua_pushstring (L, "type");
@@ -1617,6 +1724,7 @@ lua_util_stat (lua_State *L)
 static gint
 lua_util_unlink (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *fpath;
 	gint ret;
 
@@ -1644,6 +1752,7 @@ lua_util_unlink (lua_State *L)
 static gint
 lua_util_lock_file (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *fpath;
 	gint fd = -1;
 	gboolean own = FALSE;
@@ -1690,7 +1799,7 @@ lua_util_lock_file (lua_State *L)
 			return 2;
 		}
 
-		lua_pushnumber (L, fd);
+		lua_pushinteger (L, fd);
 	}
 	else {
 		return luaL_error (L, "invalid arguments");
@@ -1702,6 +1811,7 @@ lua_util_lock_file (lua_State *L)
 static gint
 lua_util_unlock_file (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gint fd = -1, ret, serrno;
 	gboolean do_close = TRUE;
 
@@ -1715,7 +1825,7 @@ lua_util_unlock_file (lua_State *L)
 #endif
 
 	if (lua_isnumber (L, 1)) {
-		fd = lua_tonumber (L, 1);
+		fd = lua_tointeger (L, 1);
 
 		if (lua_isboolean (L, 2)) {
 			do_close = lua_toboolean (L, 2);
@@ -1752,6 +1862,7 @@ lua_util_unlock_file (lua_State *L)
 static gint
 lua_util_create_file (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gint fd, mode = 00644;
 	const gchar *fpath;
 
@@ -1759,7 +1870,7 @@ lua_util_create_file (lua_State *L)
 
 	if (fpath) {
 		if (lua_isnumber (L, 2)) {
-			mode = lua_tonumber (L, 2);
+			mode = lua_tointeger (L, 2);
 		}
 
 		fd = rspamd_file_xopen (fpath, O_RDWR | O_CREAT | O_EXCL, mode, 0);
@@ -1771,7 +1882,7 @@ lua_util_create_file (lua_State *L)
 			return 2;
 		}
 
-		lua_pushnumber (L, fd);
+		lua_pushinteger (L, fd);
 	}
 	else {
 		return luaL_error (L, "invalid arguments");
@@ -1783,10 +1894,11 @@ lua_util_create_file (lua_State *L)
 static gint
 lua_util_close_file (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gint fd = -1;
 
 	if (lua_isnumber (L, 1)) {
-		fd = lua_tonumber (L, 1);
+		fd = lua_tointeger (L, 1);
 
 		if (close (fd) == -1) {
 			lua_pushboolean (L, false);
@@ -1807,10 +1919,11 @@ lua_util_close_file (lua_State *L)
 static gint
 lua_util_random_hex (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gchar *buf;
 	gint buflen;
 
-	buflen = lua_tonumber (L, 1);
+	buflen = lua_tointeger (L, 1);
 
 	if (buflen <= 0) {
 		return luaL_error (L, "invalid arguments");
@@ -1827,6 +1940,7 @@ lua_util_random_hex (lua_State *L)
 static gint
 lua_util_zstd_compress (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t = NULL, *res, tmp;
 	gsize sz, r;
 
@@ -1874,6 +1988,7 @@ lua_util_zstd_compress (lua_State *L)
 static gint
 lua_util_zstd_decompress (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t = NULL, *res;
 	gsize outlen, sz, r;
 	ZSTD_DStream *zstream;
@@ -1943,9 +2058,178 @@ lua_util_zstd_decompress (lua_State *L)
 	return 2;
 }
 
+static gint
+lua_util_gzip_compress (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_lua_text *t = NULL, *res, tmp;
+	gsize sz;
+	z_stream strm;
+	gint rc;
+	guchar *p;
+	gsize remain;
+
+	if (lua_type (L, 1) == LUA_TSTRING) {
+		t = &tmp;
+		t->start = lua_tolstring (L, 1, &sz);
+		t->len = sz;
+	}
+	else {
+		t = lua_check_text (L, 1);
+	}
+
+	if (t == NULL || t->start == NULL) {
+		return luaL_error (L, "invalid arguments");
+	}
+
+
+
+	memset (&strm, 0, sizeof (strm));
+	rc = deflateInit2 (&strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED,
+			MAX_WBITS + 16, MAX_MEM_LEVEL - 1, Z_DEFAULT_STRATEGY);
+
+	if (rc != Z_OK) {
+		return luaL_error (L, "cannot init zlib: %s", zError (rc));
+	}
+
+	sz = deflateBound (&strm, t->len);
+
+	strm.avail_in = t->len;
+	strm.next_in = (guchar *)t->start;
+
+	res = lua_newuserdata (L, sizeof (*res));
+	res->start = g_malloc (sz);
+	res->flags = RSPAMD_TEXT_FLAG_OWN;
+	rspamd_lua_setclass (L, "rspamd{text}", -1);
+
+	p = (guchar *)res->start;
+	remain = sz;
+
+	while (strm.avail_in != 0) {
+		strm.avail_out = remain;
+		strm.next_out = p;
+
+		rc = deflate (&strm, Z_FINISH);
+
+		if (rc != Z_OK && rc != Z_BUF_ERROR) {
+			if (rc == Z_STREAM_END) {
+				break;
+			}
+			else {
+				msg_err ("cannot compress data: %s", zError (rc));
+				lua_pop (L, 1); /* Text will be freed here */
+				lua_pushnil (L);
+				deflateEnd (&strm);
+
+				return 1;
+			}
+		}
+
+		res->len = strm.total_out;
+
+		if (strm.avail_out == 0 && strm.avail_in != 0) {
+			/* Need to allocate more */
+			remain = res->len;
+			res->start = g_realloc ((gpointer)res->start, strm.avail_in + sz);
+			sz = strm.avail_in + sz;
+			p = (guchar *)res->start + remain;
+			remain = sz - remain;
+		}
+	}
+
+	deflateEnd (&strm);
+	res->len = strm.total_out;
+
+	return 1;
+}
+
+
+static gint
+lua_util_gzip_decompress (lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct rspamd_lua_text *t = NULL, *res, tmp;
+	gsize sz;
+	z_stream strm;
+	gint rc;
+	guchar *p;
+	gsize remain;
+
+	if (lua_type (L, 1) == LUA_TSTRING) {
+		t = &tmp;
+		t->start = lua_tolstring (L, 1, &sz);
+		t->len = sz;
+	}
+	else {
+		t = lua_check_text (L, 1);
+	}
+
+	if (t == NULL || t->start == NULL) {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	memset (&strm, 0, sizeof (strm));
+	/* windowBits +16 to decode gzip, zlib 1.2.0.4+ */
+	rc = inflateInit2 (&strm, MAX_WBITS + 16);
+
+	if (rc != Z_OK) {
+		return luaL_error (L, "cannot init zlib");
+	}
+
+	strm.avail_in = t->len;
+	strm.next_in = (guchar *)t->start;
+
+	res = lua_newuserdata (L, sizeof (*res));
+	res->start = g_malloc (sz);
+	res->flags = RSPAMD_TEXT_FLAG_OWN;
+	rspamd_lua_setclass (L, "rspamd{text}", -1);
+
+	p = (guchar *)res->start;
+	remain = sz;
+
+	while (strm.avail_in != 0) {
+		strm.avail_out = remain;
+		strm.next_out = p;
+
+		rc = inflate (&strm, Z_FINISH);
+
+		if (rc != Z_OK && rc != Z_BUF_ERROR) {
+			if (rc == Z_STREAM_END) {
+				break;
+			}
+			else {
+				msg_err ("cannot decompress data: %s", zError (rc));
+				lua_pop (L, 1); /* Text will be freed here */
+				lua_pushnil (L);
+				inflateEnd (&strm);
+
+				return 1;
+			}
+		}
+
+		res->len = strm.total_out;
+
+		if (strm.avail_out == 0 && strm.avail_in != 0) {
+			/* Need to allocate more */
+			remain = res->len;
+			res->start = g_realloc ((gpointer)res->start, strm.avail_in + sz);
+			sz = strm.avail_in + sz;
+			p = (guchar *)res->start + remain;
+			remain = sz - remain;
+		}
+	}
+
+	inflateEnd (&strm);
+	res->len = strm.total_out;
+
+	return 2;
+}
+
+
 static gint
 lua_util_normalize_prob (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gdouble x, bias = 0.5;
 
 	x = lua_tonumber (L, 1);
@@ -1962,6 +2246,7 @@ lua_util_normalize_prob (lua_State *L)
 static gint
 lua_util_caseless_hash (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	guint64 seed = 0xdeadbabe, h;
 	struct rspamd_lua_text *t = NULL;
 	gint64 *r;
@@ -1998,6 +2283,7 @@ lua_util_caseless_hash (lua_State *L)
 static gint
 lua_util_caseless_hash_fast (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	guint64 seed = 0xdeadbabe, h;
 	struct rspamd_lua_text *t = NULL;
 	gsize sz;
@@ -2041,6 +2327,7 @@ lua_util_caseless_hash_fast (lua_State *L)
 static gint
 lua_util_is_utf_spoofed (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gsize l1, l2;
 	gint ret, nres = 2;
 	const gchar *s1 = lua_tolstring (L, 1, &l1),
@@ -2122,6 +2409,7 @@ lua_util_is_utf_spoofed (lua_State *L)
 static gint
 lua_util_get_hostname (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gchar *hostbuf;
 	gsize hostlen;
 
@@ -2146,6 +2434,7 @@ lua_util_get_hostname (lua_State *L)
 static gint
 lua_util_is_valid_utf8 (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *str;
 	gsize len;
 
@@ -2164,6 +2453,7 @@ lua_util_is_valid_utf8 (lua_State *L)
 static gint
 lua_util_readline (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *prompt = NULL;
 	gchar *input;
 
@@ -2188,6 +2478,7 @@ lua_util_readline (lua_State *L)
 static gint
 lua_util_readpassphrase (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	gchar test_password[8192];
 	gsize r;
 
@@ -2209,21 +2500,32 @@ lua_util_readpassphrase (lua_State *L)
 static gint
 lua_util_file_exists (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *fname = luaL_checkstring (L, 1);
+	gint serrno;
 
 	if (fname) {
-		lua_pushboolean (L, access (fname, R_OK) != -1);
+		if (access (fname, R_OK) == -1) {
+			serrno = errno;
+			lua_pushboolean (L, false);
+			lua_pushstring (L, strerror (serrno));
+		}
+		else {
+			lua_pushboolean (L, true);
+			lua_pushnil (L);
+		}
 	}
 	else {
 		return luaL_error (L, "invalid arguments");
 	}
 
-	return 1;
+	return 2;
 }
 
 static gint
 lua_util_mkdir (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	const gchar *dname = luaL_checkstring (L, 1);
 	gboolean recursive = FALSE;
 	gint r = -1;
@@ -2287,6 +2589,7 @@ lua_util_mkdir (lua_State *L)
 static gint
 lua_util_umask (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	mode_t mask = 0, old;
 
 	if (lua_type (L, 1) == LUA_TSTRING) {
@@ -2310,7 +2613,7 @@ lua_util_umask (lua_State *L)
 
 	old = umask (mask);
 
-	lua_pushnumber (L, old);
+	lua_pushinteger (L, old);
 
 	return 1;
 }
@@ -2318,6 +2621,7 @@ lua_util_umask (lua_State *L)
 static gint
 lua_util_isatty (lua_State *L)
 {
+	LUA_TRACE_POINT;
 	if (isatty (STDOUT_FILENO)) {
 		lua_pushboolean (L, true);
 	}
@@ -3128,7 +3432,7 @@ lua_ev_base_loop (lua_State *L)
 	}
 
 	int ret = event_base_loop (ev_base, flags);
-	lua_pushnumber (L, ret);
+	lua_pushinteger (L, ret);
 
 	return 1;
 }
