@@ -61,12 +61,13 @@ gint chartable_module_config (struct rspamd_config *cfg);
 gint chartable_module_reconfig (struct rspamd_config *cfg);
 
 module_t chartable_module = {
-	"chartable",
-	chartable_module_init,
-	chartable_module_config,
-	chartable_module_reconfig,
-	NULL,
-	RSPAMD_MODULE_VER
+		"chartable",
+		chartable_module_init,
+		chartable_module_config,
+		chartable_module_reconfig,
+		NULL,
+		RSPAMD_MODULE_VER,
+		(guint)-1,
 };
 
 struct chartable_ctx {
@@ -75,23 +76,26 @@ struct chartable_ctx {
 	const gchar *url_symbol;
 	double threshold;
 	guint max_word_len;
-
-	rspamd_mempool_t *chartable_pool;
 };
 
-static struct chartable_ctx *chartable_module_ctx = NULL;
+static inline struct chartable_ctx *
+chartable_get_context (struct rspamd_config *cfg)
+{
+	return (struct chartable_ctx *)g_ptr_array_index (cfg->c_modules,
+			chartable_module.ctx_offset);
+}
+
 static void chartable_symbol_callback (struct rspamd_task *task, void *unused);
 static void chartable_url_symbol_callback (struct rspamd_task *task, void *unused);
 
 gint
 chartable_module_init (struct rspamd_config *cfg, struct module_ctx **ctx)
 {
-	if (chartable_module_ctx == NULL) {
-		chartable_module_ctx = g_malloc (sizeof (struct chartable_ctx));
+	struct chartable_ctx *chartable_module_ctx;
 
-		chartable_module_ctx->chartable_pool = rspamd_mempool_new (rspamd_mempool_suggest_size (), NULL);
-		chartable_module_ctx->max_word_len = 10;
-	}
+	chartable_module_ctx = rspamd_mempool_alloc0 (cfg->cfg_pool,
+			sizeof (*chartable_module_ctx));
+	chartable_module_ctx->max_word_len = 10;
 
 	*ctx = (struct module_ctx *)chartable_module_ctx;
 
@@ -104,6 +108,7 @@ chartable_module_config (struct rspamd_config *cfg)
 {
 	const ucl_object_t *value;
 	gint res = TRUE;
+	struct chartable_ctx *chartable_module_ctx = chartable_get_context (cfg);
 
 	if (!rspamd_config_is_module_enabled (cfg, "chartable")) {
 		return TRUE;
@@ -164,9 +169,6 @@ chartable_module_config (struct rspamd_config *cfg)
 gint
 chartable_module_reconfig (struct rspamd_config *cfg)
 {
-	rspamd_mempool_delete (chartable_module_ctx->chartable_pool);
-	chartable_module_ctx->chartable_pool = rspamd_mempool_new (1024, NULL);
-
 	return chartable_module_config (cfg);
 }
 
@@ -347,9 +349,10 @@ rspamd_can_alias_latin (gint ch)
 
 static gdouble
 rspamd_chartable_process_word_utf (struct rspamd_task *task,
-		rspamd_stat_token_t *w,
-		gboolean is_url,
-		guint *ncap)
+								   rspamd_stat_token_t *w,
+								   gboolean is_url,
+								   guint *ncap,
+								   struct chartable_ctx *chartable_module_ctx)
 {
 	const gchar *p, *end;
 	gdouble badness = 0.0;
@@ -465,8 +468,9 @@ rspamd_chartable_process_word_utf (struct rspamd_task *task,
 
 static gdouble
 rspamd_chartable_process_word_ascii (struct rspamd_task *task,
-		rspamd_stat_token_t *w,
-		gboolean is_url)
+									 rspamd_stat_token_t *w,
+									 gboolean is_url,
+									 struct chartable_ctx *chartable_module_ctx)
 {
 	const guchar *p, *end;
 	gdouble badness = 0.0;
@@ -549,28 +553,30 @@ rspamd_chartable_process_word_ascii (struct rspamd_task *task,
 
 static void
 rspamd_chartable_process_part (struct rspamd_task *task,
-		struct rspamd_mime_text_part *part)
+							   struct rspamd_mime_text_part *part,
+							   struct chartable_ctx *chartable_module_ctx)
 {
 	rspamd_stat_token_t *w;
 	guint i, ncap = 0;
 	gdouble cur_score = 0.0;
 
-	if (part == NULL || part->normalized_words == NULL ||
-			part->normalized_words->len == 0) {
+	if (part == NULL || part->utf_words == NULL ||
+			part->utf_words->len == 0) {
 		return;
 	}
 
-	for (i = 0; i < part->normalized_words->len; i++) {
-		w = &g_array_index (part->normalized_words, rspamd_stat_token_t, i);
+	for (i = 0; i < part->utf_words->len; i++) {
+		w = &g_array_index (part->utf_words, rspamd_stat_token_t, i);
 
 		if (w->len > 0 && (w->flags & RSPAMD_STAT_TOKEN_FLAG_TEXT)) {
 
 			if (IS_PART_UTF (part)) {
 				cur_score += rspamd_chartable_process_word_utf (task, w, FALSE,
-						&ncap);
+						&ncap, chartable_module_ctx);
 			}
 			else {
-				cur_score += rspamd_chartable_process_word_ascii (task, w, FALSE);
+				cur_score += rspamd_chartable_process_word_ascii (task, w,
+						FALSE, chartable_module_ctx);
 			}
 		}
 	}
@@ -582,7 +588,7 @@ rspamd_chartable_process_part (struct rspamd_task *task,
 	 */
 	part->capital_letters += ncap;
 
-	cur_score /= (gdouble)part->normalized_words->len;
+	cur_score /= (gdouble)part->utf_words->len;
 
 	if (cur_score > 2.0) {
 		cur_score = 2.0;
@@ -600,10 +606,11 @@ chartable_symbol_callback (struct rspamd_task *task, void *unused)
 {
 	guint i;
 	struct rspamd_mime_text_part *part;
+	struct chartable_ctx *chartable_module_ctx = chartable_get_context (task->cfg);
 
 	for (i = 0; i < task->text_parts->len; i ++) {
 		part = g_ptr_array_index (task->text_parts, i);
-		rspamd_chartable_process_part (task, part);
+		rspamd_chartable_process_part (task, part, chartable_module_ctx);
 	}
 
 	if (task->subject != NULL) {
@@ -612,18 +619,27 @@ chartable_symbol_callback (struct rspamd_task *task, void *unused)
 		guint i;
 		gdouble cur_score = 0.0;
 
-		words = rspamd_tokenize_text (task->subject, strlen (task->subject),
-				TRUE,
+		UText utxt = UTEXT_INITIALIZER;
+		UErrorCode uc_err = U_ZERO_ERROR;
+		gsize slen = strlen (task->subject);
+
+		utext_openUTF8 (&utxt,
+				task->subject,
+				slen,
+				&uc_err);
+
+		words = rspamd_tokenize_text (task->subject, slen,
+				&utxt,
+				RSPAMD_TOKENIZE_UTF,
 				NULL,
 				NULL,
-				FALSE,
 				NULL);
 
 		if (words && words->len > 0) {
 			for (i = 0; i < words->len; i++) {
 				w = &g_array_index (words, rspamd_stat_token_t, i);
 				cur_score += rspamd_chartable_process_word_utf (task, w, FALSE,
-						NULL);
+						NULL, chartable_module_ctx);
 			}
 
 			cur_score /= (gdouble)words->len;
@@ -642,6 +658,8 @@ chartable_symbol_callback (struct rspamd_task *task, void *unused)
 		if (words) {
 			g_array_free (words, TRUE);
 		}
+
+		utext_close (&utxt);
 	}
 }
 
@@ -653,6 +671,7 @@ chartable_url_symbol_callback (struct rspamd_task *task, void *unused)
 	gpointer k, v;
 	rspamd_stat_token_t w;
 	gdouble cur_score = 0.0;
+	struct chartable_ctx *chartable_module_ctx = chartable_get_context (task->cfg);
 
 	g_hash_table_iter_init (&it, task->urls);
 
@@ -669,10 +688,12 @@ chartable_url_symbol_callback (struct rspamd_task *task, void *unused)
 			w.len = u->hostlen;
 
 			if (g_utf8_validate (w.begin, w.len, NULL)) {
-				cur_score += rspamd_chartable_process_word_utf (task, &w, TRUE, NULL);
+				cur_score += rspamd_chartable_process_word_utf (task, &w,
+						TRUE, NULL, chartable_module_ctx);
 			}
 			else {
-				cur_score += rspamd_chartable_process_word_ascii (task, &w, TRUE);
+				cur_score += rspamd_chartable_process_word_ascii (task, &w,
+						TRUE, chartable_module_ctx);
 			}
 		}
 	}
@@ -692,10 +713,12 @@ chartable_url_symbol_callback (struct rspamd_task *task, void *unused)
 			w.len = u->hostlen;
 
 			if (g_utf8_validate (w.begin, w.len, NULL)) {
-				cur_score += rspamd_chartable_process_word_utf (task, &w, TRUE, NULL);
+				cur_score += rspamd_chartable_process_word_utf (task, &w,
+						TRUE, NULL, chartable_module_ctx);
 			}
 			else {
-				cur_score += rspamd_chartable_process_word_ascii (task, &w, TRUE);
+				cur_score += rspamd_chartable_process_word_ascii (task, &w,
+						TRUE, chartable_module_ctx);
 			}
 		}
 	}
