@@ -19,6 +19,7 @@ local rspamd_util = require "rspamd_util"
 local rspamd_regexp = require "rspamd_regexp"
 local tcp = require "rspamd_tcp"
 local upstream_list = require "rspamd_upstream_list"
+local lua_util = require "lua_util"
 local redis_params
 
 local N = "antivirus"
@@ -32,8 +33,13 @@ antivirus {
   clamav {
     # If set force this action if any virus is found (default unset: no action is forced)
     # action = "reject";
-    # if `true` only messages with non-image attachments will be checked (default true)
-    attachments_only = true;
+    # If set, then rejection message is set to this value (mention single quotes)
+    # message = '${SCANNER}: virus found: "${VIRUS}"';
+    # Scan mime_parts seperately - otherwise the complete mail will be transfered to AV Scanner
+    #scan_mime_parts = true;
+    # Scanning Text is suitable for some av scanner databases (e.g. Sanesecurity)
+    #scan_text_mime = false;
+    #scan_image_mime = false;
     # If `max_size` is set, messages > n bytes in size are not scanned
     max_size = 20000000;
     # symbol to add (add it to metric if you want non-zero weight)
@@ -63,14 +69,27 @@ antivirus {
   return
 end
 
+local default_message = '${SCANNER}: virus found: "${VIRUS}"'
+
 local function match_patterns(default_sym, found, patterns)
-  if not patterns then return default_sym end
-  for sym, pat in pairs(patterns) do
-    if pat:match(found) then
-      return sym
+  if type(patterns) ~= 'table' then return default_sym end
+  if not patterns[1] then
+    for sym, pat in pairs(patterns) do
+      if pat:match(found) then
+        return sym
+      end
     end
+    return default_sym
+  else
+    for _, p in ipairs(patterns) do
+      for sym, pat in pairs(p) do
+        if pat:match(found) then
+          return sym
+        end
+      end
+    end
+    return default_sym
   end
-  return default_sym
 end
 
 local function yield_result(task, rule, vname)
@@ -101,18 +120,24 @@ local function yield_result(task, rule, vname)
       vname = table.concat(vname, '; ')
     end
     task:set_pre_result(rule['action'],
-        string.format('%s: virus found: "%s"', rule['type'], vname))
+        lua_util.template(rule.message or 'Rejected', {
+          SCANNER = rule['type'],
+          VIRUS = vname,
+        }), N)
   end
 end
 
 local function clamav_config(opts)
   local clamav_conf = {
-    attachments_only = true,
+    scan_mime_parts = true;
+    scan_text_mime = false;
+    scan_image_mime = false;
     default_port = 3310,
     log_clean = false,
     timeout = 15.0,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
   }
 
   for k,v in pairs(opts) do
@@ -144,12 +169,15 @@ end
 
 local function fprot_config(opts)
   local fprot_conf = {
-    attachments_only = true,
+    scan_mime_parts = true;
+    scan_text_mime = false;
+    scan_image_mime = false;
     default_port = 10200,
     timeout = 15.0,
     log_clean = false,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
   }
 
   for k,v in pairs(opts) do
@@ -181,12 +209,17 @@ end
 
 local function sophos_config(opts)
   local sophos_conf = {
-    attachments_only = true,
+    scan_mime_parts = true;
+    scan_text_mime = false;
+    scan_image_mime = false;
     default_port = 4010,
     timeout = 15.0,
     log_clean = false,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
+    savdi_report_encrypted = false,
+    savdi_report_oversize = false,
   }
 
   for k,v in pairs(opts) do
@@ -218,13 +251,16 @@ end
 
 local function savapi_config(opts)
   local savapi_conf = {
-    attachments_only = true,
+    scan_mime_parts = true;
+    scan_text_mime = false;
+    scan_image_mime = false;
     default_port = 4444, -- note: You must set ListenAddress in savapi.conf
     product_id = 0,
     log_clean = false,
     timeout = 15.0,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
   }
 
   for k,v in pairs(opts) do
@@ -254,39 +290,33 @@ local function savapi_config(opts)
   return nil
 end
 
-local function message_not_too_large(task, rule)
+local function message_not_too_large(task, content, rule)
   local max_size = tonumber(rule['max_size'])
   if not max_size then return true end
-  if task:get_size() > max_size then return false end
+  if #content > max_size then
+    rspamd_logger.infox("skip %s AV check as it is too large: %s (%s is allowed)",
+      rule.type, #content, max_size)
+    return false
+  end
   return true
 end
 
-local function need_av_check(task, rule)
-  if rule['attachments_only'] then
-    for _,p in ipairs(task:get_parts()) do
-      if p:get_filename() and not p:is_image() then
-        return message_not_too_large(task, rule)
-      end
-    end
-
-    return false
-  else
-    return message_not_too_large(task, rule)
-  end
+local function need_av_check(task, content, rule)
+  return message_not_too_large(task, content, rule)
 end
 
-local function check_av_cache(task, rule, fn)
-  local key = task:get_digest()
+local function check_av_cache(task, digest, rule, fn)
+  local key = digest
 
   local function redis_av_cb(err, data)
     if data and type(data) == 'string' then
       -- Cached
       if data ~= 'OK' then
-        rspamd_logger.debugm(N, task, 'got cached result for %s: %s', key, data)
+        lua_util.debugm(N, task, 'got cached result for %s: %s', key, data)
         data = rspamd_str_split(data, '\x30')
         yield_result(task, rule, data)
       else
-        rspamd_logger.debugm(N, task, 'got cached result for %s: %s', key, data)
+        lua_util.debugm(N, task, 'got cached result for %s: %s', key, data)
       end
     else
       if err then
@@ -315,8 +345,8 @@ local function check_av_cache(task, rule, fn)
   return false
 end
 
-local function save_av_cache(task, rule, to_save)
-  local key = task:get_digest()
+local function save_av_cache(task, digest, rule, to_save)
+  local key = digest
 
   local function redis_set_cb(err)
     -- Do nothing
@@ -324,7 +354,7 @@ local function save_av_cache(task, rule, to_save)
       rspamd_logger.errx(task, 'failed to save virus cache for %s -> "%s": %s',
         to_save, key, err)
     else
-      rspamd_logger.debugm(N, task, 'saved cached result for %s: %s', key, to_save)
+      lua_util.debugm(N, task, 'saved cached result for %s: %s', key, to_save)
     end
   end
 
@@ -348,37 +378,45 @@ local function save_av_cache(task, rule, to_save)
   return false
 end
 
-local function fprot_check(task, rule)
+local function fprot_check(task, content, digest, rule)
   local function fprot_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
     local retransmits = rule.retransmits
     local scan_id = task:get_queue_id()
     if not scan_id then scan_id = task:get_uid() end
-    local header = string.format('SCAN STREAM %s SIZE %d\n', scan_id, task:get_size())
+    local header = string.format('SCAN STREAM %s SIZE %d\n', scan_id,
+        #content)
     local footer = '\n'
 
     local function fprot_callback(err, data)
       if err then
-        if err == 'IO timeout' then
-          if retransmits > 0 then
-            retransmits = retransmits - 1
-            tcp.request({
-              task = task,
-              host = addr:to_string(),
-              port = addr:get_port(),
-              timeout = rule['timeout'],
-              callback = fprot_callback,
-              data = { header, task:get_content(), footer },
-              stop_pattern = '\n'
-            })
-          else
-            rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
-            upstream:fail()
-          end
+        -- set current upstream to fail because an error occurred
+        upstream:fail()
+
+        -- retry with another upstream until retransmits exceeds
+        if retransmits > 0 then
+
+          retransmits = retransmits - 1
+
+          -- Select a different upstream!
+          upstream = rule.upstreams:get_upstream_round_robin()
+          addr = upstream:get_addr()
+
+          lua_util.debugm(N, task, '%s [%s]: retry IP: %s', rule['symbol'], rule['type'], addr)
+
+          tcp.request({
+            task = task,
+            host = addr:to_string(),
+            port = addr:get_port(),
+            timeout = rule['timeout'],
+            callback = fprot_callback,
+            data = { header, content, footer },
+            stop_pattern = '\n'
+          })
         else
-          rspamd_logger.errx(task, 'failed to scan: %s', err)
-          upstream:fail()
+          rspamd_logger.errx(task, '%s [%s]: failed to scan, maximum retransmits exceed', rule['symbol'], rule['type'])
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan and retransmits exceed')
         end
       else
         upstream:ok()
@@ -388,10 +426,12 @@ local function fprot_check(task, rule)
         if clean then
           cached = 'OK'
           if rule['log_clean'] then
-            rspamd_logger.infox(task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
+            rspamd_logger.infox(task, '%s [%s]: message or mime_part is clean', rule['symbol'], rule['type'])
           end
         else
-          local vname = string.match(data, '^1 <infected: (.+)>')
+          -- returncodes: 1: infected, 2: suspicious, 3: both, 4-255: some error occured
+          -- see http://www.f-prot.com/support/helpfiles/unix/appendix_c.html for more detail
+          local vname = string.match(data, '^[1-3] <[%w%s]-: (.-)>')
           if not vname then
             rspamd_logger.errx(task, 'Unhandled response: %s', data)
           else
@@ -400,7 +440,7 @@ local function fprot_check(task, rule)
           end
         end
         if cached then
-          save_av_cache(task, rule, cached)
+          save_av_cache(task, digest, rule, cached)
         end
       end
     end
@@ -411,13 +451,13 @@ local function fprot_check(task, rule)
       port = addr:get_port(),
       timeout = rule['timeout'],
       callback = fprot_callback,
-      data = { header, task:get_content(), footer },
+      data = { header, content, footer },
       stop_pattern = '\n'
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, fprot_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, fprot_check_uncached) then
       return
     else
       fprot_check_uncached()
@@ -425,45 +465,57 @@ local function fprot_check(task, rule)
   end
 end
 
-local function clamav_check(task, rule)
+local function clamav_check(task, content, digest, rule)
   local function clamav_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
     local retransmits = rule.retransmits
     local header = rspamd_util.pack("c9 c1 >I4", "zINSTREAM", "\0",
-      task:get_size())
+      #content)
     local footer = rspamd_util.pack(">I4", 0)
 
     local function clamav_callback(err, data)
       if err then
-        if err == 'IO timeout' then
-          if retransmits > 0 then
-            retransmits = retransmits - 1
-            tcp.request({
-              task = task,
-              host = addr:to_string(),
-              port = addr:get_port(),
-              timeout = rule['timeout'],
-              callback = clamav_callback,
-              data = { header, task:get_content(), footer },
-              stop_pattern = '\0'
-            })
-          else
-            rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
-            upstream:fail()
-          end
+
+        -- set current upstream to fail because an error occurred
+        upstream:fail()
+
+        -- retry with another upstream until retransmits exceeds
+        if retransmits > 0 then
+
+          retransmits = retransmits - 1
+
+          -- Select a different upstream!
+          upstream = rule.upstreams:get_upstream_round_robin()
+          addr = upstream:get_addr()
+
+          lua_util.debugm(N, task, '%s [%s]: retry IP: %s', rule['symbol'], rule['type'], addr)
+
+          tcp.request({
+            task = task,
+            host = addr:to_string(),
+            port = addr:get_port(),
+            timeout = rule['timeout'],
+            callback = clamav_callback,
+            data = { header, content, footer },
+            stop_pattern = '\0'
+          })
         else
-          rspamd_logger.errx(task, 'failed to scan: %s', err)
-          upstream:fail()
+          rspamd_logger.errx(task, '%s [%s]: failed to scan, maximum retransmits exceed', rule['symbol'], rule['type'])
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan and retransmits exceed')
         end
+
       else
         upstream:ok()
         data = tostring(data)
         local cached
+        lua_util.debugm(N, task, '%s [%s]: got reply: %s', rule['symbol'], rule['type'], data)
         if data == 'stream: OK' then
           cached = 'OK'
           if rule['log_clean'] then
-            rspamd_logger.infox(task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
+            rspamd_logger.infox(task, '%s [%s]: message or mime_part is clean', rule['symbol'], rule['type'])
+          else
+            lua_util.debugm(N, task, '%s [%s]: message or mime_part is clean', rule['symbol'], rule['type'])
           end
         else
           local vname = string.match(data, 'stream: (.+) FOUND')
@@ -472,10 +524,11 @@ local function clamav_check(task, rule)
             cached = vname
           else
             rspamd_logger.errx(task, 'unhandled response: %s', data)
+            task:insert_result(rule['symbol_fail'], 0.0, 'unhandled response')
           end
         end
         if cached then
-          save_av_cache(task, rule, cached)
+          save_av_cache(task, digest, rule, cached)
         end
       end
     end
@@ -486,13 +539,13 @@ local function clamav_check(task, rule)
       port = addr:get_port(),
       timeout = rule['timeout'],
       callback = clamav_callback,
-      data = { header, task:get_content(), footer },
+      data = { header, content, footer },
       stop_pattern = '\0'
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, clamav_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, clamav_check_uncached) then
       return
     else
       clamav_check_uncached()
@@ -500,54 +553,90 @@ local function clamav_check(task, rule)
   end
 end
 
-local function sophos_check(task, rule)
+local function sophos_check(task, content, digest, rule)
   local function sophos_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
     local retransmits = rule.retransmits
     local protocol = 'SSSP/1.0\n'
-    local streamsize = string.format('SCANDATA %d\n', task:get_size())
+    local streamsize = string.format('SCANDATA %d\n', #content)
     local bye = 'BYE\n'
 
     local function sophos_callback(err, data, conn)
+
       if err then
-        if err == 'IO timeout' then
+
+          -- set current upstream to fail because an error occurred
+          upstream:fail()
+
+          -- retry with another upstream until retransmits exceeds
           if retransmits > 0 then
+
             retransmits = retransmits - 1
+
+            -- Select a different upstream!
+            upstream = rule.upstreams:get_upstream_round_robin()
+            addr = upstream:get_addr()
+
+            lua_util.debugm(N, task, '%s [%s]: retry IP: %s', rule['symbol'], rule['type'], addr)
+
             tcp.request({
               task = task,
               host = addr:to_string(),
               port = addr:get_port(),
               timeout = rule['timeout'],
               callback = sophos_callback,
-              data = { protocol, streamsize, task:get_content(), bye }
+              data = { protocol, streamsize, content, bye }
             })
           else
-            rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
-            upstream:fail()
+            rspamd_logger.errx(task, '%s [%s]: failed to scan, maximum retransmits exceed', rule['symbol'], rule['type'])
+            task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan and retransmits exceed')
           end
-        else
-          rspamd_logger.errx(task, 'failed to scan: %s', err)
-          upstream:fail()
-        end
       else
         upstream:ok()
         data = tostring(data)
+        lua_util.debugm(N, task, '%s [%s]: got reply: %s', rule['symbol'], rule['type'], data)
         local vname = string.match(data, 'VIRUS (%S+) ')
         if vname then
           yield_result(task, rule, vname)
-          save_av_cache(task, rule, vname)
+          save_av_cache(task, digest, rule, vname)
         else
           if string.find(data, 'DONE OK') then
             if rule['log_clean'] then
-              rspamd_logger.infox(task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
+              rspamd_logger.infox(task, '%s [%s]: message or mime_part is clean', rule['symbol'], rule['type'])
+            else
+              lua_util.debugm(N, task, '%s [%s]: message or mime_part is clean', rule['symbol'], rule['type'])
             end
-            save_av_cache(task, rule, 'OK')
+            save_av_cache(task, digest, rule, 'OK')
+            -- not finished - continue
           elseif string.find(data, 'ACC') or string.find(data, 'OK SSSP') then
             conn:add_read(sophos_callback)
+            -- set pseudo virus if configured, else do nothing since it's no fatal
+          elseif string.find(data, 'FAIL 0212') then
+            rspamd_logger.infox(task, 'Message is ENCRYPTED (0212 SOPHOS_SAVI_ERROR_FILE_ENCRYPTED): %s', data)
+            if rule['savdi_report_encrypted'] then
+              yield_result(task, rule, "SAVDI_FILE_ENCRYPTED")
+              save_av_cache(task, digest, rule, "SAVDI_FILE_ENCRYPTED")
+            end
+            -- set pseudo virus if configured, else set fail since part was not scanned
+          elseif string.find(data, 'REJ 4') then
+            if rule['savdi_report_oversize'] then
+              rspamd_logger.infox(task, 'SAVDI: Message is OVERSIZED (SSSP reject code 4): %s', data)
+              yield_result(task, rule, "SAVDI_FILE_OVERSIZED")
+              save_av_cache(task, digest, rule, "SAVDI_FILE_OVERSIZED")
+            else
+              rspamd_logger.errx(task, 'SAVDI: Message is OVERSIZED (SSSP reject code 4): %s', data)
+              task:insert_result(rule['symbol_fail'], 0.0, 'Message is OVERSIZED (SSSP reject code 4):' .. data)
+            end
+            -- excplicitly set REJ1 message when SAVDIreports a protocol error
+          elseif string.find(data, 'REJ 1') then
+            rspamd_logger.errx(task, 'SAVDI (Protocol error (REJ 1)): %s', data)
+            task:insert_result(rule['symbol_fail'], 0.0, 'SAVDI (Protocol error (REJ 1)):' .. data)
           else
             rspamd_logger.errx(task, 'unhandled response: %s', data)
+            task:insert_result(rule['symbol_fail'], 0.0, 'unhandled response')
           end
+
         end
       end
     end
@@ -558,12 +647,12 @@ local function sophos_check(task, rule)
       port = addr:get_port(),
       timeout = rule['timeout'],
       callback = sophos_callback,
-      data = { protocol, streamsize, task:get_content(), bye }
+      data = { protocol, streamsize, content, bye }
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, sophos_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, sophos_check_uncached) then
       return
     else
       sophos_check_uncached()
@@ -571,7 +660,7 @@ local function sophos_check(task, rule)
   end
 end
 
-local function savapi_check(task, rule)
+local function savapi_check(task, content, digest, rule)
   local function savapi_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
@@ -588,7 +677,7 @@ local function savapi_check(task, rule)
       for virus,_ in pairs(vnames) do
         table.insert(vnames_reordered, virus)
       end
-      rspamd_logger.debugm(N, task, "%s: number of virus names found %s", rule['type'], #vnames_reordered)
+      lua_util.debugm(N, task, "%s: number of virus names found %s", rule['type'], #vnames_reordered)
       if #vnames_reordered > 0 then
         local vname = {}
         for _,virus in ipairs(vnames_reordered) do
@@ -596,7 +685,7 @@ local function savapi_check(task, rule)
         end
 
         yield_result(task, rule, vname)
-        save_av_cache(task, rule, vname)
+        save_av_cache(task, digest, rule, vname)
       end
       if conn then
         conn:close()
@@ -605,14 +694,14 @@ local function savapi_check(task, rule)
 
     local function savapi_scan2_cb(err, data, conn)
       local result = tostring(data)
-      rspamd_logger.debugm(N, task, "%s: got reply: %s", rule['type'], result)
+      lua_util.debugm(N, task, "%s: got reply: %s", rule['type'], result)
 
       -- Terminal response - clean
       if string.find(result, '200') or string.find(result, '210') then
         if rule['log_clean'] then
-          rspamd_logger.infox(task, '%s: message is clean', rule['type'])
+          rspamd_logger.infox(task, '%s: message or mime_part is clean', rule['type'])
         end
-        save_av_cache(task, rule, 'OK')
+        save_av_cache(task, digest, rule, 'OK')
         conn:add_write(savapi_fin_cb, 'QUIT\n')
 
       -- Terminal response - infected
@@ -645,7 +734,7 @@ local function savapi_check(task, rule)
     local function savapi_greet2_cb(err, data, conn)
       local result = tostring(data)
       if string.find(result, '100 PRODUCT') then
-        rspamd_logger.debugm(N, task, "%s: scanning file: %s", rule['type'], message_file)
+        lua_util.debugm(N, task, "%s: scanning file: %s", rule['type'], message_file)
         conn:add_write(savapi_scan1_cb, {string.format('SCAN %s\n', message_file)})
       else
         rspamd_logger.errx(task, '%s: invalid product id %s', rule['type'], rule['product_id'])
@@ -659,24 +748,32 @@ local function savapi_check(task, rule)
 
     local function savapi_callback_init(err, data, conn)
       if err then
-        if err == 'IO timeout' then
-          if retransmits > 0 then
-            retransmits = retransmits - 1
-            tcp.request({
-              task = task,
-              host = addr:to_string(),
-              port = addr:get_port(),
-              timeout = rule['timeout'],
-              callback = savapi_callback_init,
-              stop_pattern = {'\n'},
-            })
-          else
-            rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
-            upstream:fail()
-          end
+
+        -- set current upstream to fail because an error occurred
+        upstream:fail()
+
+        -- retry with another upstream until retransmits exceeds
+        if retransmits > 0 then
+
+          retransmits = retransmits - 1
+
+          -- Select a different upstream!
+          upstream = rule.upstreams:get_upstream_round_robin()
+          addr = upstream:get_addr()
+
+          lua_util.debugm(N, task, '%s [%s]: retry IP: %s', rule['symbol'], rule['type'], addr)
+
+          tcp.request({
+            task = task,
+            host = addr:to_string(),
+            port = addr:get_port(),
+            timeout = rule['timeout'],
+            callback = savapi_callback_init,
+            stop_pattern = {'\n'},
+          })
         else
-          rspamd_logger.errx(task, 'failed to scan: %s', err)
-          upstream:fail()
+          rspamd_logger.errx(task, '%s [%s]: failed to scan, maximum retransmits exceed', rule['symbol'], rule['type'])
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan and retransmits exceed')
         end
       else
         upstream:ok()
@@ -699,8 +796,8 @@ local function savapi_check(task, rule)
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, savapi_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, savapi_check_uncached) then
       return
     else
       savapi_check_uncached()
@@ -729,18 +826,34 @@ local av_types = {
 
 local function add_antivirus_rule(sym, opts)
   if not opts['type'] then
+    rspamd_logger.errx(rspamd_config, 'unknown type for AV rule %s', sym)
     return nil
   end
 
   if not opts['symbol'] then opts['symbol'] = sym end
   local cfg = av_types[opts['type']]
 
+  if not opts['symbol_fail'] then
+    opts['symbol_fail'] = string.upper(opts['type']) .. '_FAIL'
+  end
+
+  -- WORKAROUND for deprecated attachments_only
+  if opts['attachments_only'] ~= nil then
+    opts['scan_mime_parts'] = opts['attachments_only']
+    rspamd_logger.warnx(rspamd_config, '%s [%s]: Using attachments_only is deprecated. '..
+     'Please use scan_mime_parts = %s instead', opts['symbol'], opts['type'], opts['attachments_only'])
+  end
+  -- WORKAROUND for deprecated attachments_only
+
   if not cfg then
     rspamd_logger.errx(rspamd_config, 'unknown antivirus type: %s',
       opts['type'])
   end
 
   local rule = cfg.configure(opts)
+  rule.type = opts.type
+  rule.symbol_fail = opts.symbol_fail
+
 
   if not rule then
     rspamd_logger.errx(rspamd_config, 'cannot configure %s for %s',
@@ -748,10 +861,24 @@ local function add_antivirus_rule(sym, opts)
     return nil
   end
 
-  if opts['patterns'] then
+  if type(opts['patterns']) == 'table' then
     rule['patterns'] = {}
-    for k, v in pairs(opts['patterns']) do
-      rule['patterns'][k] = rspamd_regexp.create_cached(v)
+    if opts['patterns'][1] then
+      for i, p in ipairs(opts['patterns']) do
+        if type(p) == 'table' then
+          local new_set = {}
+          for k, v in pairs(p) do
+            new_set[k] = rspamd_regexp.create_cached(v)
+          end
+          rule['patterns'][i] = new_set
+        else
+          rule['patterns'][i] = {}
+        end
+      end
+    else
+      for k, v in pairs(opts['patterns']) do
+        rule['patterns'][k] = rspamd_regexp.create_cached(v)
+      end
     end
   end
 
@@ -760,7 +887,27 @@ local function add_antivirus_rule(sym, opts)
   end
 
   return function(task)
-    return cfg.check(task, rule)
+    if rule.scan_mime_parts then
+      local parts = task:get_parts() or {}
+
+      for _,p in ipairs(parts) do
+        if (
+            (p:is_image() and rule.scan_image_mime)
+            or (p:is_text() and rule.scan_text_mime)
+            or (p:is_multipart() and rule.scan_text_mime)
+            or (not p:is_image() and not p:is_text() and not p:is_multipart())
+            ) then
+
+          local content = p:get_content()
+
+          if content and #content > 0 then
+            cfg.check(task, content, p:get_digest(), rule)
+          end
+        end
+      end
+    else
+      cfg.check(task, task:get_content(), task:get_digest(), rule)
+    end
   end
 end
 
@@ -768,6 +915,7 @@ end
 local opts = rspamd_config:get_all_opt('antivirus')
 if opts and type(opts) == 'table' then
   redis_params = rspamd_parse_redis_server('antivirus')
+  local has_valid = false
   for k, m in pairs(opts) do
     if type(m) == 'table' and m['type'] and m['servers'] then
       local cb = add_antivirus_rule(k, m)
@@ -778,14 +926,44 @@ if opts and type(opts) == 'table' then
           type = 'normal',
           name = m['symbol'],
           callback = cb,
+          score = 0.0,
+          group = 'antivirus'
         })
-        if m['patterns'] then
-          for sym in pairs(m['patterns']) do
-            rspamd_config:register_symbol({
-              type = 'virtual',
-              name = sym,
-              parent = id
-            })
+        rspamd_config:register_symbol({
+          type = 'virtual',
+          name = m['symbol_fail'],
+          parent = id,
+          score = 0.0,
+          group = 'antivirus'
+        })
+        has_valid = true
+        if type(m['patterns']) == 'table' then
+          if m['patterns'][1] then
+            for _, p in ipairs(m['patterns']) do
+              if type(p) == 'table' then
+                for sym in pairs(p) do
+                  rspamd_logger.debugm(N, rspamd_config, 'registering: %1', {
+                    type = 'virtual',
+                    name = sym,
+                    parent = m['symbol'],
+                    parent_id = id,
+                  })
+                  rspamd_config:register_symbol({
+                    type = 'virtual',
+                    name = sym,
+                    parent = id
+                  })
+                end
+              end
+            end
+          else
+            for sym in pairs(m['patterns']) do
+              rspamd_config:register_symbol({
+                type = 'virtual',
+                name = sym,
+                parent = id
+              })
+            end
           end
         end
         if m['score'] then
@@ -802,10 +980,14 @@ if opts and type(opts) == 'table' then
             name = m['symbol'],
             score = m['score'],
             description = description,
-            group = group
+            group = group or 'antivirus'
           })
         end
       end
     end
   end
+
+  if not has_valid then
+    lua_util.disable_module(N, 'config')
+  end
 end
