@@ -344,7 +344,7 @@ local full_extensions_map = {
   {"group", "text/x-ms-group"},
   {"gsm", "audio/x-gsm"},
   {"gtar", "application/x-gtar"},
-  {"gz", "application/x-gzip"},
+  {"gz", {"application/gzip", "application/x-gzip"}},
   {"h", "text/plain"},
   {"hdf", "application/x-hdf"},
   {"hdml", "text/x-hdml"},
@@ -376,7 +376,7 @@ local full_extensions_map = {
   {"hxx", "text/plain"},
   {"i", "text/plain"},
   {"ico", "image/x-icon"},
-  {"ics", "application/octet-stream"},
+  {"ics", {"text/calendar", "application/octet-stream"}},
   {"idl", "text/plain"},
   {"ief", "image/ief"},
   {"iii", "application/x-iphone"},
@@ -773,7 +773,7 @@ local full_extensions_map = {
   {"xls", {"application/vnd.ms-excel", "application/vnd.ms-office", "application/x-excel", "application/octet-stream"}},
   {"xlsb", "application/vnd.ms-excel.sheet.binary.macroEnabled.12"},
   {"xlsm", "application/vnd.ms-excel.sheet.macroEnabled.12"},
-  {"xlsx", {"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", "application/octet-stream"}},
+  {"xlsx", {"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", "application/vnd.ms-excel.12", "application/octet-stream"}},
   {"xlt", "application/vnd.ms-excel"},
   {"xltm", "application/vnd.ms-excel.template.macroEnabled.12"},
   {"xltx", "application/vnd.openxmlformats-officedocument.spreadsheetml.template"},
@@ -798,6 +798,7 @@ local full_extensions_map = {
   {"xwd", "image/x-xwindowdump"},
   {"z", "application/x-compress"},
   {"zip", {"application/zip", "application/x-zip-compressed", "application/octet-stream"}},
+  {"zlib", "application/zlib"},
 }
 
 local function check_mime_type(task)
@@ -865,6 +866,13 @@ local function check_mime_type(task)
         if ext2 then
           check_extension(settings['bad_extensions'][ext],
             settings['bad_extensions'][ext2])
+          -- Check for archive cloaking like .zip.gz
+          if settings['archive_extensions'][ext2]
+            -- Exclude multipart archive extensions, e.g. .zip.001
+            and not string.match(ext, '^%d+$')
+          then
+            task:insert_result(settings['symbol_archive_in_archive'], 1.0, string.format(".%s.%s", ext2, ext))
+          end
         else
           check_extension(settings['bad_extensions'][ext], nil)
         end
@@ -1010,49 +1018,58 @@ if opts then
     local id = rspamd_config:register_symbol({
       name = 'MIME_TYPES_CALLBACK',
       callback = check_mime_type,
-      type = 'callback,nostat'
+      type = 'callback,nostat',
+      group = 'mime_types',
     })
 
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_unknown'],
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_bad'],
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_good'],
       flags = 'nice',
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_attachment'],
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_encrypted_archive'],
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_archive_in_archive'],
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_double_extension'],
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
     rspamd_config:register_symbol({
       type = 'virtual',
       name = settings['symbol_bad_extension'],
-      parent = id
+      parent = id,
+      group = 'mime_types',
     })
   else
     lua_util.disable_module(N, "config")
