@@ -19,6 +19,12 @@ if confighelp then
   return
 end
 
+local hash = require 'rspamd_cryptobox_hash'
+local rspamd_logger = require 'rspamd_logger'
+local rspamd_util = require 'rspamd_util'
+local fun = require 'fun'
+local lua_util = require 'lua_util'
+
 -- This plugin implements various types of RBL checks
 -- Documentation can be found here:
 -- https://rspamd.com/doc/modules/rbl.html
@@ -29,10 +35,6 @@ local N = 'rbl'
 local rbls = {}
 local local_exclusions = nil
 
-local hash = require 'rspamd_cryptobox_hash'
-local rspamd_logger = require 'rspamd_logger'
-local rspamd_util = require 'rspamd_util'
-local fun = require 'fun'
 local default_monitored = '1.0.0.127'
 
 local symbols = {
@@ -86,6 +88,73 @@ local function ip_to_rbl(ip, rbl)
   return table.concat(ip:inversed_str_octets(), '.') .. '.' .. rbl
 end
 
+local function gen_check_rcvd_conditions(rbl, received_total)
+  local min_pos = tonumber(rbl['received_min_pos'])
+  local max_pos = tonumber(rbl['received_max_pos'])
+  local match_flags = rbl['received_flags']
+  local nmatch_flags = rbl['received_nflags']
+  local function basic_received_check(rh)
+    if not (rh['real_ip'] and rh['real_ip']:is_valid()) then return false end
+    if ((rh['real_ip']:get_version() == 6 and rbl['ipv6']) or
+      (rh['real_ip']:get_version() == 4 and rbl['ipv4'])) and
+      ((rbl['exclude_private_ips'] and not rh['real_ip']:is_local()) or
+      not rbl['exclude_private_ips']) and ((rbl['exclude_local_ips'] and
+      not is_excluded_ip(rh['real_ip'])) or not rbl['exclude_local_ips']) then
+        return true
+    else
+      return false
+    end
+  end
+  if not (max_pos or min_pos or match_flags or nmatch_flags) then
+    return basic_received_check
+  end
+  return function(rh, pos)
+    if not basic_received_check() then return false end
+    local got_flags = rh['flags'] or E
+    if min_pos then
+      if min_pos < 0 then
+        if min_pos == -1 then
+          if (pos ~= received_total) then
+            return false
+          end
+        else
+          if pos <= (received_total - (min_pos*-1)) then
+            return false
+          end
+        end
+      elseif pos < min_pos then
+        return false
+      end
+    end
+    if max_pos then
+      if max_pos < -1 then
+        if (received_total - (max_pos*-1)) >= pos then
+          return false
+        end
+      elseif max_pos > 0 then
+        if pos > max_pos then
+          return false
+        end
+      end
+    end
+    if match_flags then
+      for _, flag in ipairs(match_flags) do
+        if not got_flags[flag] then
+          return false
+        end
+      end
+    end
+    if nmatch_flags then
+      for _, flag in ipairs(nmatch_flags) do
+        if got_flags[flag] then
+          return false
+        end
+      end
+    end
+    return true
+  end
+end
+
 local function rbl_cb (task)
   local function gen_rbl_callback(rule)
     return function (_, to_resolve, results, err)
@@ -93,10 +162,10 @@ local function rbl_cb (task)
         rspamd_logger.errx(task, 'error looking up %s: %s', to_resolve, err)
       end
       if not results then
-        rspamd_logger.debugm(N, task, 'DNS RESPONSE: label=%1 results=%2 error=%3 rbl=%4', to_resolve, false, err, rule['rbls'][1]['symbol'])
+        lua_util.debugm(N, task, 'DNS RESPONSE: label=%1 results=%2 error=%3 rbl=%4', to_resolve, false, err, rule['rbls'][1]['symbol'])
         return
       else
-        rspamd_logger.debugm(N, task, 'DNS RESPONSE: label=%1 results=%2 error=%3 rbl=%4', to_resolve, true, err, rule['rbls'][1]['symbol'])
+        lua_util.debugm(N, task, 'DNS RESPONSE: label=%1 results=%2 error=%3 rbl=%4', to_resolve, true, err, rule['rbls'][1]['symbol'])
       end
 
       for _,rbl in ipairs(rule.rbls) do
@@ -107,7 +176,7 @@ local function rbl_cb (task)
         for _,result in pairs(results) do
           local ipstr = result:to_string()
           local foundrc
-          rspamd_logger.debugm(N, task, '%s DNS result %s', to_resolve, ipstr)
+          lua_util.debugm(N, task, '%s DNS result %s', to_resolve, ipstr)
           for s,i in pairs(rbl['returncodes']) do
             if type(i) == 'string' then
               if string.find(ipstr, '^' .. i .. '$') then
@@ -135,15 +204,13 @@ local function rbl_cb (task)
           end
         end
       end
-
-      task:inc_dns_req()
     end
   end
 
   local params = {} -- indexed by rbl name
 
   local function gen_rbl_rule(to_resolve, rbl)
-    rspamd_logger.debugm(N, task, 'DNS REQUEST: label=%1 rbl=%2', to_resolve, rbl['symbol'])
+    lua_util.debugm(N, task, 'DNS REQUEST: label=%1 rbl=%2', to_resolve, rbl['symbol'])
     if not params[to_resolve] then
       local nrule = {
         to_resolve = to_resolve,
@@ -159,7 +226,10 @@ local function rbl_cb (task)
     return params[to_resolve]
   end
 
-  local havegot = {}
+  local havegot = {
+    emails = {},
+    received = {}
+  }
   local notgot = {}
 
   local alive_rbls = fun.filter(function(_, rbl)
@@ -242,10 +312,11 @@ local function rbl_cb (task)
       if notgot['emails'] then
         return false
       end
-      if not havegot['emails'] then
+      if #havegot['emails'] == 0 then
         havegot['emails'] = task:get_emails()
         if havegot['emails'] == nil then
           notgot['emails'] = true
+          havegot['emails'] = {}
           return false
         end
       end
@@ -264,10 +335,11 @@ local function rbl_cb (task)
       if notgot['received'] then
         return false
       end
-      if not havegot['received'] then
+      if #havegot['received'] == 0 then
         havegot['received'] = task:get_received_headers()
         if next(havegot['received']) == nil then
           notgot['received'] = true
+          havegot['received'] = {}
           return false
         end
       end
@@ -374,21 +446,21 @@ local function rbl_cb (task)
     return false
   end, enabled_rbls))
 
+  havegot['received'] = fun.filter(function(h)
+    return not h['flags']['artificial']
+  end, havegot['received']):totable()
+
+  local received_total = #havegot['received']
   -- Received lists
   fun.each(function(_, rbl)
-    for _,rh in ipairs(havegot['received']) do
-      if rh['real_ip'] and rh['real_ip']:is_valid() then
-        if ((rh['real_ip']:get_version() == 6 and rbl['ipv6']) or
-          (rh['real_ip']:get_version() == 4 and rbl['ipv4'])) and
-          ((rbl['exclude_private_ips'] and not rh['real_ip']:is_local()) or
-          not rbl['exclude_private_ips']) and ((rbl['exclude_local_ips'] and
-          not is_excluded_ip(rh['real_ip'])) or not rbl['exclude_local_ips']) then
-          -- Disable forced for received resolving, as we have no control on
-          -- those headers count
-          local to_resolve = ip_to_rbl(rh['real_ip'], rbl['rbl'])
-          local rule = gen_rbl_rule(to_resolve, rbl)
-          rule.forced = false
-        end
+    local check_conditions = gen_check_rcvd_conditions(rbl, received_total)
+    for pos,rh in ipairs(havegot['received']) do
+      if check_conditions(rh, pos) then
+        local to_resolve = ip_to_rbl(rh['real_ip'], rbl['rbl'])
+        local rule = gen_rbl_rule(to_resolve, rbl)
+        -- Disable forced for received resolving, as we have no control on
+        -- those headers count
+        rule.forced = false
       end
     end
   end,
@@ -412,6 +484,7 @@ end
 local opts = rspamd_config:get_all_opt(N)
 if not (opts and type(opts) == 'table') then
   rspamd_logger.infox(rspamd_config, 'Module is unconfigured')
+  lua_util.disable_module(N, "config")
   return
 end
 
@@ -453,13 +526,19 @@ local need_dkim = false
 local id = rspamd_config:register_symbol({
   type = 'callback',
   callback = rbl_cb,
+  name = 'RBL_CALLBACK',
   flags = 'empty,nice'
 })
 
 local is_monitored = {}
+local rbls_count = 0
 for key,rbl in pairs(opts['rbls']) do
   (function()
-    if rbl['disabled'] then return end
+    if type(rbl) ~= 'table' or rbl['disabled'] then
+      rspamd_logger.infox(rspamd_config, 'disable rbl "%s"', key)
+      return
+    end
+
     for default, default_v in pairs(default_defaults) do
       if(rbl[default_v[2]] == nil) then
         rbl[default_v[2]] = opts[default]
@@ -510,12 +589,13 @@ for key,rbl in pairs(opts['rbls']) do
       (not rbl['returncodes'])) then
         rbl['symbol'] = key
     end
-    if type(rspamd_config.get_api_version) ~= 'nil' and rbl['symbol'] then
+    if rbl['symbol'] then
       rspamd_config:register_symbol({
         name = rbl['symbol'],
         parent = id,
         type = 'virtual'
       })
+      rbls_count = rbls_count + 1
 
       if rbl['dkim'] then
         need_dkim = true
@@ -559,6 +639,11 @@ for key,rbl in pairs(opts['rbls']) do
     end
   end)()
 end
+
+if rbls_count == 0 then
+  lua_util.disable_module(N, "config")
+end
+
 for _, w in pairs(white_symbols) do
   for _, b in pairs(black_symbols) do
     local csymbol = 'RBL_COMPOSITE_' .. w .. '_' .. b
@@ -567,5 +652,5 @@ for _, w in pairs(white_symbols) do
   end
 end
 if need_dkim then
-  rspamd_config:register_dependency(id, symbols['dkim_allow_symbol'])
+  rspamd_config:register_dependency('RBL_CALLBACK', symbols['dkim_allow_symbol'])
 end
