@@ -31,10 +31,11 @@ local settings = {
   fuzzy_flag = 1,
   fuzzy_weight = 10.0,
   key_prefix = 'sptr_',
-  check_authed = true,
-  check_local = true,
 }
 
+local check_authed = true
+local check_local = true
+
 local function spamtrap_cb(task)
   local rcpts = task:get_recipients('smtp')
   local authed_user = task:get_user()
@@ -42,8 +43,8 @@ local function spamtrap_cb(task)
   local called_for_domain = false
   local target
 
-  if ((not settings['check_authed'] and authed_user) or
-      (not settings['check_local'] and ip_addr and ip_addr:is_local())) then
+  if ((not check_authed and authed_user) or
+      (not check_local and ip_addr and ip_addr:is_local())) then
     rspamd_logger.infox(task, "skip spamtrap checks for local networks or authenticated user");
     return
   end
@@ -65,7 +66,7 @@ local function spamtrap_cb(task)
       rspamd_logger.infox(task, 'spamtrap found: <%s>', rcpt)
       if settings.smtp_message then
         task:set_pre_result(settings['action'],
-          lua_util.template(settings.smtp_message, { rcpt = rcpt}))
+          lua_util.template(settings.smtp_message, { rcpt = rcpt}), 'spamtrap')
       else
         local smtp_message = 'unknown error'
         if settings.action == 'no action' then
@@ -73,7 +74,7 @@ local function spamtrap_cb(task)
         elseif settings.action == 'reject' then
           smtp_message = 'message rejected'
         end
-        task:set_pre_result(settings['action'], smtp_message)
+        task:set_pre_result(settings['action'], smtp_message, 'spamtrap')
       end
     end
   end
@@ -136,11 +137,31 @@ local function spamtrap_cb(task)
 end
 
 -- Module setup
+local function try_opts(where)
+  local ret = false
+  local opts = rspamd_config:get_all_opt(where)
+  if type(opts) == 'table' then
+    if type(opts['check_local']) == 'boolean' then
+      check_local = opts['check_local']
+      ret = true
+    end
+    if type(opts['check_authed']) == 'boolean' then
+      check_authed = opts['check_authed']
+      ret = true
+    end
+  end
+
+  return ret
+end
+
 local opts = rspamd_config:get_all_opt('spamtrap')
 if not (opts and type(opts) == 'table') then
   rspamd_logger.infox(rspamd_config, 'module is unconfigured')
   return
 end
+
+if not try_opts(M) then try_opts('options') end
+
 if opts then
   for k,v in pairs(opts) do
     settings[k] = v
