@@ -206,15 +206,26 @@ spf_module_config (struct rspamd_config *cfg)
 
 	spf_module_ctx->whitelist_ip = NULL;
 
-	if ((value =
-		rspamd_config_get_module_opt (cfg, "options", "check_local")) != NULL) {
+	value = rspamd_config_get_module_opt (cfg, "spf", "check_local");
+
+	if (value == NULL) {
+		rspamd_config_get_module_opt (cfg, "options", "check_local");
+	}
+
+	if (value != NULL) {
 		spf_module_ctx->check_local = ucl_obj_toboolean (value);
 	}
 	else {
 		spf_module_ctx->check_local = FALSE;
 	}
-	if ((value =
-		rspamd_config_get_module_opt (cfg, "options", "check_authed")) != NULL) {
+
+	value = rspamd_config_get_module_opt (cfg, "spf", "check_authed");
+
+	if (value == NULL) {
+		rspamd_config_get_module_opt (cfg, "options", "check_authed");
+	}
+
+	if (value != NULL) {
 		spf_module_ctx->check_authed = ucl_obj_toboolean (value);
 	}
 	else {
