@@ -30,9 +30,11 @@ extern struct rspamd_main *rspamd_main;
 extern module_t *modules[];
 extern worker_t *workers[];
 
-static void rspamadm_confighelp (gint argc, gchar **argv);
+static void rspamadm_confighelp (gint argc, gchar **argv,
+								 const struct rspamadm_command *cmd);
 
-static const char *rspamadm_confighelp_help (gboolean full_help);
+static const char *rspamadm_confighelp_help (gboolean full_help,
+											 const struct rspamadm_command *cmd);
 
 struct rspamadm_command confighelp_command = {
 		.name = "confighelp",
@@ -55,7 +57,7 @@ static GOptionEntry entries[] = {
 };
 
 static const char *
-rspamadm_confighelp_help (gboolean full_help)
+rspamadm_confighelp_help (gboolean full_help, const struct rspamadm_command *cmd)
 {
 	const char *help_str;
 
@@ -67,9 +69,9 @@ rspamadm_confighelp_help (gboolean full_help)
 				"-j: output pretty formatted JSON\n"
 				"-k: search by keyword in doc string\n"
 				"-P: use specific Lua plugins path\n"
-				"--no-color: show colored output\n"
+				"--no-color: disable coloured output\n"
 				"--short: show only option names\n"
-				"--no-examples: do not show examples (impied by --short)\n"
+				"--no-examples: do not show examples (implied by --short)\n"
 				"--help: shows available options and commands";
 	}
 	else {
@@ -104,11 +106,11 @@ rspamadm_confighelp_show (struct rspamd_config *cfg, gint argc, gchar **argv,
 			rspamd_fprintf (stdout, "Showing help for all options:\n");
 		}
 
-		rspamadm_execute_lua_ucl_subr (cfg->lua_state,
-				argc,
+		rspamadm_execute_lua_ucl_subr (argc,
 				argv,
 				obj,
-				"confighelp");
+				"confighelp",
+				TRUE);
 
 		rspamd_fstring_free (out);
 		return;
@@ -189,7 +191,7 @@ rspamadm_confighelp_search_word (const ucl_object_t *obj, const gchar *str)
 }
 
 static void
-rspamadm_confighelp (gint argc, gchar **argv)
+rspamadm_confighelp (gint argc, gchar **argv, const struct rspamadm_command *cmd)
 {
 	struct rspamd_config *cfg;
 	ucl_object_t *doc_obj;
@@ -199,7 +201,7 @@ rspamadm_confighelp (gint argc, gchar **argv)
 	module_t *mod, **pmod;
 	worker_t **pworker;
 	struct module_ctx *mod_ctx;
-	gint i = 1, ret = 0, processed_args = 0;
+	gint i, ret = 0, processed_args = 0;
 
 	context = g_option_context_new (
 			"confighelp - displays help for the configuration options");
@@ -224,26 +226,30 @@ rspamadm_confighelp (gint argc, gchar **argv)
 		pworker++;
 	}
 
-	cfg = rspamd_config_new ();
+	cfg = rspamd_config_new (RSPAMD_CONFIG_INIT_SKIP_LUA);
+	cfg->lua_state = rspamd_main->cfg->lua_state;
 	cfg->compiled_modules = modules;
 	cfg->compiled_workers = workers;
 
-	rspamd_rcl_config_init (cfg);
+	rspamd_rcl_config_init (cfg, NULL);
 	lua_pushboolean (cfg->lua_state, true);
 	lua_setglobal (cfg->lua_state, "confighelp");
 	rspamd_rcl_add_lua_plugins_path (cfg, plugins_path, NULL);
 
 	/* Init modules to get documentation strings */
+	i = 0;
 	for (pmod = cfg->compiled_modules; pmod != NULL && *pmod != NULL; pmod++) {
 		mod = *pmod;
 		mod_ctx = g_malloc0 (sizeof (struct module_ctx));
 
 		if (mod->module_init_func (cfg, &mod_ctx) == 0) {
-			g_hash_table_insert (cfg->c_modules,
-					(gpointer) mod->name,
-					mod_ctx);
+			g_ptr_array_add (cfg->c_modules, mod_ctx);
+			mod_ctx->mod = mod;
+			mod->ctx_offset = i++;
 			mod_ctx->mod = mod;
 		}
+
+
 	}
 	/* Also init all workers */
 	for (pworker = cfg->compiled_workers; *pworker != NULL; pworker ++) {
