@@ -17,6 +17,8 @@ PORT_FPROT = 56797
 PORT_FPROT_DUPLICATE = 56798
 REDIS_ADDR = u'127.0.0.1'
 REDIS_PORT = 56379
+NGINX_ADDR = u'127.0.0.1'
+NGINX_PORT = 56380
 RSPAMD_GROUP = 'nogroup'
 RSPAMD_USER = 'nobody'
 SOCK_DGRAM = socket.SOCK_DGRAM
