@@ -460,7 +460,6 @@ sub ProcessRelated {
 
 sub ProcessLog {
   my ( $ts_format, @line ) = &log_time_format($rspamd_log);
-  my $is_syslog = defined $ts_format && $ts_format eq 'syslog';
 
   while() {
     last if eof $rspamd_log;
@@ -474,10 +473,15 @@ sub ProcessLog {
 
     if (/^.*rspamd_task_write_log.*$/) {
       &spinner;
-      my $ts =
-        ($is_syslog)
-        ? syslog2iso( join ' ', ( split /\s+/ )[ 0 .. 2 ] )
-        : join ' ', ( split /\s+/ )[ 0 .. 1 ];
+      my $ts;
+      if ( $ts_format eq 'syslog' ) {
+        $ts = syslog2iso( join ' ', ( split /\s+/ )[ 0 .. 2 ] );
+      } elsif ( $ts_format eq 'syslog5424' ) {
+        /^([0-9-]+)T([0-9:]+)/;
+        $ts = "$1 $2";
+      } else {
+        $ts = join ' ', ( split /\s+/ )[ 0 .. 1 ];
+      }
 
       next if ( $ts lt $startTime );
       next if ( defined $endTime && $ts gt $endTime );
@@ -504,12 +508,22 @@ sub ProcessLog {
 
       next if ( $skip != 0 );
 
-      $timeStamp{'end'} = $ts;
-      $timeStamp{'start'} //= $timeStamp{'end'};
-      $scanTime{'min'} = $scan_time
-        if ( !exists $scanTime{'min'} || $scanTime{'min'} > $scan_time );
-      $scanTime{'max'} = $scan_time
-        if ( $scanTime{'max'} < $scan_time );
+      if (defined($timeStamp{'end'})) {
+        $timeStamp{'end'} = $ts if ( $ts gt $timeStamp{'end'} );
+      }
+      else {
+        $timeStamp{'end'} = $ts;
+      }
+
+      if (defined($timeStamp{'start'})) {
+        $timeStamp{'start'} = $ts if ( $ts lt $timeStamp{'start'} );
+      }
+      else {
+        $timeStamp{'start'} = $ts;
+      }
+
+      $scanTime{'min'} = $scan_time if ( !exists $scanTime{'min'} || $scanTime{'min'} > $scan_time );
+      $scanTime{'max'} = $scan_time if ( $scanTime{'max'} < $scan_time );
       $scanTime{'total'} += $scan_time;
 
       $action{$act}++;
@@ -718,6 +732,12 @@ sub log_time_format {
       last;
     }
 
+    # 2018-04-16T06:25:46.012590+02:00 rspamd rspamd[12968]
+    elsif(/\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(\.\d{1,6})?(Z|[-+]\d{2}:\d{2}) \S+ rspamd\[\d+\]/) {
+      $format = 'syslog5424';
+      last;
+    }
+
     # Skip newsyslog messages
     # Aug  8 00:00:00 hostname newsyslog[63284]: logfile turned over
     elsif ( /^\w{3} (?:\s?\d|\d\d) \d\d:\d\d:\d\d\ \S+ newsyslog\[\d+\]: logfile turned over$/ ) {
